@@ -14,7 +14,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Basic authentication backend"""
+"""Basic authentication backend."""
 from __future__ import annotations
 
 from functools import wraps
@@ -24,21 +24,21 @@ from flask import Response, request
 from flask_appbuilder.const import AUTH_LDAP
 from flask_login import login_user
 
+from airflow.auth.managers.fab.models import User
 from airflow.utils.airflow_flask_app import get_airflow_app
-from airflow.www.fab_security.sqla.models import User
 
 CLIENT_AUTH: tuple[str, str] | Any | None = None
 
 
 def init_app(_):
-    """Initializes authentication backend"""
+    """Initialize authentication backend."""
 
 
 T = TypeVar("T", bound=Callable)
 
 
 def auth_current_user() -> User | None:
-    """Authenticate and set current user if Authorization header exists"""
+    """Authenticate and set current user if Authorization header exists."""
     auth = request.authorization
     if auth is None or not auth.username or not auth.password:
         return None
@@ -55,7 +55,7 @@ def auth_current_user() -> User | None:
 
 
 def requires_authentication(function: T):
-    """Decorator for functions that require authentication"""
+    """Decorate functions that require authentication."""
 
     @wraps(function)
     def decorated(*args, **kwargs):
