@@ -16,35 +16,12 @@
 # under the License.
 from __future__ import annotations
 
+from airflow.api.common.airflow_health import get_airflow_health
 from airflow.api_connexion.schemas.health_schema import health_schema
 from airflow.api_connexion.types import APIResponse
-from airflow.jobs.scheduler_job import SchedulerJob
-
-HEALTHY = "healthy"
-UNHEALTHY = "unhealthy"
 
 
 def get_health() -> APIResponse:
-    """Return the health of the airflow scheduler and metadatabase."""
-    metadatabase_status = HEALTHY
-    latest_scheduler_heartbeat = None
-    scheduler_status = UNHEALTHY
-    try:
-        scheduler_job = SchedulerJob.most_recent_job()
-
-        if scheduler_job:
-            latest_scheduler_heartbeat = scheduler_job.latest_heartbeat.isoformat()
-            if scheduler_job.is_alive():
-                scheduler_status = HEALTHY
-    except Exception:
-        metadatabase_status = UNHEALTHY
-
-    payload = {
-        "metadatabase": {"status": metadatabase_status},
-        "scheduler": {
-            "status": scheduler_status,
-            "latest_scheduler_heartbeat": latest_scheduler_heartbeat,
-        },
-    }
-
-    return health_schema.dump(payload)
+    """Return the health of the airflow scheduler, metadatabase and triggerer."""
+    airflow_health_status = get_airflow_health()
+    return health_schema.dump(airflow_health_status)
