@@ -16,7 +16,7 @@
 # under the License.
 from __future__ import annotations
 
-from sqlalchemy import func
+from sqlalchemy import func, select
 from sqlalchemy.orm import Session
 
 from airflow.api_connexion import security
@@ -36,8 +36,8 @@ from airflow.utils.session import NEW_SESSION, provide_session
 @security.requires_access([(permissions.ACTION_CAN_READ, permissions.RESOURCE_IMPORT_ERROR)])
 @provide_session
 def get_import_error(*, import_error_id: int, session: Session = NEW_SESSION) -> APIResponse:
-    """Get an import error"""
-    error = session.query(ImportErrorModel).get(import_error_id)
+    """Get an import error."""
+    error = session.get(ImportErrorModel, import_error_id)
 
     if error is None:
         raise NotFound(
@@ -48,7 +48,7 @@ def get_import_error(*, import_error_id: int, session: Session = NEW_SESSION) ->
 
 
 @security.requires_access([(permissions.ACTION_CAN_READ, permissions.RESOURCE_IMPORT_ERROR)])
-@format_parameters({'limit': check_limit})
+@format_parameters({"limit": check_limit})
 @provide_session
 def get_import_errors(
     *,
@@ -57,13 +57,13 @@ def get_import_errors(
     order_by: str = "import_error_id",
     session: Session = NEW_SESSION,
 ) -> APIResponse:
-    """Get all import errors"""
-    to_replace = {"import_error_id": 'id'}
-    allowed_filter_attrs = ['import_error_id', "timestamp", "filename"]
-    total_entries = session.query(func.count(ImportErrorModel.id)).scalar()
-    query = session.query(ImportErrorModel)
+    """Get all import errors."""
+    to_replace = {"import_error_id": "id"}
+    allowed_filter_attrs = ["import_error_id", "timestamp", "filename"]
+    total_entries = session.scalars(func.count(ImportErrorModel.id)).one()
+    query = select(ImportErrorModel)
     query = apply_sorting(query, order_by, to_replace, allowed_filter_attrs)
-    import_errors = query.offset(offset).limit(limit).all()
+    import_errors = session.scalars(query.offset(offset).limit(limit)).all()
     return import_error_collection_schema.dump(
         ImportErrorCollection(import_errors=import_errors, total_entries=total_entries)
     )
