@@ -20,11 +20,12 @@ from http import HTTPStatus
 
 from flask import Response
 from marshmallow import ValidationError
-from sqlalchemy import func
+from sqlalchemy import func, select
 from sqlalchemy.orm import Session
 
 from airflow.api_connexion import security
 from airflow.api_connexion.endpoints.request_dict import get_json_request_dict
+from airflow.api_connexion.endpoints.update_mask import extract_update_mask_data
 from airflow.api_connexion.exceptions import BadRequest, NotFound
 from airflow.api_connexion.parameters import apply_sorting, check_limit, format_parameters
 from airflow.api_connexion.schemas.variable_schema import variable_collection_schema, variable_schema
@@ -56,10 +57,10 @@ def delete_variable(*, variable_key: str) -> Response:
 @provide_session
 def get_variable(*, variable_key: str, session: Session = NEW_SESSION) -> Response:
     """Get a variable by key."""
-    var = session.query(Variable).filter(Variable.key == variable_key)
-    if not var.count():
-        raise NotFound("Variable not found")
-    return variable_schema.dump(var.first())
+    var = session.scalar(select(Variable).where(Variable.key == variable_key).limit(1))
+    if not var:
+        raise NotFound("Variable not found", detail="Variable does not exist")
+    return variable_schema.dump(var)
 
 
 @security.requires_access([(permissions.ACTION_CAN_READ, permissions.RESOURCE_VARIABLE)])
@@ -73,12 +74,12 @@ def get_variables(
     session: Session = NEW_SESSION,
 ) -> Response:
     """Get all variable values."""
-    total_entries = session.query(func.count(Variable.id)).scalar()
+    total_entries = session.execute(select(func.count(Variable.id))).scalar()
     to_replace = {"value": "val"}
     allowed_filter_attrs = ["value", "key", "id"]
-    query = session.query(Variable)
+    query = select(Variable)
     query = apply_sorting(query, order_by, to_replace, allowed_filter_attrs)
-    variables = query.offset(offset).limit(limit).all()
+    variables = session.scalars(query.offset(offset).limit(limit)).all()
     return variable_collection_schema.dump(
         {
             "variables": variables,
@@ -88,13 +89,19 @@ def get_variables(
 
 
 @security.requires_access([(permissions.ACTION_CAN_EDIT, permissions.RESOURCE_VARIABLE)])
+@provide_session
 @action_logging(
     event=action_event_from_permission(
         prefix=RESOURCE_EVENT_PREFIX,
         permission=permissions.ACTION_CAN_EDIT,
     ),
 )
-def patch_variable(*, variable_key: str, update_mask: UpdateMask = None) -> Response:
+def patch_variable(
+    *,
+    variable_key: str,
+    update_mask: UpdateMask = None,
+    session: Session = NEW_SESSION,
+) -> Response:
     """Update a variable by key."""
     try:
         data = variable_schema.load(get_json_request_dict())
@@ -103,15 +110,16 @@ def patch_variable(*, variable_key: str, update_mask: UpdateMask = None) -> Resp
 
     if data["key"] != variable_key:
         raise BadRequest("Invalid post body", detail="key from request body doesn't match uri parameter")
-
+    non_update_fields = ["key"]
+    variable = session.scalar(select(Variable).filter_by(key=variable_key).limit(1))
+    if not variable:
+        raise NotFound("Variable not found", detail="Variable does not exist")
     if update_mask:
-        if "key" in update_mask:
-            raise BadRequest("key is a ready only field")
-        if "value" not in update_mask:
-            raise BadRequest("No field to update")
-
-    Variable.set(data["key"], data["val"])
-    return variable_schema.dump(data)
+        data = extract_update_mask_data(update_mask, non_update_fields, data)
+    for key, val in data.items():
+        setattr(variable, key, val)
+    session.add(variable)
+    return variable_schema.dump(variable)
 
 
 @security.requires_access([(permissions.ACTION_CAN_CREATE, permissions.RESOURCE_VARIABLE)])
