@@ -26,10 +26,12 @@ from pathlib import Path
 from typing import Any
 from urllib.parse import urlsplit, urlunsplit
 
+from sqlalchemy import select
 from sqlalchemy.orm import exc
 
 from airflow.cli.simple_table import AirflowConsole
 from airflow.compat.functools import cache
+from airflow.configuration import conf
 from airflow.exceptions import AirflowNotFoundException
 from airflow.hooks.base import BaseHook
 from airflow.models import Connection
@@ -37,6 +39,8 @@ from airflow.providers_manager import ProvidersManager
 from airflow.secrets.local_filesystem import load_connections_dict
 from airflow.utils import cli as cli_utils, helpers, yaml
 from airflow.utils.cli import suppress_logs_and_warning
+from airflow.utils.db import create_default_connections as db_create_default_connections
+from airflow.utils.providers_configuration_loader import providers_configuration_loaded
 from airflow.utils.session import create_session
 
 
@@ -59,6 +63,7 @@ def _connection_mapper(conn: Connection) -> dict[str, Any]:
 
 
 @suppress_logs_and_warning
+@providers_configuration_loaded
 def connections_get(args):
     """Get a connection."""
     try:
@@ -73,12 +78,14 @@ def connections_get(args):
 
 
 @suppress_logs_and_warning
+@providers_configuration_loaded
 def connections_list(args):
-    """Lists all connections at the command line."""
+    """List all connections at the command line."""
     with create_session() as session:
-        query = session.query(Connection)
+        query = select(Connection)
         if args.conn_id:
-            query = query.filter(Connection.conn_id == args.conn_id)
+            query = query.where(Connection.conn_id == args.conn_id)
+        query = session.scalars(query)
         conns = query.all()
 
         AirflowConsole().print_as(
@@ -101,6 +108,10 @@ def _connection_to_dict(conn: Connection) -> dict:
     )
 
 
+def create_default_connections(args):
+    db_create_default_connections()
+
+
 def _format_connections(conns: list[Connection], file_format: str, serialization_format: str) -> str:
     if serialization_format == "json":
         serializer_func = lambda x: json.dumps(_connection_to_dict(x))
@@ -138,7 +149,7 @@ def _valid_uri(uri: str) -> bool:
 
 @cache
 def _get_connection_types() -> list[str]:
-    """Returns connection types available."""
+    """Return connection types available."""
     _connection_types = ["fs", "mesos_framework-id", "email", "generic"]
     providers_manager = ProvidersManager()
     for connection_type, provider_info in providers_manager.hooks.items():
@@ -147,8 +158,9 @@ def _get_connection_types() -> list[str]:
     return _connection_types
 
 
+@providers_configuration_loaded
 def connections_export(args):
-    """Exports all connections to a file."""
+    """Export all connections to a file."""
     file_formats = [".yaml", ".json", ".env"]
     if args.format:
         warnings.warn("Option `--format` is deprecated.  Use `--file-format` instead.", DeprecationWarning)
@@ -176,7 +188,7 @@ def connections_export(args):
         raise SystemExit("Option `--serialization-format` may only be used with file type `env`.")
 
     with create_session() as session:
-        connections = session.query(Connection).order_by(Connection.conn_id).all()
+        connections = session.scalars(select(Connection).order_by(Connection.conn_id)).all()
 
     msg = _format_connections(
         conns=connections,
@@ -197,8 +209,9 @@ alternative_conn_specs = ["conn_type", "conn_host", "conn_login", "conn_password
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def connections_add(args):
-    """Adds new connection."""
+    """Add new connection."""
     has_uri = bool(args.conn_uri)
     has_json = bool(args.conn_json)
     has_type = bool(args.conn_type)
@@ -264,7 +277,7 @@ def connections_add(args):
             new_conn.set_extra(args.conn_extra)
 
     with create_session() as session:
-        if not session.query(Connection).filter(Connection.conn_id == new_conn.conn_id).first():
+        if not session.scalar(select(Connection).where(Connection.conn_id == new_conn.conn_id).limit(1)):
             session.add(new_conn)
             msg = "Successfully added `conn_id`={conn_id} : {uri}"
             msg = msg.format(
@@ -288,11 +301,12 @@ def connections_add(args):
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def connections_delete(args):
-    """Deletes connection from DB."""
+    """Delete connection from DB."""
     with create_session() as session:
         try:
-            to_delete = session.query(Connection).filter(Connection.conn_id == args.conn_id).one()
+            to_delete = session.scalars(select(Connection).where(Connection.conn_id == args.conn_id)).one()
         except exc.NoResultFound:
             raise SystemExit(f"Did not find a connection with `conn_id`={args.conn_id}")
         except exc.MultipleResultsFound:
@@ -303,8 +317,9 @@ def connections_delete(args):
 
 
 @cli_utils.action_cli(check_db=False)
+@providers_configuration_loaded
 def connections_import(args):
-    """Imports connections from a file."""
+    """Import connections from a file."""
     if os.path.exists(args.file):
         _import_helper(args.file, args.overwrite)
     else:
@@ -325,7 +340,7 @@ def _import_helper(file_path: str, overwrite: bool) -> None:
                 print(f"Could not import connection. {e}")
                 continue
 
-            existing_conn_id = session.query(Connection.id).filter(Connection.conn_id == conn_id).scalar()
+            existing_conn_id = session.scalar(select(Connection.id).where(Connection.conn_id == conn_id))
             if existing_conn_id is not None:
                 if not overwrite:
                     print(f"Could not import connection {conn_id}: connection already exists.")
@@ -340,9 +355,16 @@ def _import_helper(file_path: str, overwrite: bool) -> None:
 
 
 @suppress_logs_and_warning
+@providers_configuration_loaded
 def connections_test(args) -> None:
     """Test an Airflow connection."""
     console = AirflowConsole()
+    if conf.get("core", "test_connection", fallback="Disabled").lower().strip() != "enabled":
+        console.print(
+            "[bold yellow]\nTesting connections is disabled in Airflow configuration. "
+            "Contact your deployment admin to enable it.\n"
+        )
+        raise SystemExit(1)
 
     print(f"Retrieving connection: {args.conn_id!r}")
     try:
