@@ -28,6 +28,7 @@ import sys
 import warnings
 
 from graphviz.dot import Dot
+from sqlalchemy import delete, select
 from sqlalchemy.orm import Session
 
 from airflow import settings
@@ -44,6 +45,7 @@ from airflow.timetables.base import DataInterval
 from airflow.utils import cli as cli_utils, timezone
 from airflow.utils.cli import get_dag, get_dags, process_subdir, sigint_handler, suppress_logs_and_warning
 from airflow.utils.dot_renderer import render_dag, render_dag_dependencies
+from airflow.utils.providers_configuration_loader import providers_configuration_loaded
 from airflow.utils.session import NEW_SESSION, create_session, provide_session
 from airflow.utils.state import DagRunState
 
@@ -71,12 +73,19 @@ def _run_dag_backfill(dags: list[DAG], args) -> None:
 
         if args.dry_run:
             print(f"Dry run of DAG {dag.dag_id} on {args.start_date}")
-            dr = DagRun(dag.dag_id, execution_date=args.start_date)
-            for task in dag.tasks:
-                print(f"Task {task.task_id} located in DAG {dag.dag_id}")
-                ti = TaskInstance(task, run_id=None)
-                ti.dag_run = dr
-                ti.dry_run()
+            dagrun_infos = dag.iter_dagrun_infos_between(earliest=args.start_date, latest=args.end_date)
+            for dagrun_info in dagrun_infos:
+                dr = DagRun(
+                    dag.dag_id,
+                    execution_date=dagrun_info.logical_date,
+                    data_interval=dagrun_info.data_interval,
+                )
+
+                for task in dag.tasks:
+                    print(f"Task {task.task_id} located in DAG {dag.dag_id}")
+                    ti = TaskInstance(task, run_id=None)
+                    ti.dag_run = dr
+                    ti.dry_run()
         else:
             if args.reset_dagruns:
                 DAG.clear_dags(
@@ -112,8 +121,9 @@ def _run_dag_backfill(dags: list[DAG], args) -> None:
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def dag_backfill(args, dag: list[DAG] | DAG | None = None) -> None:
-    """Creates backfill job or dry run for a DAG or list of DAGs using regex."""
+    """Create backfill job or dry run for a DAG or list of DAGs using regex."""
     logging.basicConfig(level=settings.LOGGING_LEVEL, format=settings.SIMPLE_LOG_FORMAT)
     signal.signal(signal.SIGTERM, sigint_handler)
     warnings.warn(
@@ -142,8 +152,9 @@ def dag_backfill(args, dag: list[DAG] | DAG | None = None) -> None:
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def dag_trigger(args) -> None:
-    """Creates a dag run for the specified dag."""
+    """Create a dag run for the specified dag."""
     api_client = get_current_api_client()
     try:
         message = api_client.trigger_dag(
@@ -162,8 +173,9 @@ def dag_trigger(args) -> None:
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def dag_delete(args) -> None:
-    """Deletes all DB records related to the specified dag."""
+    """Delete all DB records related to the specified dag."""
     api_client = get_current_api_client()
     if (
         args.yes
@@ -180,19 +192,22 @@ def dag_delete(args) -> None:
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def dag_pause(args) -> None:
     """Pauses a DAG."""
     set_is_paused(True, args)
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def dag_unpause(args) -> None:
     """Unpauses a DAG."""
     set_is_paused(False, args)
 
 
+@providers_configuration_loaded
 def set_is_paused(is_paused: bool, args) -> None:
-    """Sets is_paused for DAG by a given dag_id."""
+    """Set is_paused for DAG by a given dag_id."""
     dag = DagModel.get_dagmodel(args.dag_id)
 
     if not dag:
@@ -203,8 +218,9 @@ def set_is_paused(is_paused: bool, args) -> None:
     print(f"Dag: {args.dag_id}, paused: {is_paused}")
 
 
+@providers_configuration_loaded
 def dag_dependencies_show(args) -> None:
-    """Displays DAG dependencies, save to file or show as imgcat image."""
+    """Display DAG dependencies, save to file or show as imgcat image."""
     dot = render_dag_dependencies(SerializedDagModel.get_dag_dependencies())
     filename = args.save
     imgcat = args.imgcat
@@ -222,8 +238,9 @@ def dag_dependencies_show(args) -> None:
         print(dot.source)
 
 
+@providers_configuration_loaded
 def dag_show(args) -> None:
-    """Displays DAG or saves it's graphic representation to the file."""
+    """Display DAG or saves its graphic representation to the file."""
     dag = get_dag(args.subdir, args.dag_id)
     dot = render_dag(dag)
     filename = args.save
@@ -265,10 +282,12 @@ def _save_dot_to_file(dot: Dot, filename: str) -> None:
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 @provide_session
 def dag_state(args, session: Session = NEW_SESSION) -> None:
     """
-    Returns the state (and conf if exists) of a DagRun at the command line.
+    Return the state (and conf if exists) of a DagRun at the command line.
+
     >>> airflow dags state tutorial 2015-01-01T00:00:00.000000
     running
     >>> airflow dags state a_dag_with_conf_passed 2015-01-01T00:00:00.000000
@@ -278,7 +297,7 @@ def dag_state(args, session: Session = NEW_SESSION) -> None:
 
     if not dag:
         raise SystemExit(f"DAG: {args.dag_id} does not exist in 'dag' table")
-    dr = session.query(DagRun).filter_by(dag_id=args.dag_id, execution_date=args.execution_date).one_or_none()
+    dr = session.scalar(select(DagRun).filter_by(dag_id=args.dag_id, execution_date=args.execution_date))
     out = dr.state if dr else None
     conf_out = ""
     if out and dr.conf:
@@ -287,9 +306,11 @@ def dag_state(args, session: Session = NEW_SESSION) -> None:
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def dag_next_execution(args) -> None:
     """
-    Returns the next execution datetime of a DAG at the command line.
+    Return the next execution datetime of a DAG at the command line.
+
     >>> airflow dags next-execution tutorial
     2018-08-31 10:38:00
     """
@@ -299,7 +320,9 @@ def dag_next_execution(args) -> None:
         print("[INFO] Please be reminded this DAG is PAUSED now.", file=sys.stderr)
 
     with create_session() as session:
-        last_parsed_dag: DagModel = session.query(DagModel).filter(DagModel.dag_id == dag.dag_id).one()
+        last_parsed_dag: DagModel = session.scalars(
+            select(DagModel).where(DagModel.dag_id == dag.dag_id)
+        ).one()
 
     def print_execution_interval(interval: DataInterval | None):
         if interval is None:
@@ -323,8 +346,9 @@ def dag_next_execution(args) -> None:
 
 @cli_utils.action_cli
 @suppress_logs_and_warning
+@providers_configuration_loaded
 def dag_list_dags(args) -> None:
-    """Displays dags with or without stats at the command line."""
+    """Display dags with or without stats at the command line."""
     dagbag = DagBag(process_subdir(args.subdir))
     if dagbag.import_errors:
         from rich import print as rich_print
@@ -348,9 +372,10 @@ def dag_list_dags(args) -> None:
 
 @cli_utils.action_cli
 @suppress_logs_and_warning
+@providers_configuration_loaded
 @provide_session
 def dag_details(args, session=NEW_SESSION):
-    """Get DAG details given a DAG id"""
+    """Get DAG details given a DAG id."""
     dag = DagModel.get_dagmodel(args.dag_id, session=session)
     if not dag:
         raise SystemExit(f"DAG: {args.dag_id} does not exist in 'dag' table")
@@ -369,8 +394,9 @@ def dag_details(args, session=NEW_SESSION):
 
 @cli_utils.action_cli
 @suppress_logs_and_warning
+@providers_configuration_loaded
 def dag_list_import_errors(args) -> None:
-    """Displays dags with import errors on the command line."""
+    """Display dags with import errors on the command line."""
     dagbag = DagBag(process_subdir(args.subdir))
     data = []
     for filename, errors in dagbag.import_errors.items():
@@ -383,8 +409,9 @@ def dag_list_import_errors(args) -> None:
 
 @cli_utils.action_cli
 @suppress_logs_and_warning
+@providers_configuration_loaded
 def dag_report(args) -> None:
-    """Displays dagbag stats at the command line."""
+    """Display dagbag stats at the command line."""
     dagbag = DagBag(process_subdir(args.subdir))
     AirflowConsole().print_as(
         data=dagbag.dagbag_stats,
@@ -401,9 +428,10 @@ def dag_report(args) -> None:
 
 @cli_utils.action_cli
 @suppress_logs_and_warning
+@providers_configuration_loaded
 @provide_session
 def dag_list_jobs(args, dag: DAG | None = None, session: Session = NEW_SESSION) -> None:
-    """Lists latest n jobs."""
+    """List latest n jobs."""
     queries = []
     if dag:
         args.dag_id = dag.dag_id
@@ -418,8 +446,10 @@ def dag_list_jobs(args, dag: DAG | None = None, session: Session = NEW_SESSION)
         queries.append(Job.state == args.state)
 
     fields = ["dag_id", "state", "job_type", "start_date", "end_date"]
-    all_jobs = session.query(Job).filter(*queries).order_by(Job.start_date.desc()).limit(args.limit).all()
-    all_jobs = [{f: str(job.__getattribute__(f)) for f in fields} for job in all_jobs]
+    all_jobs_iter = session.scalars(
+        select(Job).where(*queries).order_by(Job.start_date.desc()).limit(args.limit)
+    )
+    all_jobs = [{f: str(job.__getattribute__(f)) for f in fields} for job in all_jobs_iter]
 
     AirflowConsole().print_as(
         data=all_jobs,
@@ -429,9 +459,10 @@ def dag_list_jobs(args, dag: DAG | None = None, session: Session = NEW_SESSION)
 
 @cli_utils.action_cli
 @suppress_logs_and_warning
+@providers_configuration_loaded
 @provide_session
 def dag_list_dag_runs(args, dag: DAG | None = None, session: Session = NEW_SESSION) -> None:
-    """Lists dag runs for a given DAG."""
+    """List dag runs for a given DAG."""
     if dag:
         args.dag_id = dag.dag_id
     else:
@@ -465,8 +496,9 @@ def dag_list_dag_runs(args, dag: DAG | None = None, session: Session = NEW_SESSI
     )
 
 
-@provide_session
 @cli_utils.action_cli
+@providers_configuration_loaded
+@provide_session
 def dag_test(args, dag: DAG | None = None, session: Session = NEW_SESSION) -> None:
     """Execute one single DagRun for a given DAG and execution date."""
     run_conf = None
@@ -482,14 +514,12 @@ def dag_test(args, dag: DAG | None = None, session: Session = NEW_SESSION) -> No
     imgcat = args.imgcat_dagrun
     filename = args.save_dagrun
     if show_dagrun or imgcat or filename:
-        tis = (
-            session.query(TaskInstance)
-            .filter(
+        tis = session.scalars(
+            select(TaskInstance).where(
                 TaskInstance.dag_id == args.dag_id,
                 TaskInstance.execution_date == execution_date,
             )
-            .all()
-        )
+        ).all()
 
         dot_graph = render_dag(dag, tis=tis)
         print()
@@ -501,11 +531,12 @@ def dag_test(args, dag: DAG | None = None, session: Session = NEW_SESSION) -> No
             print(dot_graph.source)
 
 
-@provide_session
 @cli_utils.action_cli
+@providers_configuration_loaded
+@provide_session
 def dag_reserialize(args, session: Session = NEW_SESSION) -> None:
     """Serialize a DAG instance."""
-    session.query(SerializedDagModel).delete(synchronize_session=False)
+    session.execute(delete(SerializedDagModel).execution_options(synchronize_session=False))
 
     if not args.clear_only:
         dagbag = DagBag(process_subdir(args.subdir))
