@@ -14,40 +14,83 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Database sub-commands"""
+"""Database sub-commands."""
+from __future__ import annotations
+
+import logging
 import os
 import textwrap
+import warnings
 from tempfile import NamedTemporaryFile
 
 from packaging.version import parse as parse_version
+from tenacity import RetryCallState, Retrying, stop_after_attempt, wait_fixed
 
 from airflow import settings
 from airflow.exceptions import AirflowException
 from airflow.utils import cli as cli_utils, db
-from airflow.utils.db import REVISION_HEADS_MAP
-from airflow.utils.db_cleanup import config_dict, run_cleanup
+from airflow.utils.db import _REVISION_HEADS_MAP
+from airflow.utils.db_cleanup import config_dict, drop_archived_tables, export_archived_records, run_cleanup
 from airflow.utils.process_utils import execute_interactive
+from airflow.utils.providers_configuration_loader import providers_configuration_loaded
+
+log = logging.getLogger(__name__)
 
 
+@providers_configuration_loaded
 def initdb(args):
-    """Initializes the metadata database"""
-    print("DB: " + repr(settings.engine.url))
+    """Initialize the metadata database."""
+    warnings.warn(
+        "`db init` is deprecated.  Use `db migrate` instead to migrate the db and/or "
+        "airflow connections create-default-connections to create the default connections",
+        DeprecationWarning,
+    )
+    print(f"DB: {settings.engine.url!r}")
     db.initdb()
     print("Initialization done")
 
 
+@providers_configuration_loaded
 def resetdb(args):
-    """Resets the metadata database"""
-    print("DB: " + repr(settings.engine.url))
+    """Reset the metadata database."""
+    print(f"DB: {settings.engine.url!r}")
     if not (args.yes or input("This will drop existing tables if they exist. Proceed? (y/n)").upper() == "Y"):
         raise SystemExit("Cancelled")
     db.resetdb(skip_init=args.skip_init)
 
 
-@cli_utils.action_cli(check_db=False)
 def upgradedb(args):
-    """Upgrades the metadata database"""
-    print("DB: " + repr(settings.engine.url))
+    """Upgrades the metadata database."""
+    warnings.warn("`db upgrade` is deprecated. Use `db migrate` instead.", DeprecationWarning)
+    migratedb(args)
+
+
+def get_version_revision(version: str, recursion_limit=10) -> str | None:
+    """
+    Recursively search for the revision of the given version.
+
+    This searches REVISION_HEADS_MAP for the revision of the given version, recursively
+    searching for the previous version if the given version is not found.
+    """
+    if version in _REVISION_HEADS_MAP:
+        return _REVISION_HEADS_MAP[version]
+    try:
+        major, minor, patch = map(int, version.split("."))
+    except ValueError:
+        return None
+    new_version = f"{major}.{minor}.{patch - 1}"
+    recursion_limit -= 1
+    if recursion_limit <= 0:
+        # Prevent infinite recursion as I can't imagine 10 successive versions without migration
+        return None
+    return get_version_revision(new_version, recursion_limit)
+
+
+@cli_utils.action_cli(check_db=False)
+@providers_configuration_loaded
+def migratedb(args):
+    """Migrates the metadata database."""
+    print(f"DB: {settings.engine.url!r}")
     if args.to_revision and args.to_version:
         raise SystemExit("Cannot supply both `--to-revision` and `--to-version`.")
     if args.from_version and args.from_revision:
@@ -61,32 +104,38 @@ def upgradedb(args):
     if args.from_revision:
         from_revision = args.from_revision
     elif args.from_version:
-        if parse_version(args.from_version) < parse_version('2.0.0'):
+        if parse_version(args.from_version) < parse_version("2.0.0"):
             raise SystemExit("--from-version must be greater or equal to than 2.0.0")
-        from_revision = REVISION_HEADS_MAP.get(args.from_version)
+        from_revision = get_version_revision(args.from_version)
         if not from_revision:
             raise SystemExit(f"Unknown version {args.from_version!r} supplied as `--from-version`.")
 
     if args.to_version:
-        to_revision = REVISION_HEADS_MAP.get(args.to_version)
+        to_revision = get_version_revision(args.to_version)
         if not to_revision:
             raise SystemExit(f"Upgrading to version {args.to_version} is not supported.")
     elif args.to_revision:
         to_revision = args.to_revision
 
     if not args.show_sql_only:
-        print("Performing upgrade with database " + repr(settings.engine.url))
+        print(f"Performing upgrade to the metadata database {settings.engine.url!r}")
     else:
         print("Generating sql for upgrade -- upgrade commands will *not* be submitted.")
 
-    db.upgradedb(to_revision=to_revision, from_revision=from_revision, show_sql_only=args.show_sql_only)
+    db.upgradedb(
+        to_revision=to_revision,
+        from_revision=from_revision,
+        show_sql_only=args.show_sql_only,
+        reserialize_dags=args.reserialize_dags,
+    )
     if not args.show_sql_only:
-        print("Upgrades done")
+        print("Database migrating done!")
 
 
 @cli_utils.action_cli(check_db=False)
+@providers_configuration_loaded
 def downgrade(args):
-    """Downgrades the metadata database"""
+    """Downgrades the metadata database."""
     if args.to_revision and args.to_version:
         raise SystemExit("Cannot supply both `--to-revision` and `--to-version`.")
     if args.from_version and args.from_revision:
@@ -101,17 +150,17 @@ def downgrade(args):
     if args.from_revision:
         from_revision = args.from_revision
     elif args.from_version:
-        from_revision = REVISION_HEADS_MAP.get(args.from_version)
+        from_revision = get_version_revision(args.from_version)
         if not from_revision:
             raise SystemExit(f"Unknown version {args.from_version!r} supplied as `--from-version`.")
     if args.to_version:
-        to_revision = REVISION_HEADS_MAP.get(args.to_version)
+        to_revision = get_version_revision(args.to_version)
         if not to_revision:
             raise SystemExit(f"Downgrading to version {args.to_version} is not supported.")
     elif args.to_revision:
         to_revision = args.to_revision
     if not args.show_sql_only:
-        print("Performing downgrade with database " + repr(settings.engine.url))
+        print(f"Performing downgrade with database {settings.engine.url!r}")
     else:
         print("Generating sql for downgrade -- downgrade commands will *not* be submitted.")
 
@@ -131,18 +180,20 @@ def downgrade(args):
         raise SystemExit("Cancelled")
 
 
+@providers_configuration_loaded
 def check_migrations(args):
-    """Function to wait for all airflow migrations to complete. Used for launching airflow in k8s"""
+    """Wait for all airflow migrations to complete. Used for launching airflow in k8s."""
     db.check_migrations(timeout=args.migration_wait_timeout)
 
 
 @cli_utils.action_cli(check_db=False)
+@providers_configuration_loaded
 def shell(args):
-    """Run a shell that allows to access metadata database"""
+    """Run a shell that allows to access metadata database."""
     url = settings.engine.url
-    print("DB: " + repr(url))
+    print(f"DB: {url!r}")
 
-    if url.get_backend_name() == 'mysql':
+    if url.get_backend_name() == "mysql":
         with NamedTemporaryFile(suffix="my.cnf") as f:
             content = textwrap.dedent(
                 f"""
@@ -157,32 +208,47 @@ def shell(args):
             f.write(content.encode())
             f.flush()
             execute_interactive(["mysql", f"--defaults-extra-file={f.name}"])
-    elif url.get_backend_name() == 'sqlite':
+    elif url.get_backend_name() == "sqlite":
         execute_interactive(["sqlite3", url.database])
-    elif url.get_backend_name() == 'postgresql':
+    elif url.get_backend_name() == "postgresql":
         env = os.environ.copy()
-        env['PGHOST'] = url.host or ""
-        env['PGPORT'] = str(url.port or "5432")
-        env['PGUSER'] = url.username or ""
+        env["PGHOST"] = url.host or ""
+        env["PGPORT"] = str(url.port or "5432")
+        env["PGUSER"] = url.username or ""
         # PostgreSQL does not allow the use of PGPASSFILE if the current user is root.
         env["PGPASSWORD"] = url.password or ""
-        env['PGDATABASE'] = url.database
+        env["PGDATABASE"] = url.database
         execute_interactive(["psql"], env=env)
-    elif url.get_backend_name() == 'mssql':
+    elif url.get_backend_name() == "mssql":
         env = os.environ.copy()
-        env['MSSQL_CLI_SERVER'] = url.host
-        env['MSSQL_CLI_DATABASE'] = url.database
-        env['MSSQL_CLI_USER'] = url.username
-        env['MSSQL_CLI_PASSWORD'] = url.password
+        env["MSSQL_CLI_SERVER"] = url.host
+        env["MSSQL_CLI_DATABASE"] = url.database
+        env["MSSQL_CLI_USER"] = url.username
+        env["MSSQL_CLI_PASSWORD"] = url.password
         execute_interactive(["mssql-cli"], env=env)
     else:
         raise AirflowException(f"Unknown driver: {url.drivername}")
 
 
 @cli_utils.action_cli(check_db=False)
-def check(_):
-    """Runs a check command that checks if db is available."""
-    db.check()
+@providers_configuration_loaded
+def check(args):
+    """Run a check command that checks if db is available."""
+    retries: int = args.retry
+    retry_delay: int = args.retry_delay
+
+    def _warn_remaining_retries(retrystate: RetryCallState):
+        remain = retries - retrystate.attempt_number
+        log.warning("%d retries remain. Will retry in %d seconds", remain, retry_delay)
+
+    for attempt in Retrying(
+        stop=stop_after_attempt(1 + retries),
+        wait=wait_fixed(retry_delay),
+        reraise=True,
+        before_sleep=_warn_remaining_retries,
+    ):
+        with attempt:
+            db.check()
 
 
 # lazily imported by CLI parser for `help` command
@@ -190,8 +256,9 @@ all_tables = sorted(config_dict)
 
 
 @cli_utils.action_cli(check_db=False)
+@providers_configuration_loaded
 def cleanup_tables(args):
-    """Purges old records in metadata database"""
+    """Purges old records in metadata database."""
     run_cleanup(
         table_names=args.tables,
         dry_run=args.dry_run,
@@ -200,3 +267,26 @@ def cleanup_tables(args):
         confirm=not args.yes,
         skip_archive=args.skip_archive,
     )
+
+
+@cli_utils.action_cli(check_db=False)
+@providers_configuration_loaded
+def export_archived(args):
+    """Export archived records from metadata database."""
+    export_archived_records(
+        export_format=args.export_format,
+        output_path=args.output_path,
+        table_names=args.tables,
+        drop_archives=args.drop_archives,
+        needs_confirm=not args.yes,
+    )
+
+
+@cli_utils.action_cli(check_db=False)
+@providers_configuration_loaded
+def drop_archived(args):
+    """Drop archived tables from metadata database."""
+    drop_archived_tables(
+        table_names=args.tables,
+        needs_confirm=not args.yes,
+    )
