@@ -22,16 +22,17 @@ import getpass
 import json
 import os
 import random
-import re
 import string
 from typing import Any
 
+import re2
 from marshmallow import Schema, fields, validate
 from marshmallow.exceptions import ValidationError
 
 from airflow.cli.simple_table import AirflowConsole
 from airflow.utils import cli as cli_utils
 from airflow.utils.cli import suppress_logs_and_warning
+from airflow.utils.providers_configuration_loader import providers_configuration_loaded
 
 
 class UserSchema(Schema):
@@ -46,8 +47,9 @@ class UserSchema(Schema):
 
 
 @suppress_logs_and_warning
+@providers_configuration_loaded
 def users_list(args):
-    """Lists users at the command line."""
+    """List users at the command line."""
     from airflow.utils.cli_app_builder import get_application_builder
 
     with get_application_builder() as appbuilder:
@@ -60,8 +62,9 @@ def users_list(args):
 
 
 @cli_utils.action_cli(check_db=True)
+@providers_configuration_loaded
 def users_create(args):
-    """Creates new user in the DB."""
+    """Create new user in the DB."""
     from airflow.utils.cli_app_builder import get_application_builder
 
     with get_application_builder() as appbuilder:
@@ -108,8 +111,9 @@ def _find_user(args):
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def users_delete(args):
-    """Deletes user from DB."""
+    """Delete user from DB."""
     user = _find_user(args)
 
     # Clear the associated user roles first.
@@ -125,8 +129,9 @@ def users_delete(args):
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def users_manage_role(args, remove=False):
-    """Deletes or appends user roles."""
+    """Delete or appends user roles."""
     user = _find_user(args)
 
     from airflow.utils.cli_app_builder import get_application_builder
@@ -153,8 +158,9 @@ def users_manage_role(args, remove=False):
             print(f'User "{user.username}" added to role "{args.role}"')
 
 
+@providers_configuration_loaded
 def users_export(args):
-    """Exports all users to the json file."""
+    """Export all users to the json file."""
     from airflow.utils.cli_app_builder import get_application_builder
 
     with get_application_builder() as appbuilder:
@@ -164,7 +170,7 @@ def users_export(args):
         # In the User model the first and last name fields have underscores,
         # but the corresponding parameters in the CLI don't
         def remove_underscores(s):
-            return re.sub("_", "", s)
+            return re2.sub("_", "", s)
 
         users = [
             {
@@ -182,8 +188,9 @@ def users_export(args):
 
 
 @cli_utils.action_cli
+@providers_configuration_loaded
 def users_import(args):
-    """Imports users from the json file."""
+    """Import users from the json file."""
     json_file = getattr(args, "import")
     if not os.path.exists(json_file):
         raise SystemExit(f"File '{json_file}' does not exist")
