@@ -15,7 +15,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""This package is deprecated. Please use `airflow.task.task_runner`."""
+"""This package is deprecated. Please use :mod:`airflow.task.task_runner`."""
 from __future__ import annotations
 
 import warnings
