@@ -23,6 +23,7 @@ from typing import TYPE_CHECKING, Any, Iterable, Iterator, Mapping, Sequence
 
 import attr
 import pendulum
+from sqlalchemy import select, update
 from sqlalchemy.exc import OperationalError
 from sqlalchemy.orm.session import Session, make_transient
 from tabulate import tabulate
@@ -67,7 +68,7 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
 
     job_type = "BackfillJob"
 
-    STATES_COUNT_AS_RUNNING = (State.RUNNING, State.QUEUED)
+    STATES_COUNT_AS_RUNNING = (TaskInstanceState.RUNNING, TaskInstanceState.QUEUED)
 
     @attr.define
     class _DagRunTaskStatus:
@@ -169,7 +170,7 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
 
     def _update_counters(self, ti_status: _DagRunTaskStatus, session: Session) -> None:
         """
-        Updates the counters per state of the tasks that were running.
+        Update the counters per state of the tasks that were running.
 
         Can re-add to tasks to run when required.
 
@@ -183,7 +184,7 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
 
         filter_for_tis = TI.filter_for_tis(list(ti_status.running.values()))
         if filter_for_tis is not None:
-            refreshed_tis = session.query(TI).filter(filter_for_tis).all()
+            refreshed_tis = session.scalars(select(TI).where(filter_for_tis)).all()
 
         for ti in refreshed_tis:
             # Use primary key to match in memory information
@@ -218,7 +219,7 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
             # is changed externally, e.g. by clearing tasks from the ui. We need to cover
             # for that as otherwise those tasks would fall outside the scope of
             # the backfill suddenly.
-            elif ti.state == State.NONE:
+            elif ti.state is None:
                 self.log.warning(
                     "FIXME: task instance %s state was set to none externally or "
                     "reaching concurrency limits. Re-adding task to queue.",
@@ -237,8 +238,11 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
         # Batch schedule of task instances
         if tis_to_be_scheduled:
             filter_for_tis = TI.filter_for_tis(tis_to_be_scheduled)
-            session.query(TI).filter(filter_for_tis).update(
-                values={TI.state: TaskInstanceState.SCHEDULED}, synchronize_session=False
+            session.execute(
+                update(TI)
+                .where(filter_for_tis)
+                .values(state=TaskInstanceState.SCHEDULED)
+                .execution_options(synchronize_session=False)
             )
             session.flush()
 
@@ -447,7 +451,7 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
 
         is_unit_test = airflow_conf.getboolean("core", "unit_test_mode")
 
-        while (len(ti_status.to_run) > 0 or len(ti_status.running) > 0) and len(ti_status.deadlocked) == 0:
+        while (ti_status.to_run or ti_status.running) and not ti_status.deadlocked:
             self.log.debug("*** Clearing out not_ready list ***")
             ti_status.not_ready.clear()
 
@@ -536,10 +540,7 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
 
                         cfg_path = None
 
-                        executor_class, _ = ExecutorLoader.import_executor_cls(
-                            self.job.executor_class,
-                        )
-                        if executor_class.is_local:
+                        if executor.is_local:
                             cfg_path = tmp_configuration_copy()
 
                         executor.queue_task_instance(
@@ -587,59 +588,83 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
             try:
                 for task in self.dag.topological_sort(include_subdag_tasks=True):
                     for key, ti in list(ti_status.to_run.items()):
-                        if task.task_id != ti.task_id:
-                            continue
+                        # Attempt to workaround deadlock on backfill by attempting to commit the transaction
+                        # state update few times before giving up
+                        max_attempts = 5
+                        for i in range(max_attempts):
+                            if task.task_id != ti.task_id:
+                                continue
 
-                        pool = session.query(models.Pool).filter(models.Pool.pool == task.pool).first()
-                        if not pool:
-                            raise PoolNotFound(f"Unknown pool: {task.pool}")
-
-                        open_slots = pool.open_slots(session=session)
-                        if open_slots <= 0:
-                            raise NoAvailablePoolSlot(
-                                f"Not scheduling since there are {open_slots} open slots in pool {task.pool}"
+                            pool = session.scalar(
+                                select(models.Pool).where(models.Pool.pool == task.pool).limit(1)
                             )
+                            if not pool:
+                                raise PoolNotFound(f"Unknown pool: {task.pool}")
 
-                        num_running_task_instances_in_dag = DAG.get_num_task_instances(
-                            self.dag_id,
-                            states=self.STATES_COUNT_AS_RUNNING,
-                            session=session,
-                        )
+                            open_slots = pool.open_slots(session=session)
+                            if open_slots <= 0:
+                                raise NoAvailablePoolSlot(
+                                    f"Not scheduling since there are {open_slots} "
+                                    f"open slots in pool {task.pool}"
+                                )
 
-                        if num_running_task_instances_in_dag >= self.dag.max_active_tasks:
-                            raise DagConcurrencyLimitReached(
-                                "Not scheduling since DAG max_active_tasks limit is reached."
-                            )
-
-                        if task.max_active_tis_per_dag is not None:
-                            num_running_task_instances_in_task = DAG.get_num_task_instances(
-                                dag_id=self.dag_id,
-                                task_ids=[task.task_id],
+                            num_running_task_instances_in_dag = DAG.get_num_task_instances(
+                                self.dag_id,
                                 states=self.STATES_COUNT_AS_RUNNING,
                                 session=session,
                             )
 
-                            if num_running_task_instances_in_task >= task.max_active_tis_per_dag:
-                                raise TaskConcurrencyLimitReached(
-                                    "Not scheduling since Task concurrency limit is reached."
+                            if num_running_task_instances_in_dag >= self.dag.max_active_tasks:
+                                raise DagConcurrencyLimitReached(
+                                    "Not scheduling since DAG max_active_tasks limit is reached."
                                 )
 
-                        if task.max_active_tis_per_dagrun is not None:
-                            num_running_task_instances_in_task_dagrun = DAG.get_num_task_instances(
-                                dag_id=self.dag_id,
-                                run_id=ti.run_id,
-                                task_ids=[task.task_id],
-                                states=self.STATES_COUNT_AS_RUNNING,
-                                session=session,
-                            )
-
-                            if num_running_task_instances_in_task_dagrun >= task.max_active_tis_per_dagrun:
-                                raise TaskConcurrencyLimitReached(
-                                    "Not scheduling since Task concurrency per DAG run limit is reached."
+                            if task.max_active_tis_per_dag is not None:
+                                num_running_task_instances_in_task = DAG.get_num_task_instances(
+                                    dag_id=self.dag_id,
+                                    task_ids=[task.task_id],
+                                    states=self.STATES_COUNT_AS_RUNNING,
+                                    session=session,
                                 )
 
-                        _per_task_process(key, ti, session)
-                        session.commit()
+                                if num_running_task_instances_in_task >= task.max_active_tis_per_dag:
+                                    raise TaskConcurrencyLimitReached(
+                                        "Not scheduling since Task concurrency limit is reached."
+                                    )
+
+                            if task.max_active_tis_per_dagrun is not None:
+                                num_running_task_instances_in_task_dagrun = DAG.get_num_task_instances(
+                                    dag_id=self.dag_id,
+                                    run_id=ti.run_id,
+                                    task_ids=[task.task_id],
+                                    states=self.STATES_COUNT_AS_RUNNING,
+                                    session=session,
+                                )
+
+                                if (
+                                    num_running_task_instances_in_task_dagrun
+                                    >= task.max_active_tis_per_dagrun
+                                ):
+                                    raise TaskConcurrencyLimitReached(
+                                        "Not scheduling since Task concurrency per DAG run limit is reached."
+                                    )
+
+                            _per_task_process(key, ti, session)
+                            try:
+                                session.commit()
+                                # break the retry loop
+                                break
+                            except OperationalError:
+                                self.log.error(
+                                    "Failed to commit task state due to operational error. "
+                                    "The job will retry this operation so if your backfill succeeds, "
+                                    "you can safely ignore this message.",
+                                    exc_info=True,
+                                )
+                                session.rollback()
+                                if i == max_attempts - 1:
+                                    raise
+                                # retry the loop
             except (NoAvailablePoolSlot, DagConcurrencyLimitReached, TaskConcurrencyLimitReached) as e:
                 self.log.debug(e)
 
@@ -652,11 +677,7 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
             # If the set of tasks that aren't ready ever equals the set of
             # tasks to run and there are no running tasks then the backfill
             # is deadlocked
-            if (
-                ti_status.not_ready
-                and ti_status.not_ready == set(ti_status.to_run)
-                and len(ti_status.running) == 0
-            ):
+            if ti_status.not_ready and ti_status.not_ready == set(ti_status.to_run) and not ti_status.running:
                 self.log.warning("Deadlock discovered for ti_status.to_run=%s", ti_status.to_run.values())
                 ti_status.deadlocked.update(ti_status.to_run.values())
                 ti_status.to_run.clear()
@@ -693,7 +714,7 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
             _dag_runs = ti_status.active_runs[:]
             for run in _dag_runs:
                 run.update_state(session=session)
-                if run.state in State.finished:
+                if run.state in State.finished_dr_states:
                     ti_status.finished_runs += 1
                     ti_status.active_runs.remove(run)
                     executed_run_dates.append(run.execution_date)
@@ -813,8 +834,7 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
         session: Session = NEW_SESSION,
     ) -> None:
         """
-        Go through the dag_runs and update the state based on the task_instance state.
-        Then set DAG runs that are not finished to failed.
+        Update the state of each dagrun based on the task_instance state and set unfinished runs to failed.
 
         :param dag_runs: DAG runs
         :param session: session
@@ -822,7 +842,7 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
         """
         for dag_run in dag_runs:
             dag_run.update_state()
-            if dag_run.state not in State.finished:
+            if dag_run.state not in State.finished_dr_states:
                 dag_run.set_state(DagRunState.FAILED)
             session.merge(dag_run)
 
@@ -937,6 +957,13 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
             # TODO: we will need to terminate running task instances and set the
             # state to failed.
             self._set_unfinished_dag_runs_to_failed(ti_status.active_runs)
+        except OperationalError:
+            self.log.error(
+                "Backfill job dead-locked. The job will retry the job so it is likely "
+                "to heal itself. If your backfill succeeds you can ignore this exception.",
+                exc_info=True,
+            )
+            raise
         finally:
             session.commit()
             executor.end()
@@ -969,12 +996,14 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
         resettable_states = [TaskInstanceState.SCHEDULED, TaskInstanceState.QUEUED]
         if filter_by_dag_run is None:
             resettable_tis = (
-                session.query(TaskInstance)
-                .join(TaskInstance.dag_run)
-                .filter(
-                    DagRun.state == DagRunState.RUNNING,
-                    DagRun.run_type != DagRunType.BACKFILL_JOB,
-                    TaskInstance.state.in_(resettable_states),
+                session.scalars(
+                    select(TaskInstance)
+                    .join(TaskInstance.dag_run)
+                    .where(
+                        DagRun.state == DagRunState.RUNNING,
+                        DagRun.run_type != DagRunType.BACKFILL_JOB,
+                        TaskInstance.state.in_(resettable_states),
+                    )
                 )
             ).all()
         else:
@@ -989,23 +1018,22 @@ class BackfillJobRunner(BaseJobRunner[Job], LoggingMixin):
                 return result
 
             filter_for_tis = TaskInstance.filter_for_tis(items)
-            reset_tis = (
-                session.query(TaskInstance)
-                .filter(filter_for_tis, TaskInstance.state.in_(resettable_states))
+            reset_tis = session.scalars(
+                select(TaskInstance)
+                .where(filter_for_tis, TaskInstance.state.in_(resettable_states))
                 .with_for_update()
-                .all()
-            )
+            ).all()
 
             for ti in reset_tis:
-                ti.state = State.NONE
+                ti.state = None
                 session.merge(ti)
 
             return result + reset_tis
 
         reset_tis = helpers.reduce_in_chunks(query, tis_to_reset, [], self.job.max_tis_per_query)
 
-        task_instance_str = "\n\t".join(repr(x) for x in reset_tis)
+        task_instance_str = "\n".join(f"\t{x!r}" for x in reset_tis)
         session.flush()
 
-        self.log.info("Reset the following %s TaskInstances:\n\t%s", len(reset_tis), task_instance_str)
+        self.log.info("Reset the following %s TaskInstances:\n%s", len(reset_tis), task_instance_str)
         return len(reset_tis)
