@@ -28,13 +28,14 @@ import warnings
 from collections import Counter
 from dataclasses import dataclass
 from datetime import datetime, timedelta
+from functools import lru_cache, partial
 from pathlib import Path
-from typing import TYPE_CHECKING, Collection, Iterable, Iterator
+from typing import TYPE_CHECKING, Any, Callable, Collection, Iterable, Iterator
 
-from sqlalchemy import and_, func, not_, or_, text
+from sqlalchemy import and_, delete, func, not_, or_, select, text, update
+from sqlalchemy.engine import Result
 from sqlalchemy.exc import OperationalError
-from sqlalchemy.orm import load_only, selectinload
-from sqlalchemy.orm.session import Session, make_transient
+from sqlalchemy.orm import Query, Session, joinedload, load_only, make_transient, selectinload
 from sqlalchemy.sql import expression
 
 from airflow import settings
@@ -73,7 +74,7 @@ from airflow.utils.sqlalchemy import (
     tuple_in_condition,
     with_row_locks,
 )
-from airflow.utils.state import DagRunState, State, TaskInstanceState
+from airflow.utils.state import DagRunState, JobState, State, TaskInstanceState
 from airflow.utils.types import DagRunType
 
 if TYPE_CHECKING:
@@ -89,7 +90,7 @@ DM = DagModel
 @dataclass
 class ConcurrencyMap:
     """
-    Dataclass to represent concurrency maps
+    Dataclass to represent concurrency maps.
 
     It contains a map from (dag_id, task_id) to # of task instances, a map from (dag_id, task_id)
     to # of task instances in the given state list and a map from (dag_id, run_id, task_id)
@@ -239,7 +240,7 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         signal.signal(signal.SIGUSR2, self._debug_dump)
 
     def _exit_gracefully(self, signum: int, frame: FrameType | None) -> None:
-        """Helper method to clean up processor_agent to avoid leaving orphan processes."""
+        """Clean up processor_agent to avoid leaving orphan processes."""
         if not _is_parent_process():
             # Only the parent process should perform the cleanup.
             return
@@ -271,9 +272,9 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         :param states: List of states to query for
         :return: Concurrency map
         """
-        ti_concurrency_query: list[tuple[str, str, str, int]] = (
-            session.query(TI.task_id, TI.run_id, TI.dag_id, func.count("*"))
-            .filter(TI.state.in_(states))
+        ti_concurrency_query: Result = session.execute(
+            select(TI.task_id, TI.run_id, TI.dag_id, func.count("*"))
+            .where(TI.state.in_(states))
             .group_by(TI.task_id, TI.run_id, TI.dag_id)
         )
         return ConcurrencyMap.from_concurrency_map(
@@ -354,33 +355,33 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             # DagRuns which are not backfilled, in the given states,
             # and the dag is not paused
             query = (
-                session.query(TI)
+                select(TI)
                 .with_hint(TI, "USE INDEX (ti_state)", dialect_name="mysql")
                 .join(TI.dag_run)
-                .filter(DR.run_type != DagRunType.BACKFILL_JOB, DR.state == DagRunState.RUNNING)
+                .where(DR.run_type != DagRunType.BACKFILL_JOB, DR.state == DagRunState.RUNNING)
                 .join(TI.dag_model)
-                .filter(not_(DM.is_paused))
-                .filter(TI.state == TaskInstanceState.SCHEDULED)
-                .options(selectinload("dag_model"))
+                .where(not_(DM.is_paused))
+                .where(TI.state == TaskInstanceState.SCHEDULED)
+                .options(selectinload(TI.dag_model))
                 .order_by(-TI.priority_weight, DR.execution_date, TI.map_index)
             )
 
             if starved_pools:
-                query = query.filter(not_(TI.pool.in_(starved_pools)))
+                query = query.where(not_(TI.pool.in_(starved_pools)))
 
             if starved_dags:
-                query = query.filter(not_(TI.dag_id.in_(starved_dags)))
+                query = query.where(not_(TI.dag_id.in_(starved_dags)))
 
             if starved_tasks:
                 task_filter = tuple_in_condition((TI.dag_id, TI.task_id), starved_tasks)
-                query = query.filter(not_(task_filter))
+                query = query.where(not_(task_filter))
 
             if starved_tasks_task_dagrun_concurrency:
                 task_filter = tuple_in_condition(
                     (TI.dag_id, TI.run_id, TI.task_id),
                     starved_tasks_task_dagrun_concurrency,
                 )
-                query = query.filter(not_(task_filter))
+                query = query.where(not_(task_filter))
 
             query = query.limit(max_tis)
 
@@ -388,12 +389,14 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             timer.start()
 
             try:
-                task_instances_to_examine: list[TI] = with_row_locks(
+                query = with_row_locks(
                     query,
                     of=TI,
                     session=session,
                     **skip_locked(session=session),
-                ).all()
+                )
+                task_instances_to_examine: list[TI] = session.scalars(query).all()
+
                 timer.stop(send=True)
             except OperationalError as e:
                 timer.stop(send=False)
@@ -402,15 +405,13 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             # TODO[HA]: This was wrong before anyway, as it only looked at a sub-set of dags, not everything.
             # Stats.gauge('scheduler.tasks.pending', len(task_instances_to_examine))
 
-            if len(task_instances_to_examine) == 0:
+            if not task_instances_to_examine:
                 self.log.debug("No tasks to consider for execution.")
                 break
 
             # Put one task instance on each line
-            task_instance_str = "\n\t".join(repr(x) for x in task_instances_to_examine)
-            self.log.info(
-                "%s tasks up for execution:\n\t%s", len(task_instances_to_examine), task_instance_str
-            )
+            task_instance_str = "\n".join(f"\t{x!r}" for x in task_instances_to_examine)
+            self.log.info("%s tasks up for execution:\n%s", len(task_instances_to_examine), task_instance_str)
 
             for task_instance in task_instances_to_examine:
                 pool_name = task_instance.pool
@@ -501,9 +502,12 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
                             dag_id,
                             task_instance,
                         )
-                        session.query(TI).filter(
-                            TI.dag_id == dag_id, TI.state == TaskInstanceState.SCHEDULED
-                        ).update({TI.state: TaskInstanceState.FAILED}, synchronize_session="fetch")
+                        session.execute(
+                            update(TI)
+                            .where(TI.dag_id == dag_id, TI.state == TaskInstanceState.SCHEDULED)
+                            .values(state=TaskInstanceState.FAILED)
+                            .execution_options(synchronize_session="fetch")
+                        )
                         continue
 
                     task_concurrency_limit: int | None = None
@@ -578,27 +582,34 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
 
         for pool_name, num_starving_tasks in pool_num_starving_tasks.items():
             Stats.gauge(f"pool.starving_tasks.{pool_name}", num_starving_tasks)
+            # Same metric with tagging
+            Stats.gauge("pool.starving_tasks", num_starving_tasks, tags={"pool_name": pool_name})
 
         Stats.gauge("scheduler.tasks.starving", num_starving_tasks_total)
         Stats.gauge("scheduler.tasks.executable", len(executable_tis))
 
-        if len(executable_tis) > 0:
-            task_instance_str = "\n\t".join(repr(x) for x in executable_tis)
-            self.log.info("Setting the following tasks to queued state:\n\t%s", task_instance_str)
+        if executable_tis:
+            task_instance_str = "\n".join(f"\t{x!r}" for x in executable_tis)
+            self.log.info("Setting the following tasks to queued state:\n%s", task_instance_str)
 
             # set TIs to queued state
             filter_for_tis = TI.filter_for_tis(executable_tis)
-            session.query(TI).filter(filter_for_tis).update(
-                # TODO[ha]: should we use func.now()? How does that work with DB timezone
-                # on mysql when it's not UTC?
-                {
-                    TI.state: TaskInstanceState.QUEUED,
-                    TI.queued_dttm: timezone.utcnow(),
-                    TI.queued_by_job_id: self.job.id,
-                },
-                synchronize_session=False,
+            session.execute(
+                update(TI)
+                .where(filter_for_tis)
+                .values(
+                    # TODO[ha]: should we use func.now()? How does that work with DB timezone
+                    # on mysql when it's not UTC?
+                    state=TaskInstanceState.QUEUED,
+                    queued_dttm=timezone.utcnow(),
+                    queued_by_job_id=self.job.id,
+                )
+                .execution_options(synchronize_session=False)
             )
 
+            for ti in executable_tis:
+                ti.emit_state_change_metric(TaskInstanceState.QUEUED)
+
         for ti in executable_tis:
             make_transient(ti)
         return executable_tis
@@ -612,8 +623,8 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         """
         # actually enqueue them
         for ti in task_instances:
-            if ti.dag_run.state in State.finished:
-                ti.set_state(State.NONE, session=session)
+            if ti.dag_run.state in State.finished_dr_states:
+                ti.set_state(None, session=session)
                 continue
             command = ti.command_as_list(
                 local=True,
@@ -637,7 +648,7 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
 
         There are three steps:
         1. Pick TIs by priority with the constraint that they are in the expected states
-        and that we do exceed max_active_runs or pool limits.
+        and that we do not exceed max_active_runs or pool limits.
         2. Change the state for the TIs above atomically.
         3. Enqueue the TIs in the executor.
 
@@ -668,14 +679,12 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         tis_with_right_state: list[TaskInstanceKey] = []
 
         # Report execution
-        for ti_key, value in event_buffer.items():
-            state: str
-            state, _ = value
+        for ti_key, (state, _) in event_buffer.items():
             # We create map (dag_id, task_id, execution_date) -> in-memory try_number
             ti_primary_key_to_try_number_map[ti_key.primary] = ti_key.try_number
 
             self.log.info("Received executor event with state %s for task instance %s", state, ti_key)
-            if state in (State.FAILED, State.SUCCESS, State.QUEUED):
+            if state in (TaskInstanceState.FAILED, TaskInstanceState.SUCCESS, TaskInstanceState.QUEUED):
                 tis_with_right_state.append(ti_key)
 
         # Return if no finished tasks
@@ -684,7 +693,7 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
 
         # Check state of finished tasks
         filter_for_tis = TI.filter_for_tis(tis_with_right_state)
-        query = session.query(TI).filter(filter_for_tis).options(selectinload("dag_model"))
+        query = select(TI).where(filter_for_tis).options(selectinload(TI.dag_model))
         # row lock this entire set of taskinstances to make sure the scheduler doesn't fail when we have
         # multi-schedulers
         tis: Iterator[TI] = with_row_locks(
@@ -693,12 +702,13 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             session=session,
             **skip_locked(session=session),
         )
+        tis = session.scalars(tis)
         for ti in tis:
             try_number = ti_primary_key_to_try_number_map[ti.key.primary]
             buffer_key = ti.key.with_try_number(try_number)
             state, info = event_buffer.pop(buffer_key)
 
-            if state == State.QUEUED:
+            if state == TaskInstanceState.QUEUED:
                 ti.external_executor_id = info
                 self.log.info("Setting external_id for %s to %s", ti, info)
                 continue
@@ -752,11 +762,11 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             if ti_queued and not ti_requeued:
                 Stats.incr(
                     "scheduler.tasks.killed_externally",
-                    tags={"dag_id": ti.dag_id, "run_id": ti.run_id, "task_id": ti.task_id},
+                    tags={"dag_id": ti.dag_id, "task_id": ti.task_id},
                 )
                 msg = (
                     "Executor reports task instance %s finished (%s) although the "
-                    "task says its %s. (Info: %s) Was the task killed externally?"
+                    "task says it's %s. (Info: %s) Was the task killed externally?"
                 )
                 self.log.error(msg, ti, state, ti.state, info)
 
@@ -867,11 +877,11 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
     @provide_session
     def _update_dag_run_state_for_paused_dags(self, session: Session = NEW_SESSION) -> None:
         try:
-            paused_runs = (
-                session.query(DagRun)
+            paused_runs = session.scalars(
+                select(DagRun)
                 .join(DagRun.dag_model)
                 .join(TI)
-                .filter(
+                .where(
                     DagModel.is_paused == expression.true(),
                     DagRun.state == DagRunState.RUNNING,
                     DagRun.run_type != DagRunType.BACKFILL_JOB,
@@ -893,7 +903,7 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
 
     def _run_scheduler_loop(self) -> None:
         """
-        The actual scheduler loop.
+        Harvest DAG parsing results, queue tasks, and perform executor heartbeat; the actual scheduler loop.
 
         The main steps in the loop are:
             #. Harvest DAG parsing results through DagFileProcessorAgent
@@ -1009,7 +1019,7 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
 
     def _do_scheduling(self, session: Session) -> int:
         """
-        This function is where the main scheduling decisions take places.
+        Make the main scheduling decisions.
 
         It:
         - Creates any necessary DAG runs by examining the next_dagrun_create_after column of DagModel
@@ -1051,8 +1061,13 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             callback_tuples = self._schedule_all_dag_runs(guard, dag_runs, session)
 
         # Send the callbacks after we commit to ensure the context is up to date when it gets run
+        # cache saves time during scheduling of many dag_runs for same dag
+        cached_get_dag: Callable[[str], DAG | None] = lru_cache()(
+            partial(self.dagbag.get_dag, session=session)
+        )
         for dag_run, callback_to_run in callback_tuples:
-            dag = self.dagbag.get_dag(dag_run.dag_id, session=session)
+            dag = cached_get_dag(dag_run.dag_id)
+
             if not dag:
                 self.log.error("DAG '%s' not found in serialized_dag table", dag_run.dag_id)
                 continue
@@ -1095,7 +1110,7 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         return num_queued_tis
 
     @retry_db_transaction
-    def _get_next_dagruns_to_examine(self, state: DagRunState, session: Session):
+    def _get_next_dagruns_to_examine(self, state: DagRunState, session: Session) -> Query:
         """Get Next DagRuns to Examine with retries."""
         return DagRun.next_dagruns_to_examine(state, session)
 
@@ -1119,22 +1134,21 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         # END: create dagruns
 
     def _create_dag_runs(self, dag_models: Collection[DagModel], session: Session) -> None:
-        """
-        Unconditionally create a DAG run for the given DAG, and update the dag_model's fields to control
-        if/when the next DAGRun should be created
-        """
+        """Create a DAG run and update the dag_model to control if/when the next DAGRun should be created."""
         # Bulk Fetch DagRuns with dag_id and execution_date same
         # as DagModel.dag_id and DagModel.next_dagrun
         # This list is used to verify if the DagRun already exist so that we don't attempt to create
         # duplicate dag runs
         existing_dagruns = (
-            session.query(DagRun.dag_id, DagRun.execution_date)
-            .filter(
-                tuple_in_condition(
-                    (DagRun.dag_id, DagRun.execution_date),
-                    ((dm.dag_id, dm.next_dagrun) for dm in dag_models),
-                ),
+            session.execute(
+                select(DagRun.dag_id, DagRun.execution_date).where(
+                    tuple_in_condition(
+                        (DagRun.dag_id, DagRun.execution_date),
+                        ((dm.dag_id, dm.next_dagrun) for dm in dag_models),
+                    ),
+                )
             )
+            .unique()
             .all()
         )
 
@@ -1194,8 +1208,10 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             for dag_id, (_, last_time) in dataset_triggered_dag_info.items()
         }
         existing_dagruns: set[tuple[str, timezone.DateTime]] = set(
-            session.query(DagRun.dag_id, DagRun.execution_date).filter(
-                tuple_in_condition((DagRun.dag_id, DagRun.execution_date), exec_dates.items())
+            session.execute(
+                select(DagRun.dag_id, DagRun.execution_date).where(
+                    tuple_in_condition((DagRun.dag_id, DagRun.execution_date), exec_dates.items())
+                )
             )
         )
 
@@ -1224,15 +1240,15 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             # instead of falling in a loop of Integrity Error.
             exec_date = exec_dates[dag.dag_id]
             if (dag.dag_id, exec_date) not in existing_dagruns:
-                previous_dag_run = (
-                    session.query(DagRun)
-                    .filter(
+                previous_dag_run = session.scalar(
+                    select(DagRun)
+                    .where(
                         DagRun.dag_id == dag.dag_id,
                         DagRun.execution_date < exec_date,
                         DagRun.run_type == DagRunType.DATASET_TRIGGERED,
                     )
                     .order_by(DagRun.execution_date.desc())
-                    .first()
+                    .limit(1)
                 )
                 dataset_event_filters = [
                     DagScheduleDatasetReference.dag_id == dag.dag_id,
@@ -1241,16 +1257,15 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
                 if previous_dag_run:
                     dataset_event_filters.append(DatasetEvent.timestamp > previous_dag_run.execution_date)
 
-                dataset_events = (
-                    session.query(DatasetEvent)
+                dataset_events = session.scalars(
+                    select(DatasetEvent)
                     .join(
                         DagScheduleDatasetReference,
                         DatasetEvent.dataset_id == DagScheduleDatasetReference.dataset_id,
                     )
                     .join(DatasetEvent.source_dag_run)
-                    .filter(*dataset_event_filters)
-                    .all()
-                )
+                    .where(*dataset_event_filters)
+                ).all()
 
                 data_interval = dag.timetable.data_interval_for_events(exec_date, dataset_events)
                 run_id = dag.timetable.generate_run_id(
@@ -1274,9 +1289,9 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
                 )
                 Stats.incr("dataset.triggered_dagruns")
                 dag_run.consumed_dataset_events.extend(dataset_events)
-                session.query(DatasetDagRunQueue).filter(
-                    DatasetDagRunQueue.target_dag_id == dag_run.dag_id
-                ).delete()
+                session.execute(
+                    delete(DatasetDagRunQueue).where(DatasetDagRunQueue.target_dag_id == dag_run.dag_id)
+                )
 
     def _should_update_dag_next_dagruns(
         self, dag: DAG, dag_model: DagModel, total_active_runs: int | None = None, *, session: Session
@@ -1329,8 +1344,14 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
                     tags={"dag_id": dag.dag_id},
                 )
 
+        # cache saves time during scheduling of many dag_runs for same dag
+        cached_get_dag: Callable[[str], DAG | None] = lru_cache()(
+            partial(self.dagbag.get_dag, session=session)
+        )
+
         for dag_run in dag_runs:
-            dag = dag_run.dag = self.dagbag.get_dag(dag_run.dag_id, session=session)
+            dag = dag_run.dag = cached_get_dag(dag_run.dag_id)
+
             if not dag:
                 self.log.error("DAG '%s' not found in serialized_dag table", dag_run.dag_id)
                 continue
@@ -1349,15 +1370,15 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
                 dag_run.notify_dagrun_state_changed()
 
     @retry_db_transaction
-    def _schedule_all_dag_runs(self, guard, dag_runs, session):
-        """Makes scheduling decisions for all `dag_runs`"""
-        callback_tuples = []
-        for dag_run in dag_runs:
-            callback_to_run = self._schedule_dag_run(dag_run, session)
-            callback_tuples.append((dag_run, callback_to_run))
-
+    def _schedule_all_dag_runs(
+        self,
+        guard: CommitProhibitorGuard,
+        dag_runs: Iterable[DagRun],
+        session: Session,
+    ) -> list[tuple[DagRun, DagCallbackRequest | None]]:
+        """Make scheduling decisions for all `dag_runs`."""
+        callback_tuples = [(run, self._schedule_dag_run(run, session=session)) for run in dag_runs]
         guard.commit()
-
         return callback_tuples
 
     def _schedule_dag_run(
@@ -1374,11 +1395,24 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         callback: DagCallbackRequest | None = None
 
         dag = dag_run.dag = self.dagbag.get_dag(dag_run.dag_id, session=session)
-        dag_model = DM.get_dagmodel(dag_run.dag_id, session)
+        # Adopt row locking to account for inconsistencies when next_dagrun_create_after = None
+        query = (
+            session.query(DagModel)
+            .filter(DagModel.dag_id == dag_run.dag_id)
+            .options(joinedload(DagModel.parent_dag))
+        )
+        dag_model = with_row_locks(
+            query, of=DagModel, session=session, **skip_locked(session=session)
+        ).one_or_none()
 
-        if not dag or not dag_model:
+        if not dag:
             self.log.error("Couldn't find DAG %s in DAG bag or database!", dag_run.dag_id)
             return callback
+        if not dag_model:
+            self.log.info(
+                "DAG %s scheduling was skipped, probably because the DAG record was locked", dag_run.dag_id
+            )
+            return callback
 
         if (
             dag_run.start_date
@@ -1386,11 +1420,11 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             and dag_run.start_date < timezone.utcnow() - dag.dagrun_timeout
         ):
             dag_run.set_state(DagRunState.FAILED)
-            unfinished_task_instances = (
-                session.query(TI)
-                .filter(TI.dag_id == dag_run.dag_id)
-                .filter(TI.run_id == dag_run.run_id)
-                .filter(TI.state.in_(State.unfinished))
+            unfinished_task_instances = session.scalars(
+                select(TI)
+                .where(TI.dag_id == dag_run.dag_id)
+                .where(TI.run_id == dag_run.run_id)
+                .where(TI.state.in_(State.unfinished))
             )
             for task_instance in unfinished_task_instances:
                 task_instance.state = TaskInstanceState.SKIPPED
@@ -1426,7 +1460,7 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         # TODO[HA]: Rename update_state -> schedule_dag_run, ?? something else?
         schedulable_tis, callback_to_run = dag_run.update_state(session=session, execute_callbacks=False)
         # Check if DAG not scheduled then skip interval calculation to same scheduler runtime
-        if dag_run.state in State.finished:
+        if dag_run.state in State.finished_dr_states:
             # Work out if we should allow creating a new DagRun now?
             if self._should_update_dag_next_dagruns(dag, dag_model, session=session):
                 dag_model.calculate_dagrun_date_fields(dag, dag.get_run_data_interval(dag_run))
@@ -1434,7 +1468,7 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         # query to update all the TIs across all the execution dates and dag
         # IDs in a single query, but it turns out that can be _very very slow_
         # see #11147/commit ee90807ac for more details
-        dag_run.schedule_tis(schedulable_tis, session)
+        dag_run.schedule_tis(schedulable_tis, session, max_tis_per_query=self.job.max_tis_per_query)
 
         return callback_to_run
 
@@ -1468,7 +1502,7 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             self.log.debug("callback is empty")
 
     def _send_sla_callbacks_to_processor(self, dag: DAG) -> None:
-        """Sends SLA Callbacks to DagFileProcessor if tasks have SLAs set and check_slas=True."""
+        """Send SLA Callbacks to DagFileProcessor if tasks have SLAs set and check_slas=True."""
         if not settings.CHECK_SLAS:
             return
 
@@ -1505,15 +1539,13 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         """
         self.log.debug("Calling SchedulerJob._fail_tasks_stuck_in_queued method")
 
-        tasks_stuck_in_queued = (
-            session.query(TI)
-            .filter(
-                TI.state == State.QUEUED,
+        tasks_stuck_in_queued = session.scalars(
+            select(TI).where(
+                TI.state == TaskInstanceState.QUEUED,
                 TI.queued_dttm < (timezone.utcnow() - timedelta(seconds=self._task_queued_timeout)),
                 TI.queued_by_job_id == self.job.id,
             )
-            .all()
-        )
+        ).all()
         try:
             tis_for_warning_message = self.job.executor.cleanup_stuck_queued_tasks(tis=tasks_stuck_in_queued)
             if tis_for_warning_message:
@@ -1537,20 +1569,21 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             Stats.gauge(f"pool.open_slots.{pool_name}", slot_stats["open"])
             Stats.gauge(f"pool.queued_slots.{pool_name}", slot_stats["queued"])
             Stats.gauge(f"pool.running_slots.{pool_name}", slot_stats["running"])
-            # tagged metrics
+            Stats.gauge(f"pool.deferred_slots.{pool_name}", slot_stats["deferred"])
+            # Same metrics with tagging
             Stats.gauge("pool.open_slots", slot_stats["open"], tags={"pool_name": pool_name})
             Stats.gauge("pool.queued_slots", slot_stats["queued"], tags={"pool_name": pool_name})
             Stats.gauge("pool.running_slots", slot_stats["running"], tags={"pool_name": pool_name})
+            Stats.gauge("pool.deferred_slots", slot_stats["deferred"], tags={"pool_name": pool_name})
 
     @provide_session
     def adopt_or_reset_orphaned_tasks(self, session: Session = NEW_SESSION) -> int:
         """
-        Reset any TaskInstance still in QUEUED or SCHEDULED states that were
-        enqueued by a SchedulerJob that is no longer running.
+        Adopt or reset any TaskInstance in resettable state if its SchedulerJob is no longer running.
 
         :return: the number of TIs reset
         """
-        self.log.info("Resetting orphaned tasks for active dag runs")
+        self.log.info("Adopting or resetting orphaned tasks for active dag runs")
         timeout = conf.getint("scheduler", "scheduler_health_check_threshold")
 
         for attempt in run_with_db_retries(logger=self.log):
@@ -1562,55 +1595,55 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
                 )
                 self.log.debug("Calling SchedulerJob.adopt_or_reset_orphaned_tasks method")
                 try:
-                    num_failed = (
-                        session.query(Job)
-                        .filter(
+                    num_failed = session.execute(
+                        update(Job)
+                        .where(
                             Job.job_type == "SchedulerJob",
-                            Job.state == State.RUNNING,
+                            Job.state == JobState.RUNNING,
                             Job.latest_heartbeat < (timezone.utcnow() - timedelta(seconds=timeout)),
                         )
-                        .update({"state": State.FAILED})
-                    )
+                        .values(state=JobState.FAILED)
+                    ).rowcount
 
                     if num_failed:
                         self.log.info("Marked %d SchedulerJob instances as failed", num_failed)
                         Stats.incr(self.__class__.__name__.lower() + "_end", num_failed)
 
-                    resettable_states = [TaskInstanceState.QUEUED, TaskInstanceState.RUNNING]
                     query = (
-                        session.query(TI)
-                        .filter(TI.state.in_(resettable_states))
+                        select(TI)
+                        .where(TI.state.in_(State.adoptable_states))
                         # outerjoin is because we didn't use to have queued_by_job
                         # set, so we need to pick up anything pre upgrade. This (and the
                         # "or queued_by_job_id IS NONE") can go as soon as scheduler HA is
                         # released.
                         .outerjoin(TI.queued_by_job)
-                        .filter(or_(TI.queued_by_job_id.is_(None), Job.state != State.RUNNING))
+                        .where(or_(TI.queued_by_job_id.is_(None), Job.state != JobState.RUNNING))
                         .join(TI.dag_run)
-                        .filter(
+                        .where(
                             DagRun.run_type != DagRunType.BACKFILL_JOB,
-                            DagRun.state == State.RUNNING,
+                            DagRun.state == DagRunState.RUNNING,
                         )
                         .options(load_only(TI.dag_id, TI.task_id, TI.run_id))
                     )
 
                     # Lock these rows, so that another scheduler can't try and adopt these too
-                    tis_to_reset_or_adopt = with_row_locks(
+                    tis_to_adopt_or_reset = with_row_locks(
                         query, of=TI, session=session, **skip_locked(session=session)
-                    ).all()
-                    to_reset = self.job.executor.try_adopt_task_instances(tis_to_reset_or_adopt)
+                    )
+                    tis_to_adopt_or_reset = session.scalars(tis_to_adopt_or_reset).all()
+                    to_reset = self.job.executor.try_adopt_task_instances(tis_to_adopt_or_reset)
 
                     reset_tis_message = []
                     for ti in to_reset:
                         reset_tis_message.append(repr(ti))
-                        ti.state = State.NONE
+                        ti.state = None
                         ti.queued_by_job_id = None
 
-                    for ti in set(tis_to_reset_or_adopt) - set(to_reset):
+                    for ti in set(tis_to_adopt_or_reset) - set(to_reset):
                         ti.queued_by_job_id = self.job.id
 
                     Stats.incr("scheduler.orphaned_tasks.cleared", len(to_reset))
-                    Stats.incr("scheduler.orphaned_tasks.adopted", len(tis_to_reset_or_adopt) - len(to_reset))
+                    Stats.incr("scheduler.orphaned_tasks.adopted", len(tis_to_adopt_or_reset) - len(to_reset))
 
                     if to_reset:
                         task_instance_str = "\n\t".join(reset_tis_message)
@@ -1631,35 +1664,28 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
 
     @provide_session
     def check_trigger_timeouts(self, session: Session = NEW_SESSION) -> None:
-        """
-        Looks at all tasks that are in the "deferred" state and whose trigger
-        or execution timeout has passed, so they can be marked as failed.
-        """
-        num_timed_out_tasks = (
-            session.query(TI)
-            .filter(
+        """Mark any "deferred" task as failed if the trigger or execution timeout has passed."""
+        num_timed_out_tasks = session.execute(
+            update(TI)
+            .where(
                 TI.state == TaskInstanceState.DEFERRED,
                 TI.trigger_timeout < timezone.utcnow(),
             )
-            .update(
-                # We have to schedule these to fail themselves so it doesn't
-                # happen inside the scheduler.
-                {
-                    "state": TaskInstanceState.SCHEDULED,
-                    "next_method": "__fail__",
-                    "next_kwargs": {"error": "Trigger/execution timeout"},
-                    "trigger_id": None,
-                }
+            .values(
+                state=TaskInstanceState.SCHEDULED,
+                next_method="__fail__",
+                next_kwargs={"error": "Trigger/execution timeout"},
+                trigger_id=None,
             )
-        )
+        ).rowcount
         if num_timed_out_tasks:
             self.log.info("Timed out %i deferred tasks without fired triggers", num_timed_out_tasks)
 
     def _find_zombies(self) -> None:
         """
-        Find zombie task instances, which are tasks haven't heartbeated for too long
-        or have a no-longer-running LocalTaskJob, and create a TaskCallbackRequest
-        to be handled by the DAG processor.
+        Find zombie task instances and create a TaskCallbackRequest to be handled by the DAG processor.
+
+        Zombie instances are tasks haven't heartbeated for too long or have a no-longer-running LocalTaskJob.
         """
         from airflow.jobs.job import Job
 
@@ -1668,19 +1694,22 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
 
         with create_session() as session:
             zombies: list[tuple[TI, str, str]] = (
-                session.query(TI, DM.fileloc, DM.processor_subdir)
-                .with_hint(TI, "USE INDEX (ti_state)", dialect_name="mysql")
-                .join(Job, TI.job_id == Job.id)
-                .join(DM, TI.dag_id == DM.dag_id)
-                .filter(TI.state == TaskInstanceState.RUNNING)
-                .filter(
-                    or_(
-                        Job.state != State.RUNNING,
-                        Job.latest_heartbeat < limit_dttm,
+                session.execute(
+                    select(TI, DM.fileloc, DM.processor_subdir)
+                    .with_hint(TI, "USE INDEX (ti_state)", dialect_name="mysql")
+                    .join(Job, TI.job_id == Job.id)
+                    .join(DM, TI.dag_id == DM.dag_id)
+                    .where(TI.state == TaskInstanceState.RUNNING)
+                    .where(
+                        or_(
+                            Job.state != JobState.RUNNING,
+                            Job.latest_heartbeat < limit_dttm,
+                        )
                     )
+                    .where(Job.job_type == "LocalTaskJob")
+                    .where(TI.queued_by_job_id == self.job.id)
                 )
-                .filter(Job.job_type == "LocalTaskJob")
-                .filter(TI.queued_by_job_id == self.job.id)
+                .unique()
                 .all()
             )
 
@@ -1697,12 +1726,10 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
             )
             self.log.error("Detected zombie job: %s", request)
             self.job.executor.send_callback(request)
-            Stats.incr(
-                "zombies_killed", tags={"dag_id": ti.dag_id, "run_id": ti.run_id, "task_id": ti.task_id}
-            )
+            Stats.incr("zombies_killed", tags={"dag_id": ti.dag_id, "task_id": ti.task_id})
 
     @staticmethod
-    def _generate_zombie_message_details(ti: TI):
+    def _generate_zombie_message_details(ti: TI) -> dict[str, Any]:
         zombie_message_details = {
             "DAG Id": ti.dag_id,
             "Task Id": ti.task_id,
@@ -1724,15 +1751,15 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
         Find all dags that were not updated by Dag Processor recently and mark them as inactive.
 
         In case one of DagProcessors is stopped (in case there are multiple of them
-        for different dag folders), it's dags are never marked as inactive.
+        for different dag folders), its dags are never marked as inactive.
         Also remove dags from SerializedDag table.
         Executed on schedule only if [scheduler]standalone_dag_processor is True.
         """
         self.log.debug("Checking dags not parsed within last %s seconds.", self._dag_stale_not_seen_duration)
         limit_lpt = timezone.utcnow() - timedelta(seconds=self._dag_stale_not_seen_duration)
-        stale_dags = (
-            session.query(DagModel).filter(DagModel.is_active, DagModel.last_parsed_time < limit_lpt).all()
-        )
+        stale_dags = session.scalars(
+            select(DagModel).where(DagModel.is_active, DagModel.last_parsed_time < limit_lpt)
+        ).all()
         if not stale_dags:
             self.log.debug("Not stale dags found.")
             return
@@ -1751,11 +1778,12 @@ class SchedulerJobRunner(BaseJobRunner[Job], LoggingMixin):
     @provide_session
     def _orphan_unreferenced_datasets(self, session: Session = NEW_SESSION) -> None:
         """
-        Detects datasets that are no longer referenced in any DAG schedule parameters or task outlets and
-        sets the dataset is_orphaned flag to True
+        Detect orphaned datasets and set is_orphaned flag to True.
+
+        An orphaned dataset is no longer referenced in any DAG schedule parameters or task outlets.
         """
-        orphaned_dataset_query = (
-            session.query(DatasetModel)
+        orphaned_dataset_query = session.scalars(
+            select(DatasetModel)
             .join(
                 DagScheduleDatasetReference,
                 isouter=True,
