@@ -26,16 +26,16 @@ from __future__ import annotations
 from alembic import op
 
 # revision identifiers, used by Alembic.
-revision = '03bc53e68815'
-down_revision = ('0a2a5b66e19d', 'bf00311e1990')
+revision = "03bc53e68815"
+down_revision = ("0a2a5b66e19d", "bf00311e1990")
 branch_labels = None
 depends_on = None
-airflow_version = '1.10.2'
+airflow_version = "1.10.2"
 
 
 def upgrade():
-    op.create_index('sm_dag', 'sla_miss', ['dag_id'], unique=False)
+    op.create_index("sm_dag", "sla_miss", ["dag_id"], unique=False)
 
 
 def downgrade():
-    op.drop_index('sm_dag', table_name='sla_miss')
+    op.drop_index("sm_dag", table_name="sla_miss")
