@@ -30,20 +30,20 @@ from alembic import op
 from airflow.migrations.db_types import StringID
 
 # revision identifiers, used by Alembic.
-revision = 'b3b105409875'
-down_revision = 'd38e04c12aa2'
+revision = "b3b105409875"
+down_revision = "d38e04c12aa2"
 branch_labels = None
 depends_on = None
-airflow_version = '1.10.7'
+airflow_version = "1.10.7"
 
 
 def upgrade():
     """Apply Add ``root_dag_id`` to ``DAG``"""
-    op.add_column('dag', sa.Column('root_dag_id', StringID(), nullable=True))
-    op.create_index('idx_root_dag_id', 'dag', ['root_dag_id'], unique=False)
+    op.add_column("dag", sa.Column("root_dag_id", StringID(), nullable=True))
+    op.create_index("idx_root_dag_id", "dag", ["root_dag_id"], unique=False)
 
 
 def downgrade():
     """Unapply Add ``root_dag_id`` to ``DAG``"""
-    op.drop_index('idx_root_dag_id', table_name='dag')
-    op.drop_column('dag', 'root_dag_id')
+    op.drop_index("idx_root_dag_id", table_name="dag")
+    op.drop_column("dag", "root_dag_id")
