@@ -49,11 +49,18 @@ from typing import (
 import attr
 import pendulum
 from dateutil.relativedelta import relativedelta
+from sqlalchemy import select
 from sqlalchemy.orm import Session
 from sqlalchemy.orm.exc import NoResultFound
 
 from airflow.configuration import conf
-from airflow.exceptions import AirflowException, RemovedInAirflow3Warning, TaskDeferred
+from airflow.exceptions import (
+    AirflowException,
+    FailStopDagInvalidTriggerRule,
+    RemovedInAirflow3Warning,
+    TaskDeferralError,
+    TaskDeferred,
+)
 from airflow.lineage import apply_lineage, prepare_lineage
 from airflow.models.abstractoperator import (
     DEFAULT_IGNORE_FIRST_DEPENDS_ON_PAST,
@@ -74,7 +81,7 @@ from airflow.models.mappedoperator import OperatorPartial, validate_mapping_kwar
 from airflow.models.param import ParamsDict
 from airflow.models.pool import Pool
 from airflow.models.taskinstance import TaskInstance, clear_task_instances
-from airflow.models.taskmixin import DAGNode, DependencyMixin
+from airflow.models.taskmixin import DependencyMixin
 from airflow.serialization.enums import DagAttributeTypes
 from airflow.ti_deps.deps.base_ti_dep import BaseTIDep
 from airflow.ti_deps.deps.not_in_retry_period_dep import NotInRetryPeriodDep
@@ -85,6 +92,7 @@ from airflow.triggers.base import BaseTrigger
 from airflow.utils import timezone
 from airflow.utils.context import Context
 from airflow.utils.decorators import fixup_decorator_warning_stack
+from airflow.utils.edgemodifier import EdgeModifier
 from airflow.utils.helpers import validate_key
 from airflow.utils.operator_resources import Resources
 from airflow.utils.session import NEW_SESSION, provide_session
@@ -98,6 +106,7 @@ if TYPE_CHECKING:
     import jinja2  # Slow import.
 
     from airflow.models.dag import DAG
+    from airflow.models.operator import Operator
     from airflow.models.taskinstancekey import TaskInstanceKey
     from airflow.models.xcom_arg import XComArg
     from airflow.utils.task_group import TaskGroup
@@ -352,8 +361,7 @@ class BaseOperatorMeta(abc.ABCMeta):
     @classmethod
     def _apply_defaults(cls, func: T) -> T:
         """
-        Function decorator that Looks for an argument named "default_args", and
-        fills the unspecified arguments from it.
+        Look for an argument named "default_args", and fill the unspecified arguments from it.
 
         Since python2.* isn't clear about which arguments are missing when
         calling a function, and that this can be quite confusing with multi-level
@@ -407,7 +415,7 @@ class BaseOperatorMeta(abc.ABCMeta):
                 if arg not in kwargs and arg in default_args:
                     kwargs[arg] = default_args[arg]
 
-            missing_args = non_optional_args - set(kwargs)
+            missing_args = non_optional_args.difference(kwargs)
             if len(missing_args) == 1:
                 raise AirflowException(f"missing keyword argument {missing_args.pop()!r}")
             elif missing_args:
@@ -460,10 +468,12 @@ class BaseOperatorMeta(abc.ABCMeta):
 @functools.total_ordering
 class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
     """
-    Abstract base class for all operators. Since operators create objects that
-    become nodes in the dag, BaseOperator contains many recursive methods for
-    dag crawling behavior. To derive this class, you are expected to override
-    the constructor as well as the 'execute' method.
+    Abstract base class for all operators.
+
+    Since operators create objects that become nodes in the DAG, BaseOperator
+    contains many recursive methods for DAG crawling behavior. To derive from
+    this class, you are expected to override the constructor and the 'execute'
+    method.
 
     Operators derived from this class should perform or trigger certain tasks
     synchronously (wait for completion). Example of operators could be an
@@ -720,10 +730,6 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
     # Set to True for an operator instantiated by a mapped operator.
     __from_mapped = False
 
-    _is_setup = False
-    _is_teardown = False
-    _on_failure_fail_dagrun = False
-
     def __init__(
         self,
         task_id: str,
@@ -866,6 +872,8 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
             )
 
         self.trigger_rule: TriggerRule = TriggerRule(trigger_rule)
+        FailStopDagInvalidTriggerRule.check(dag=dag, trigger_rule=self.trigger_rule)
+
         self.depends_on_past: bool = depends_on_past
         self.ignore_first_depends_on_past: bool = ignore_first_depends_on_past
         self.wait_for_past_depends_before_skipping: bool = wait_for_past_depends_before_skipping
@@ -955,33 +963,11 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
             )
             self.template_fields = [self.template_fields]
 
+        self._is_setup = False
+        self._is_teardown = False
         if SetupTeardownContext.active:
             SetupTeardownContext.update_context_map(self)
 
-    @classmethod
-    def as_setup(cls, *args, **kwargs):
-        from airflow.settings import _ENABLE_AIP_52
-
-        if not _ENABLE_AIP_52:
-            raise AirflowException("AIP-52 Setup tasks are disabled.")
-
-        op = cls(*args, **kwargs)
-        op._is_setup = True
-        return op
-
-    @classmethod
-    def as_teardown(cls, *args, **kwargs):
-        from airflow.settings import _ENABLE_AIP_52
-
-        if not _ENABLE_AIP_52:
-            raise AirflowException("AIP-52 Teardown tasks are disabled.")
-
-        on_failure_fail_dagrun = kwargs.pop("on_failure_fail_dagrun", False)
-        op = cls(*args, **kwargs)
-        op._is_teardown = True
-        op._on_failure_fail_dagrun = on_failure_fail_dagrun
-        return op
-
     def __eq__(self, other):
         if type(self) is type(other):
             # Use getattr() instead of __dict__ as __dict__ doesn't return
@@ -1006,9 +992,10 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
     # including lineage information
     def __or__(self, other):
         """
-        Called for [This Operator] | [Operator], The inlets of other
-        will be set to pick up the outlets from this operator. Other will
-        be set as a downstream task of this operator.
+        Return [This Operator] | [Operator].
+
+        The inlets of other will be set to pick up the outlets from this operator.
+        Other will be set as a downstream task of this operator.
         """
         if isinstance(other, BaseOperator):
             if not self.outlets and not self.supports_lineage:
@@ -1024,8 +1011,9 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
 
     def __gt__(self, other):
         """
-        Called for [Operator] > [Outlet], so that if other is an attr annotated object
-        it is set as an outlet of this Operator.
+        Return [Operator] > [Outlet].
+
+        If other is an attr annotated object it is set as an outlet of this Operator.
         """
         if not isinstance(other, Iterable):
             other = [other]
@@ -1039,8 +1027,9 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
 
     def __lt__(self, other):
         """
-        Called for [Inlet] > [Operator] or [Operator] < [Inlet], so that if other is
-        an attr annotated object it is set as an inlet to this operator
+        Return [Inlet] > [Operator] or [Operator] < [Inlet].
+
+        If other is an attr annotated object it is set as an inlet to this operator.
         """
         if not isinstance(other, Iterable):
             other = [other]
@@ -1066,19 +1055,25 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
             self.set_xcomargs_dependencies()
 
     def add_inlets(self, inlets: Iterable[Any]):
-        """Sets inlets to this operator"""
+        """Set inlets to this operator."""
         self.inlets.extend(inlets)
 
     def add_outlets(self, outlets: Iterable[Any]):
-        """Defines the outlets of this operator"""
+        """Define the outlets of this operator."""
         self.outlets.extend(outlets)
 
     def get_inlet_defs(self):
-        """:meta private:"""
+        """Get inlet definitions on this task.
+
+        :meta private:
+        """
         return self.inlets
 
     def get_outlet_defs(self):
-        """:meta private:"""
+        """Get outlet definitions on this task.
+
+        :meta private:
+        """
         return self.outlets
 
     def get_dag(self) -> DAG | None:
@@ -1086,7 +1081,7 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
 
     @property  # type: ignore[override]
     def dag(self) -> DAG:  # type: ignore[override]
-        """Returns the Operator's DAG if set, otherwise raises an error"""
+        """Returns the Operator's DAG if set, otherwise raises an error."""
         if self._dag:
             return self._dag
         else:
@@ -1094,10 +1089,7 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
 
     @dag.setter
     def dag(self, dag: DAG | None):
-        """
-        Operators can be assigned to one DAG, one time. Repeat assignments to
-        that same DAG are ok.
-        """
+        """Operators can be assigned to one DAG, one time. Repeat assignments to that same DAG are ok."""
         from airflow.models.dag import DAG
 
         if dag is None:
@@ -1118,7 +1110,7 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
         self._dag = dag
 
     def has_dag(self):
-        """Returns True if the Operator has been assigned to a DAG."""
+        """Return True if the Operator has been assigned to a DAG."""
         return self._dag is not None
 
     deps: frozenset[BaseTIDep] = frozenset(
@@ -1136,19 +1128,17 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
     """
 
     def prepare_for_execution(self) -> BaseOperator:
-        """
-        Lock task for execution to disable custom action in __setattr__ and
-        returns a copy of the task
-        """
+        """Lock task for execution to disable custom action in ``__setattr__`` and return a copy."""
         other = copy.copy(self)
         other._lock_for_execution = True
         return other
 
     def set_xcomargs_dependencies(self) -> None:
         """
-        Resolves upstream dependencies of a task. In this way passing an ``XComArg``
-        as value for a template field will result in creating upstream relation between
-        two tasks.
+        Resolve upstream dependencies of a task.
+
+        In this way passing an ``XComArg`` as value for a template field
+        will result in creating upstream relation between two tasks.
 
         **Example**: ::
 
@@ -1174,13 +1164,14 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
 
     @prepare_lineage
     def pre_execute(self, context: Any):
-        """This hook is triggered right before self.execute() is called."""
+        """Execute right before self.execute() is called."""
         if self._pre_execute_hook is not None:
             self._pre_execute_hook(context)
 
     def execute(self, context: Context) -> Any:
         """
-        This is the main method to derive when creating an operator.
+        Derive when creating an operator.
+
         Context is the same dictionary used as when rendering jinja templates.
 
         Refer to get_template_context for more context.
@@ -1190,19 +1181,19 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
     @apply_lineage
     def post_execute(self, context: Any, result: Any = None):
         """
-        This hook is triggered right after self.execute() is called.
-        It is passed the execution context and any results returned by the
-        operator.
+        Execute right after self.execute() is called.
+
+        It is passed the execution context and any results returned by the operator.
         """
         if self._post_execute_hook is not None:
             self._post_execute_hook(context, result)
 
     def on_kill(self) -> None:
         """
-        Override this method to clean up subprocesses when a task instance
-        gets killed. Any use of the threading, subprocess or multiprocessing
-        module within an operator needs to be cleaned up, or it will leave
-        ghost processes behind.
+        Override this method to clean up subprocesses when a task instance gets killed.
+
+        Any use of the threading, subprocess or multiprocessing module within an
+        operator needs to be cleaned up, or it will leave ghost processes behind.
         """
 
     def __deepcopy__(self, memo):
@@ -1262,16 +1253,13 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
         downstream: bool = False,
         session: Session = NEW_SESSION,
     ):
-        """
-        Clears the state of task instances associated with the task, following
-        the parameters specified.
-        """
-        qry = session.query(TaskInstance).filter(TaskInstance.dag_id == self.dag_id)
+        """Clear the state of task instances associated with the task, following the parameters specified."""
+        qry = select(TaskInstance).where(TaskInstance.dag_id == self.dag_id)
 
         if start_date:
-            qry = qry.filter(TaskInstance.execution_date >= start_date)
+            qry = qry.where(TaskInstance.execution_date >= start_date)
         if end_date:
-            qry = qry.filter(TaskInstance.execution_date <= end_date)
+            qry = qry.where(TaskInstance.execution_date <= end_date)
 
         tasks = [self.task_id]
 
@@ -1281,8 +1269,8 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
         if downstream:
             tasks += [t.task_id for t in self.get_flat_relatives(upstream=False)]
 
-        qry = qry.filter(TaskInstance.task_id.in_(tasks))
-        results = qry.all()
+        qry = qry.where(TaskInstance.task_id.in_(tasks))
+        results = session.scalars(qry).all()
         count = len(results)
         clear_task_instances(results, session, dag=self.dag)
         session.commit()
@@ -1298,17 +1286,17 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
         """Get task instances related to this task for a specific date range."""
         from airflow.models import DagRun
 
-        end_date = end_date or timezone.utcnow()
-        return (
-            session.query(TaskInstance)
+        query = (
+            select(TaskInstance)
             .join(TaskInstance.dag_run)
-            .filter(TaskInstance.dag_id == self.dag_id)
-            .filter(TaskInstance.task_id == self.task_id)
-            .filter(DagRun.execution_date >= start_date)
-            .filter(DagRun.execution_date <= end_date)
-            .order_by(DagRun.execution_date)
-            .all()
+            .where(TaskInstance.dag_id == self.dag_id)
+            .where(TaskInstance.task_id == self.task_id)
         )
+        if start_date:
+            query = query.where(DagRun.execution_date >= start_date)
+        if end_date:
+            query = query.where(DagRun.execution_date <= end_date)
+        return session.scalars(query.order_by(DagRun.execution_date)).all()
 
     @provide_session
     def run(
@@ -1337,14 +1325,12 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
         for info in self.dag.iter_dagrun_infos_between(start_date, end_date, align=False):
             ignore_depends_on_past = info.logical_date == start_date and ignore_first_depends_on_past
             try:
-                dag_run = (
-                    session.query(DagRun)
-                    .filter(
+                dag_run = session.scalars(
+                    select(DagRun).where(
                         DagRun.dag_id == self.dag_id,
                         DagRun.execution_date == info.logical_date,
                     )
-                    .one()
-                )
+                ).one()
                 ti = TaskInstance(self, run_id=dag_run.run_id)
             except NoResultFound:
                 # This is _mostly_ only used in tests
@@ -1370,7 +1356,7 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
             )
 
     def dry_run(self) -> None:
-        """Performs dry run for the operator - just render template fields."""
+        """Perform dry run for the operator - just render template fields."""
         self.log.info("Dry run")
         for field in self.template_fields:
             try:
@@ -1385,18 +1371,15 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
                 self.log.info("Rendering template for %s", field)
                 self.log.info(content)
 
-    def get_direct_relatives(self, upstream: bool = False) -> Iterable[DAGNode]:
-        """
-        Get list of the direct relatives to the current task, upstream or
-        downstream.
-        """
+    def get_direct_relatives(self, upstream: bool = False) -> Iterable[Operator]:
+        """Get list of the direct relatives to the current task, upstream or downstream."""
         if upstream:
             return self.upstream_list
         else:
             return self.downstream_list
 
     def __repr__(self):
-        return "<Task({self.task_type}): {self.task_id}>".format(self=self)
+        return f"<Task({self.task_type}): {self.task_id}>"
 
     @property
     def operator_class(self) -> type[BaseOperator]:  # type: ignore[override]
@@ -1404,12 +1387,12 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
 
     @property
     def task_type(self) -> str:
-        """@property: type of the task"""
+        """@property: type of the task."""
         return self.__class__.__name__
 
     @property
     def operator_name(self) -> str:
-        """@property: use a more friendly display name for the operator, if set"""
+        """@property: use a more friendly display name for the operator, if set."""
         try:
             return self.custom_operator_name  # type: ignore
         except AttributeError:
@@ -1427,11 +1410,48 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
 
     @property
     def output(self) -> XComArg:
-        """Returns reference to XCom pushed by current operator"""
+        """Returns reference to XCom pushed by current operator."""
         from airflow.models.xcom_arg import XComArg
 
         return XComArg(operator=self)
 
+    @property
+    def is_setup(self) -> bool:
+        """Whether the operator is a setup task.
+
+        :meta private:
+        """
+        return self._is_setup
+
+    @is_setup.setter
+    def is_setup(self, value: bool) -> None:
+        """Setter for is_setup property.
+
+        :meta private:
+        """
+        if self.is_teardown and value:
+            raise ValueError(f"Cannot mark task '{self.task_id}' as setup; task is already a teardown.")
+        self._is_setup = value
+
+    @property
+    def is_teardown(self) -> bool:
+        """Whether the operator is a teardown task.
+
+        :meta private:
+        """
+        return self._is_teardown
+
+    @is_teardown.setter
+    def is_teardown(self, value: bool) -> None:
+        """
+        Setter for is_teardown property.
+
+        :meta private:
+        """
+        if self.is_setup and value:
+            raise ValueError(f"Cannot mark task '{self.task_id}' as teardown; task is already a setup.")
+        self._is_teardown = value
+
     @staticmethod
     def xcom_push(
         context: Any,
@@ -1453,12 +1473,14 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
         context["ti"].xcom_push(key=key, value=value, execution_date=execution_date)
 
     @staticmethod
+    @provide_session
     def xcom_pull(
         context: Any,
         task_ids: str | list[str] | None = None,
         dag_id: str | None = None,
         key: str = XCOM_RETURN_KEY,
         include_prior_dates: bool | None = None,
+        session: Session = NEW_SESSION,
     ) -> Any:
         """
         Pull XComs that optionally meet certain criteria.
@@ -1487,7 +1509,11 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
             are returned as well.
         """
         return context["ti"].xcom_pull(
-            key=key, task_ids=task_ids, dag_id=dag_id, include_prior_dates=include_prior_dates
+            key=key,
+            task_ids=task_ids,
+            dag_id=dag_id,
+            include_prior_dates=include_prior_dates,
+            session=session,
         )
 
     @classmethod
@@ -1512,6 +1538,9 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
                     "_BaseOperator__instantiated",
                     "_BaseOperator__init_kwargs",
                     "_BaseOperator__from_mapped",
+                    "_is_setup",
+                    "_is_teardown",
+                    "_on_failure_fail_dagrun",
                 }
                 | {  # Class level defaults need to be added to this list
                     "start_date",
@@ -1525,9 +1554,9 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
                     "template_fields",
                     "template_fields_renderers",
                     "params",
-                    "_is_setup",
-                    "_is_teardown",
-                    "_on_failure_fail_dagrun",
+                    "is_setup",
+                    "is_teardown",
+                    "on_failure_fail_dagrun",
                 }
             )
             DagContext.pop_context_managed_dag()
@@ -1535,12 +1564,12 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
         return cls.__serialized_fields
 
     def serialize_for_task_group(self) -> tuple[DagAttributeTypes, Any]:
-        """Required by DAGNode."""
+        """Serialize; required by DAGNode."""
         return DagAttributeTypes.OP, self.task_id
 
     @property
     def inherits_from_empty_operator(self):
-        """Used to determine if an Operator is inherited from EmptyOperator"""
+        """Used to determine if an Operator is inherited from EmptyOperator."""
         # This looks like `isinstance(self, EmptyOperator) would work, but this also
         # needs to cope when `self` is a Serialized instance of a EmptyOperator or one
         # of its subclasses (which don't inherit from anything but BaseOperator).
@@ -1555,16 +1584,37 @@ class BaseOperator(AbstractOperator, metaclass=BaseOperatorMeta):
         timeout: timedelta | None = None,
     ):
         """
-        Marks this Operator as being "deferred" - that is, suspending its
-        execution until the provided trigger fires an event.
+        Mark this Operator "deferred", suspending its execution until the provided trigger fires an event.
 
         This is achieved by raising a special exception (TaskDeferred)
         which is caught in the main _execute_task wrapper.
         """
         raise TaskDeferred(trigger=trigger, method_name=method_name, kwargs=kwargs, timeout=timeout)
 
+    def resume_execution(self, next_method: str, next_kwargs: dict[str, Any] | None, context: Context):
+        """This method is called when a deferred task is resumed."""
+        # __fail__ is a special signal value for next_method that indicates
+        # this task was scheduled specifically to fail.
+        if next_method == "__fail__":
+            next_kwargs = next_kwargs or {}
+            traceback = next_kwargs.get("traceback")
+            if traceback is not None:
+                self.log.error("Trigger failed:\n%s", "\n".join(traceback))
+            raise TaskDeferralError(next_kwargs.get("error", "Unknown"))
+        # Grab the callable off the Operator/Task and add in any kwargs
+        execute_callable = getattr(self, next_method)
+        if next_kwargs:
+            execute_callable = functools.partial(execute_callable, **next_kwargs)
+        return execute_callable(context)
+
     def unmap(self, resolve: None | dict[str, Any] | tuple[Context, Session]) -> BaseOperator:
-        """:meta private:"""
+        """Get the "normal" operator from the current operator.
+
+        Since a BaseOperator is not mapped to begin with, this simply returns
+        the original operator.
+
+        :meta private:
+        """
         return self
 
 
@@ -1680,8 +1730,7 @@ def chain(*tasks: DependencyMixin | Sequence[DependencyMixin]) -> None:
 
     :param tasks: Individual and/or list of tasks, EdgeModifiers, XComArgs, or TaskGroups to set dependencies
     """
-    for index, up_task in enumerate(tasks[:-1]):
-        down_task = tasks[index + 1]
+    for up_task, down_task in zip(tasks, tasks[1:]):
         if isinstance(up_task, DependencyMixin):
             up_task.set_downstream(down_task)
             continue
@@ -1802,6 +1851,44 @@ def cross_downstream(
         task.set_downstream(to_tasks)
 
 
+def chain_linear(*elements: DependencyMixin | Sequence[DependencyMixin]):
+    """
+    Simplify task dependency definition.
+
+    E.g.: suppose you want precedence like so::
+
+            ╭─op2─╮ ╭─op4─╮
+        op1─┤     ├─├─op5─┤─op7
+            ╰-op3─╯ ╰-op6─╯
+
+    Then you can accomplish like so::
+
+        chain_linear(
+            op1,
+            [op2, op3],
+            [op4, op5, op6],
+            op7
+        )
+
+    :param elements: a list of operators / lists of operators
+    """
+    if not elements:
+        raise ValueError("No tasks provided; nothing to do.")
+    prev_elem = None
+    deps_set = False
+    for curr_elem in elements:
+        if isinstance(curr_elem, EdgeModifier):
+            raise ValueError("Labels are not supported by chain_linear")
+        if prev_elem is not None:
+            for task in prev_elem:
+                task >> curr_elem
+                if not deps_set:
+                    deps_set = True
+        prev_elem = [curr_elem] if isinstance(curr_elem, DependencyMixin) else curr_elem
+    if not deps_set:
+        raise ValueError("No dependencies were set. Did you forget to expand with `*`?")
+
+
 # pyupgrade assumes all type annotations can be lazily evaluated, but this is
 # not the case for attrs-decorated classes, since cattrs needs to evaluate the
 # annotation expressions at runtime, and Python before 3.9.0 does not lazily
