@@ -20,17 +20,18 @@ import logging
 import os
 import struct
 from datetime import datetime
-from typing import Iterable
+from typing import Collection, Iterable
 
-from sqlalchemy import BigInteger, Column, String, Text
+from sqlalchemy import BigInteger, Column, String, Text, delete, select
 from sqlalchemy.dialects.mysql import MEDIUMTEXT
+from sqlalchemy.orm import Session
 from sqlalchemy.sql.expression import literal
 
 from airflow.exceptions import AirflowException, DagCodeNotFound
 from airflow.models.base import Base
 from airflow.utils import timezone
 from airflow.utils.file import correct_maybe_zipped, open_maybe_zipped
-from airflow.utils.session import provide_session
+from airflow.utils.session import NEW_SESSION, provide_session
 from airflow.utils.sqlalchemy import UtcDateTime
 
 log = logging.getLogger(__name__)
@@ -44,13 +45,13 @@ class DagCode(Base):
     For details on dag serialization see SerializedDagModel
     """
 
-    __tablename__ = 'dag_code'
+    __tablename__ = "dag_code"
 
     fileloc_hash = Column(BigInteger, nullable=False, primary_key=True, autoincrement=False)
     fileloc = Column(String(2000), nullable=False)
     # The max length of fileloc exceeds the limit of indexing.
     last_updated = Column(UtcDateTime, nullable=False)
-    source_code = Column(Text().with_variant(MEDIUMTEXT(), 'mysql'), nullable=False)
+    source_code = Column(Text().with_variant(MEDIUMTEXT(), "mysql"), nullable=False)
 
     def __init__(self, full_filepath: str, source_code: str | None = None):
         self.fileloc = full_filepath
@@ -59,8 +60,8 @@ class DagCode(Base):
         self.source_code = source_code or DagCode.code(self.fileloc)
 
     @provide_session
-    def sync_to_db(self, session=None):
-        """Writes code into database.
+    def sync_to_db(self, session: Session = NEW_SESSION) -> None:
+        """Write code into database.
 
         :param session: ORM Session
         """
@@ -68,20 +69,19 @@ class DagCode(Base):
 
     @classmethod
     @provide_session
-    def bulk_sync_to_db(cls, filelocs: Iterable[str], session=None):
-        """Writes code in bulk into database.
+    def bulk_sync_to_db(cls, filelocs: Iterable[str], session: Session = NEW_SESSION) -> None:
+        """Write code in bulk into database.
 
         :param filelocs: file paths of DAGs to sync
         :param session: ORM Session
         """
         filelocs = set(filelocs)
         filelocs_to_hashes = {fileloc: DagCode.dag_fileloc_hash(fileloc) for fileloc in filelocs}
-        existing_orm_dag_codes = (
-            session.query(DagCode)
+        existing_orm_dag_codes = session.scalars(
+            select(DagCode)
             .filter(DagCode.fileloc_hash.in_(filelocs_to_hashes.values()))
             .with_for_update(of=DagCode)
-            .all()
-        )
+        ).all()
 
         if existing_orm_dag_codes:
             existing_orm_dag_codes_map = {
@@ -125,34 +125,49 @@ class DagCode(Base):
 
     @classmethod
     @provide_session
-    def remove_deleted_code(cls, alive_dag_filelocs: list[str], session=None):
-        """Deletes code not included in alive_dag_filelocs.
+    def remove_deleted_code(
+        cls,
+        alive_dag_filelocs: Collection[str],
+        processor_subdir: str,
+        session: Session = NEW_SESSION,
+    ) -> None:
+        """Delete code not included in alive_dag_filelocs.
 
         :param alive_dag_filelocs: file paths of alive DAGs
+        :param processor_subdir: dag processor subdir
         :param session: ORM Session
         """
         alive_fileloc_hashes = [cls.dag_fileloc_hash(fileloc) for fileloc in alive_dag_filelocs]
 
         log.debug("Deleting code from %s table ", cls.__tablename__)
 
-        session.query(cls).filter(
-            cls.fileloc_hash.notin_(alive_fileloc_hashes), cls.fileloc.notin_(alive_dag_filelocs)
-        ).delete(synchronize_session='fetch')
+        session.execute(
+            delete(cls)
+            .where(
+                cls.fileloc_hash.notin_(alive_fileloc_hashes),
+                cls.fileloc.notin_(alive_dag_filelocs),
+                cls.fileloc.contains(processor_subdir),
+            )
+            .execution_options(synchronize_session="fetch")
+        )
 
     @classmethod
     @provide_session
-    def has_dag(cls, fileloc: str, session=None) -> bool:
-        """Checks a file exist in dag_code table.
+    def has_dag(cls, fileloc: str, session: Session = NEW_SESSION) -> bool:
+        """Check a file exist in dag_code table.
 
         :param fileloc: the file to check
         :param session: ORM Session
         """
         fileloc_hash = cls.dag_fileloc_hash(fileloc)
-        return session.query(literal(True)).filter(cls.fileloc_hash == fileloc_hash).one_or_none() is not None
+        return (
+            session.scalars(select(literal(True)).where(cls.fileloc_hash == fileloc_hash)).one_or_none()
+            is not None
+        )
 
     @classmethod
     def get_code_by_fileloc(cls, fileloc: str) -> str:
-        """Returns source code for a given fileloc.
+        """Return source code for a given fileloc.
 
         :param fileloc: file path of a DAG
         :return: source code as string
@@ -161,7 +176,7 @@ class DagCode(Base):
 
     @classmethod
     def code(cls, fileloc) -> str:
-        """Returns source code for this DagCode object.
+        """Return source code for this DagCode object.
 
         :return: source code as string
         """
@@ -169,14 +184,14 @@ class DagCode(Base):
 
     @staticmethod
     def _get_code_from_file(fileloc):
-        with open_maybe_zipped(fileloc, 'r') as f:
+        with open_maybe_zipped(fileloc, "r") as f:
             code = f.read()
         return code
 
     @classmethod
     @provide_session
-    def _get_code_from_db(cls, fileloc, session=None):
-        dag_code = session.query(cls).filter(cls.fileloc_hash == cls.dag_fileloc_hash(fileloc)).first()
+    def _get_code_from_db(cls, fileloc, session: Session = NEW_SESSION) -> str:
+        dag_code = session.scalar(select(cls).where(cls.fileloc_hash == cls.dag_fileloc_hash(fileloc)))
         if not dag_code:
             raise DagCodeNotFound()
         else:
@@ -195,4 +210,4 @@ class DagCode(Base):
         import hashlib
 
         # Only 7 bytes because MySQL BigInteger can hold only 8 bytes (signed).
-        return struct.unpack('>Q', hashlib.sha1(full_filepath.encode('utf-8')).digest()[-8:])[0] >> 8
+        return struct.unpack(">Q", hashlib.sha1(full_filepath.encode("utf-8")).digest()[-8:])[0] >> 8
