@@ -17,25 +17,27 @@
 # under the License.
 from __future__ import annotations
 
-from typing import Iterable
+from typing import Any
 
-from sqlalchemy import Column, Integer, String, Text, func
+from sqlalchemy import Boolean, Column, Integer, String, Text, func, select
 from sqlalchemy.orm.session import Session
 
 from airflow.exceptions import AirflowException, PoolNotFound
 from airflow.models.base import Base
 from airflow.ti_deps.dependencies_states import EXECUTION_STATES
 from airflow.typing_compat import TypedDict
+from airflow.utils.db import exists_query
 from airflow.utils.session import NEW_SESSION, provide_session
 from airflow.utils.sqlalchemy import nowait, with_row_locks
-from airflow.utils.state import State
+from airflow.utils.state import TaskInstanceState
 
 
 class PoolStats(TypedDict):
-    """Dictionary containing Pool Stats"""
+    """Dictionary containing Pool Stats."""
 
     total: int
     running: int
+    deferred: int
     queued: int
     open: int
 
@@ -50,6 +52,7 @@ class Pool(Base):
     # -1 for infinite
     slots = Column(Integer, default=0)
     description = Column(Text)
+    include_deferred = Column(Boolean, nullable=False)
 
     DEFAULT_POOL_NAME = "default_pool"
 
@@ -58,13 +61,13 @@ class Pool(Base):
 
     @staticmethod
     @provide_session
-    def get_pools(session: Session = NEW_SESSION):
+    def get_pools(session: Session = NEW_SESSION) -> list[Pool]:
         """Get all pools."""
-        return session.query(Pool).all()
+        return session.scalars(select(Pool)).all()
 
     @staticmethod
     @provide_session
-    def get_pool(pool_name: str, session: Session = NEW_SESSION):
+    def get_pool(pool_name: str, session: Session = NEW_SESSION) -> Pool | None:
         """
         Get the Pool with specific pool name from the Pools.
 
@@ -72,11 +75,11 @@ class Pool(Base):
         :param session: SQLAlchemy ORM Session
         :return: the pool object
         """
-        return session.query(Pool).filter(Pool.pool == pool_name).first()
+        return session.scalar(select(Pool).where(Pool.pool == pool_name))
 
     @staticmethod
     @provide_session
-    def get_default_pool(session: Session = NEW_SESSION):
+    def get_default_pool(session: Session = NEW_SESSION) -> Pool | None:
         """
         Get the Pool of the default_pool from the Pools.
 
@@ -95,39 +98,45 @@ class Pool(Base):
         :param session: SQLAlchemy ORM Session
         :return: True if id is default_pool, otherwise False
         """
-        return (
-            session.query(func.count(Pool.id))
-            .filter(Pool.id == id, Pool.pool == Pool.DEFAULT_POOL_NAME)
-            .scalar()
-            > 0
+        return exists_query(
+            Pool.id == id,
+            Pool.pool == Pool.DEFAULT_POOL_NAME,
+            session=session,
         )
 
     @staticmethod
     @provide_session
-    def create_or_update_pool(name: str, slots: int, description: str, session: Session = NEW_SESSION):
+    def create_or_update_pool(
+        name: str,
+        slots: int,
+        description: str,
+        include_deferred: bool,
+        session: Session = NEW_SESSION,
+    ) -> Pool:
         """Create a pool with given parameters or update it if it already exists."""
         if not name:
-            return
-        pool = session.query(Pool).filter_by(pool=name).first()
+            raise ValueError("Pool name must not be empty")
+
+        pool = session.scalar(select(Pool).filter_by(pool=name))
         if pool is None:
-            pool = Pool(pool=name, slots=slots, description=description)
+            pool = Pool(pool=name, slots=slots, description=description, include_deferred=include_deferred)
             session.add(pool)
         else:
             pool.slots = slots
             pool.description = description
+            pool.include_deferred = include_deferred
 
         session.commit()
-
         return pool
 
     @staticmethod
     @provide_session
-    def delete_pool(name: str, session: Session = NEW_SESSION):
+    def delete_pool(name: str, session: Session = NEW_SESSION) -> Pool:
         """Delete pool by a given name."""
         if name == Pool.DEFAULT_POOL_NAME:
             raise AirflowException(f"{Pool.DEFAULT_POOL_NAME} cannot be deleted")
 
-        pool = session.query(Pool).filter_by(pool=name).first()
+        pool = session.scalar(select(Pool).filter_by(pool=name))
         if pool is None:
             raise PoolNotFound(f"Pool '{name}' doesn't exist")
 
@@ -144,7 +153,7 @@ class Pool(Base):
         session: Session = NEW_SESSION,
     ) -> dict[str, PoolStats]:
         """
-        Get Pool stats (Number of Running, Queued, Open & Total tasks)
+        Get Pool stats (Number of Running, Queued, Open & Total tasks).
 
         If ``lock_rows`` is True, and the database engine in use supports the ``NOWAIT`` syntax, then a
         non-blocking lock will be attempted -- if the lock is not available then SQLAlchemy will throw an
@@ -156,26 +165,31 @@ class Pool(Base):
         from airflow.models.taskinstance import TaskInstance  # Avoid circular import
 
         pools: dict[str, PoolStats] = {}
+        pool_includes_deferred: dict[str, bool] = {}
 
-        query = session.query(Pool.pool, Pool.slots)
+        query = select(Pool.pool, Pool.slots, Pool.include_deferred)
 
         if lock_rows:
             query = with_row_locks(query, session=session, **nowait(session))
 
-        pool_rows: Iterable[tuple[str, int]] = query.all()
-        for (pool_name, total_slots) in pool_rows:
+        pool_rows = session.execute(query)
+        for (pool_name, total_slots, include_deferred) in pool_rows:
             if total_slots == -1:
                 total_slots = float("inf")  # type: ignore
-            pools[pool_name] = PoolStats(total=total_slots, running=0, queued=0, open=0)
+            pools[pool_name] = PoolStats(total=total_slots, running=0, queued=0, open=0, deferred=0)
+            pool_includes_deferred[pool_name] = include_deferred
 
-        state_count_by_pool = (
-            session.query(TaskInstance.pool, TaskInstance.state, func.sum(TaskInstance.pool_slots))
-            .filter(TaskInstance.state.in_(list(EXECUTION_STATES)))
+        allowed_execution_states = EXECUTION_STATES | {
+            TaskInstanceState.DEFERRED,
+        }
+        state_count_by_pool = session.execute(
+            select(TaskInstance.pool, TaskInstance.state, func.sum(TaskInstance.pool_slots))
+            .filter(TaskInstance.state.in_(allowed_execution_states))
             .group_by(TaskInstance.pool, TaskInstance.state)
-        ).all()
+        )
 
         # calculate queued and running metrics
-        for (pool_name, state, count) in state_count_by_pool:
+        for pool_name, state, count in state_count_by_pool:
             # Some databases return decimal.Decimal here.
             count = int(count)
 
@@ -183,22 +197,26 @@ class Pool(Base):
             if not stats_dict:
                 continue
             # TypedDict key must be a string literal, so we use if-statements to set value
-            if state == "running":
+            if state == TaskInstanceState.RUNNING:
                 stats_dict["running"] = count
-            elif state == "queued":
+            elif state == TaskInstanceState.QUEUED:
                 stats_dict["queued"] = count
+            elif state == TaskInstanceState.DEFERRED:
+                stats_dict["deferred"] = count
             else:
-                raise AirflowException(f"Unexpected state. Expected values: {EXECUTION_STATES}.")
+                raise AirflowException(f"Unexpected state. Expected values: {allowed_execution_states}.")
 
         # calculate open metric
         for pool_name, stats_dict in pools.items():
             stats_dict["open"] = stats_dict["total"] - stats_dict["running"] - stats_dict["queued"]
+            if pool_includes_deferred[pool_name]:
+                stats_dict["open"] -= stats_dict["deferred"]
 
         return pools
 
-    def to_json(self):
+    def to_json(self) -> dict[str, Any]:
         """
-        Get the Pool in a json structure
+        Get the Pool in a json structure.
 
         :return: the pool object in json format
         """
@@ -207,10 +225,11 @@ class Pool(Base):
             "pool": self.pool,
             "slots": self.slots,
             "description": self.description,
+            "include_deferred": self.include_deferred,
         }
 
     @provide_session
-    def occupied_slots(self, session: Session = NEW_SESSION):
+    def occupied_slots(self, session: Session = NEW_SESSION) -> int:
         """
         Get the number of slots used by running/queued tasks at the moment.
 
@@ -219,16 +238,26 @@ class Pool(Base):
         """
         from airflow.models.taskinstance import TaskInstance  # Avoid circular import
 
+        occupied_states = self.get_occupied_states()
+
         return int(
-            session.query(func.sum(TaskInstance.pool_slots))
-            .filter(TaskInstance.pool == self.pool)
-            .filter(TaskInstance.state.in_(list(EXECUTION_STATES)))
-            .scalar()
+            session.scalar(
+                select(func.sum(TaskInstance.pool_slots))
+                .filter(TaskInstance.pool == self.pool)
+                .filter(TaskInstance.state.in_(occupied_states))
+            )
             or 0
         )
 
+    def get_occupied_states(self):
+        if self.include_deferred:
+            return EXECUTION_STATES | {
+                TaskInstanceState.DEFERRED,
+            }
+        return EXECUTION_STATES
+
     @provide_session
-    def running_slots(self, session: Session = NEW_SESSION):
+    def running_slots(self, session: Session = NEW_SESSION) -> int:
         """
         Get the number of slots used by running tasks at the moment.
 
@@ -238,15 +267,16 @@ class Pool(Base):
         from airflow.models.taskinstance import TaskInstance  # Avoid circular import
 
         return int(
-            session.query(func.sum(TaskInstance.pool_slots))
-            .filter(TaskInstance.pool == self.pool)
-            .filter(TaskInstance.state == State.RUNNING)
-            .scalar()
+            session.scalar(
+                select(func.sum(TaskInstance.pool_slots))
+                .filter(TaskInstance.pool == self.pool)
+                .filter(TaskInstance.state == TaskInstanceState.RUNNING)
+            )
             or 0
         )
 
     @provide_session
-    def queued_slots(self, session: Session = NEW_SESSION):
+    def queued_slots(self, session: Session = NEW_SESSION) -> int:
         """
         Get the number of slots used by queued tasks at the moment.
 
@@ -256,15 +286,16 @@ class Pool(Base):
         from airflow.models.taskinstance import TaskInstance  # Avoid circular import
 
         return int(
-            session.query(func.sum(TaskInstance.pool_slots))
-            .filter(TaskInstance.pool == self.pool)
-            .filter(TaskInstance.state == State.QUEUED)
-            .scalar()
+            session.scalar(
+                select(func.sum(TaskInstance.pool_slots))
+                .filter(TaskInstance.pool == self.pool)
+                .filter(TaskInstance.state == TaskInstanceState.QUEUED)
+            )
             or 0
         )
 
     @provide_session
-    def scheduled_slots(self, session: Session = NEW_SESSION):
+    def scheduled_slots(self, session: Session = NEW_SESSION) -> int:
         """
         Get the number of slots scheduled at the moment.
 
@@ -274,10 +305,30 @@ class Pool(Base):
         from airflow.models.taskinstance import TaskInstance  # Avoid circular import
 
         return int(
-            session.query(func.sum(TaskInstance.pool_slots))
-            .filter(TaskInstance.pool == self.pool)
-            .filter(TaskInstance.state == State.SCHEDULED)
-            .scalar()
+            session.scalar(
+                select(func.sum(TaskInstance.pool_slots))
+                .filter(TaskInstance.pool == self.pool)
+                .filter(TaskInstance.state == TaskInstanceState.SCHEDULED)
+            )
+            or 0
+        )
+
+    @provide_session
+    def deferred_slots(self, session: Session = NEW_SESSION) -> int:
+        """
+        Get the number of slots deferred at the moment.
+
+        :param session: SQLAlchemy ORM Session
+        :return: the number of deferred slots
+        """
+        from airflow.models.taskinstance import TaskInstance  # Avoid circular import
+
+        return int(
+            session.scalar(
+                select(func.sum(TaskInstance.pool_slots)).where(
+                    TaskInstance.pool == self.pool, TaskInstance.state == TaskInstanceState.DEFERRED
+                )
+            )
             or 0
         )
 
@@ -291,5 +342,4 @@ class Pool(Base):
         """
         if self.slots == -1:
             return float("inf")
-        else:
-            return self.slots - self.occupied_slots(session)
+        return self.slots - self.occupied_slots(session)
