@@ -21,7 +21,7 @@ import json
 import logging
 from typing import Any
 
-from sqlalchemy import Boolean, Column, Integer, String, Text
+from sqlalchemy import Boolean, Column, Integer, String, Text, delete
 from sqlalchemy.dialects.mysql import MEDIUMTEXT
 from sqlalchemy.orm import Session, declared_attr, reconstructor, synonym
 
@@ -29,6 +29,7 @@ from airflow.api_internal.internal_api_call import internal_api_call
 from airflow.configuration import ensure_secrets_loaded
 from airflow.models.base import ID_LEN, Base
 from airflow.models.crypto import get_fernet
+from airflow.secrets.cache import SecretCache
 from airflow.secrets.metastore import MetastoreBackend
 from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.log.secrets_masker import mask_secret
@@ -38,10 +39,7 @@ log = logging.getLogger(__name__)
 
 
 class Variable(Base, LoggingMixin):
-    """
-    Variables are a generic way to store and retrieve arbitrary content or settings
-    as a simple key value store within Airflow.
-    """
+    """A generic way to store and retrieve arbitrary content or settings as a simple key/value store."""
 
     __tablename__ = "variable"
     __NO_DEFAULT_SENTINEL = object()
@@ -68,7 +66,7 @@ class Variable(Base, LoggingMixin):
         return f"{self.key} : {self._val}"
 
     def get_val(self):
-        """Get Airflow Variable from Metadata DB and decode it using the Fernet Key"""
+        """Get Airflow Variable from Metadata DB and decode it using the Fernet Key."""
         from cryptography.fernet import InvalidToken as InvalidFernetToken
 
         if self._val is not None and self.is_encrypted:
@@ -93,14 +91,15 @@ class Variable(Base, LoggingMixin):
 
     @declared_attr
     def val(cls):
-        """Get Airflow Variable from Metadata DB and decode it using the Fernet Key"""
+        """Get Airflow Variable from Metadata DB and decode it using the Fernet Key."""
         return synonym("_val", descriptor=property(cls.get_val, cls.set_val))
 
     @classmethod
     def setdefault(cls, key, default, description=None, deserialize_json=False):
         """
-        Like a Python builtin dict object, setdefault returns the current value
-        for a key, and if it isn't there, stores the default value and returns it.
+        Return the current value for a key or store the default value and return it.
+
+        Works the same as the Python builtin dict object.
 
         :param key: Dict key for this Variable
         :param default: Default value to set and return if the variable
@@ -127,8 +126,7 @@ class Variable(Base, LoggingMixin):
         default_var: Any = __NO_DEFAULT_SENTINEL,
         deserialize_json: bool = False,
     ) -> Any:
-        """
-        Gets a value for an Airflow Variable Key
+        """Get a value for an Airflow Variable Key.
 
         :param key: Variable Key
         :param default_var: Default value of the Variable if the Variable doesn't exist
@@ -158,16 +156,15 @@ class Variable(Base, LoggingMixin):
         description: str | None = None,
         serialize_json: bool = False,
         session: Session = None,
-    ):
-        """
-        Sets a value for an Airflow Variable with a given Key.
-        This operation will overwrite an existing variable.
+    ) -> None:
+        """Set a value for an Airflow Variable with a given Key.
+
+        This operation overwrites an existing variable.
 
         :param key: Variable Key
         :param value: Value to set for the Variable
         :param description: Description of the Variable
         :param serialize_json: Serialize the value to a JSON string
-        :param session: SQL Alchemy Sessions
         """
         # check if the secret exists in the custom secrets' backend.
         Variable.check_for_write_conflict(key)
@@ -179,6 +176,10 @@ class Variable(Base, LoggingMixin):
         Variable.delete(key, session=session)
         session.add(Variable(key=key, val=stored_value, description=description))
         session.flush()
+        # invalidate key in cache for faster propagation
+        # we cannot save the value set because it's possible that it's shadowed by a custom backend
+        # (see call to check_for_write_conflict above)
+        SecretCache.invalidate_variable(key)
 
     @staticmethod
     @provide_session
@@ -188,14 +189,12 @@ class Variable(Base, LoggingMixin):
         value: Any,
         serialize_json: bool = False,
         session: Session = None,
-    ):
-        """
-        Updates a given Airflow Variable with the Provided value
+    ) -> None:
+        """Update a given Airflow Variable with the Provided value.
 
         :param key: Variable Key
         :param value: Value to set for the Variable
         :param serialize_json: Serialize the value to a JSON string
-        :param session: SQL Alchemy Session
         """
         Variable.check_for_write_conflict(key)
 
@@ -212,24 +211,23 @@ class Variable(Base, LoggingMixin):
     @provide_session
     @internal_api_call
     def delete(key: str, session: Session = None) -> int:
-        """
-        Delete an Airflow Variable for a given key
+        """Delete an Airflow Variable for a given key.
 
-        :param key: Variable Key
-        :param session: SQL Alchemy Sessions
+        :param key: Variable Keys
         """
-        return session.query(Variable).filter(Variable.key == key).delete()
+        rows = session.execute(delete(Variable).where(Variable.key == key)).rowcount
+        SecretCache.invalidate_variable(key)
+        return rows
 
     def rotate_fernet_key(self):
-        """Rotate Fernet Key"""
+        """Rotate Fernet Key."""
         fernet = get_fernet()
         if self._val and self.is_encrypted:
             self._val = fernet.rotate(self._val.encode("utf-8")).decode()
 
     @staticmethod
     def check_for_write_conflict(key: str) -> None:
-        """
-        Logs a warning if a variable exists outside of the metastore.
+        """Log a warning if a variable exists outside the metastore.
 
         If we try to write a variable to the metastore while the same key
         exists in an environment variable or custom secrets backend, then
@@ -265,15 +263,26 @@ class Variable(Base, LoggingMixin):
         :param key: Variable Key
         :return: Variable Value
         """
+        # check cache first
+        # enabled only if SecretCache.init() has been called first
+        try:
+            return SecretCache.get_variable(key)
+        except SecretCache.NotPresentException:
+            pass  # continue business
+
+        var_val = None
+        # iterate over backends if not in cache (or expired)
         for secrets_backend in ensure_secrets_loaded():
             try:
                 var_val = secrets_backend.get_variable(key=key)
                 if var_val is not None:
-                    return var_val
+                    break
             except Exception:
                 log.exception(
                     "Unable to retrieve variable from secrets backend (%s). "
                     "Checking subsequent secrets backend.",
                     type(secrets_backend).__name__,
                 )
-        return None
+
+        SecretCache.save_variable(key, var_val)  # we save None as well
+        return var_val
