@@ -25,8 +25,10 @@ operators talk to the
 or the ``api/2.1/jobs/runs/submit``
 `endpoint <https://docs.databricks.com/dev-tools/api/latest/jobs.html#operation/JobsRunsSubmit>`_.
 """
+from __future__ import annotations
+
 import json
-from typing import Any, Dict, List, Optional
+from typing import Any
 
 from requests import exceptions as requests_exceptions
 
@@ -37,27 +39,32 @@ RESTART_CLUSTER_ENDPOINT = ("POST", "api/2.0/clusters/restart")
 START_CLUSTER_ENDPOINT = ("POST", "api/2.0/clusters/start")
 TERMINATE_CLUSTER_ENDPOINT = ("POST", "api/2.0/clusters/delete")
 
-RUN_NOW_ENDPOINT = ('POST', 'api/2.1/jobs/run-now')
-SUBMIT_RUN_ENDPOINT = ('POST', 'api/2.1/jobs/runs/submit')
-GET_RUN_ENDPOINT = ('GET', 'api/2.1/jobs/runs/get')
-CANCEL_RUN_ENDPOINT = ('POST', 'api/2.1/jobs/runs/cancel')
-OUTPUT_RUNS_JOB_ENDPOINT = ('GET', 'api/2.1/jobs/runs/get-output')
+RUN_NOW_ENDPOINT = ("POST", "api/2.1/jobs/run-now")
+SUBMIT_RUN_ENDPOINT = ("POST", "api/2.1/jobs/runs/submit")
+GET_RUN_ENDPOINT = ("GET", "api/2.1/jobs/runs/get")
+CANCEL_RUN_ENDPOINT = ("POST", "api/2.1/jobs/runs/cancel")
+DELETE_RUN_ENDPOINT = ("POST", "api/2.1/jobs/runs/delete")
+REPAIR_RUN_ENDPOINT = ("POST", "api/2.1/jobs/runs/repair")
+OUTPUT_RUNS_JOB_ENDPOINT = ("GET", "api/2.1/jobs/runs/get-output")
+CANCEL_ALL_RUNS_ENDPOINT = ("POST", "api/2.1/jobs/runs/cancel-all")
 
-INSTALL_LIBS_ENDPOINT = ('POST', 'api/2.0/libraries/install')
-UNINSTALL_LIBS_ENDPOINT = ('POST', 'api/2.0/libraries/uninstall')
+INSTALL_LIBS_ENDPOINT = ("POST", "api/2.0/libraries/install")
+UNINSTALL_LIBS_ENDPOINT = ("POST", "api/2.0/libraries/uninstall")
 
-LIST_JOBS_ENDPOINT = ('GET', 'api/2.1/jobs/list')
+LIST_JOBS_ENDPOINT = ("GET", "api/2.1/jobs/list")
 
-WORKSPACE_GET_STATUS_ENDPOINT = ('GET', 'api/2.0/workspace/get-status')
+WORKSPACE_GET_STATUS_ENDPOINT = ("GET", "api/2.0/workspace/get-status")
 
-RUN_LIFE_CYCLE_STATES = ['PENDING', 'RUNNING', 'TERMINATING', 'TERMINATED', 'SKIPPED', 'INTERNAL_ERROR']
+RUN_LIFE_CYCLE_STATES = ["PENDING", "RUNNING", "TERMINATING", "TERMINATED", "SKIPPED", "INTERNAL_ERROR"]
+
+SPARK_VERSIONS_ENDPOINT = ("GET", "api/2.0/clusters/spark-versions")
 
 
 class RunState:
     """Utility class for the run state concept of Databricks runs."""
 
     def __init__(
-        self, life_cycle_state: str, result_state: str = '', state_message: str = '', *args, **kwargs
+        self, life_cycle_state: str, result_state: str = "", state_message: str = "", *args, **kwargs
     ) -> None:
         self.life_cycle_state = life_cycle_state
         self.result_state = result_state
@@ -69,17 +76,17 @@ class RunState:
         if self.life_cycle_state not in RUN_LIFE_CYCLE_STATES:
             raise AirflowException(
                 (
-                    'Unexpected life cycle state: {}: If the state has '
-                    'been introduced recently, please check the Databricks user '
-                    'guide for troubleshooting information'
+                    "Unexpected life cycle state: {}: If the state has "
+                    "been introduced recently, please check the Databricks user "
+                    "guide for troubleshooting information"
                 ).format(self.life_cycle_state)
             )
-        return self.life_cycle_state in ('TERMINATED', 'SKIPPED', 'INTERNAL_ERROR')
+        return self.life_cycle_state in ("TERMINATED", "SKIPPED", "INTERNAL_ERROR")
 
     @property
     def is_successful(self) -> bool:
-        """True if the result state is SUCCESS"""
-        return self.result_state == 'SUCCESS'
+        """True if the result state is SUCCESS."""
+        return self.result_state == "SUCCESS"
 
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, RunState):
@@ -97,7 +104,7 @@ class RunState:
         return json.dumps(self.__dict__)
 
     @classmethod
-    def from_json(cls, data: str) -> 'RunState':
+    def from_json(cls, data: str) -> RunState:
         return RunState(**json.loads(data))
 
 
@@ -115,7 +122,7 @@ class DatabricksHook(BaseDatabricksHook):
     :param retry_args: An optional dictionary with arguments passed to ``tenacity.Retrying`` class.
     """
 
-    hook_name = 'Databricks'
+    hook_name = "Databricks"
 
     def __init__(
         self,
@@ -123,9 +130,10 @@ class DatabricksHook(BaseDatabricksHook):
         timeout_seconds: int = 180,
         retry_limit: int = 3,
         retry_delay: float = 1.0,
-        retry_args: Optional[Dict[Any, Any]] = None,
+        retry_args: dict[Any, Any] | None = None,
+        caller: str = "DatabricksHook",
     ) -> None:
-        super().__init__(databricks_conn_id, timeout_seconds, retry_limit, retry_delay, retry_args)
+        super().__init__(databricks_conn_id, timeout_seconds, retry_limit, retry_delay, retry_args, caller)
 
     def run_now(self, json: dict) -> int:
         """
@@ -133,10 +141,9 @@ class DatabricksHook(BaseDatabricksHook):
 
         :param json: The data used in the body of the request to the ``run-now`` endpoint.
         :return: the run_id as an int
-        :rtype: str
         """
         response = self._do_api_call(RUN_NOW_ENDPOINT, json)
-        return response['run_id']
+        return response["run_id"]
 
     def submit_run(self, json: dict) -> int:
         """
@@ -144,46 +151,53 @@ class DatabricksHook(BaseDatabricksHook):
 
         :param json: The data used in the body of the request to the ``submit`` endpoint.
         :return: the run_id as an int
-        :rtype: str
         """
         response = self._do_api_call(SUBMIT_RUN_ENDPOINT, json)
-        return response['run_id']
+        return response["run_id"]
 
-    def list_jobs(self, limit: int = 25, offset: int = 0, expand_tasks: bool = False) -> List[Dict[str, Any]]:
+    def list_jobs(
+        self, limit: int = 25, offset: int = 0, expand_tasks: bool = False, job_name: str | None = None
+    ) -> list[dict[str, Any]]:
         """
         Lists the jobs in the Databricks Job Service.
 
         :param limit: The limit/batch size used to retrieve jobs.
         :param offset: The offset of the first job to return, relative to the most recently created job.
         :param expand_tasks: Whether to include task and cluster details in the response.
+        :param job_name: Optional name of a job to search.
         :return: A list of jobs.
         """
         has_more = True
-        jobs = []
+        all_jobs = []
 
         while has_more:
-            json = {
-                'limit': limit,
-                'offset': offset,
-                'expand_tasks': expand_tasks,
+            payload: dict[str, Any] = {
+                "limit": limit,
+                "expand_tasks": expand_tasks,
+                "offset": offset,
             }
-            response = self._do_api_call(LIST_JOBS_ENDPOINT, json)
-            jobs += response['jobs'] if 'jobs' in response else []
-            has_more = response.get('has_more', False)
+            if job_name:
+                payload["name"] = job_name
+            response = self._do_api_call(LIST_JOBS_ENDPOINT, payload)
+            jobs = response.get("jobs", [])
+            if job_name:
+                all_jobs += [j for j in jobs if j["settings"]["name"] == job_name]
+            else:
+                all_jobs += jobs
+            has_more = response.get("has_more", False)
             if has_more:
-                offset += len(response['jobs'])
+                offset += len(jobs)
 
-        return jobs
+        return all_jobs
 
-    def find_job_id_by_name(self, job_name: str) -> Optional[int]:
+    def find_job_id_by_name(self, job_name: str) -> int | None:
         """
         Finds job id by its name. If there are multiple jobs with the same name, raises AirflowException.
 
         :param job_name: The name of the job to look up.
         :return: The job_id as an int or None if no job was found.
         """
-        all_jobs = self.list_jobs()
-        matching_jobs = [j for j in all_jobs if j['settings']['name'] == job_name]
+        matching_jobs = self.list_jobs(job_name=job_name)
 
         if len(matching_jobs) > 1:
             raise AirflowException(
@@ -193,7 +207,7 @@ class DatabricksHook(BaseDatabricksHook):
         if not matching_jobs:
             return None
         else:
-            return matching_jobs[0]['job_id']
+            return matching_jobs[0]["job_id"]
 
     def get_run_page_url(self, run_id: int) -> str:
         """
@@ -202,19 +216,20 @@ class DatabricksHook(BaseDatabricksHook):
         :param run_id: id of the run
         :return: URL of the run page
         """
-        json = {'run_id': run_id}
+        json = {"run_id": run_id}
         response = self._do_api_call(GET_RUN_ENDPOINT, json)
-        return response['run_page_url']
+        return response["run_page_url"]
 
     async def a_get_run_page_url(self, run_id: int) -> str:
         """
         Async version of `get_run_page_url()`.
+
         :param run_id: id of the run
         :return: URL of the run page
         """
-        json = {'run_id': run_id}
+        json = {"run_id": run_id}
         response = await self._a_do_api_call(GET_RUN_ENDPOINT, json)
-        return response['run_page_url']
+        return response["run_page_url"]
 
     def get_job_id(self, run_id: int) -> int:
         """
@@ -223,9 +238,9 @@ class DatabricksHook(BaseDatabricksHook):
         :param run_id: id of the run
         :return: Job id for given Databricks run
         """
-        json = {'run_id': run_id}
+        json = {"run_id": run_id}
         response = self._do_api_call(GET_RUN_ENDPOINT, json)
-        return response['job_id']
+        return response["job_id"]
 
     def get_run_state(self, run_id: int) -> RunState:
         """
@@ -242,22 +257,45 @@ class DatabricksHook(BaseDatabricksHook):
         :param run_id: id of the run
         :return: state of the run
         """
-        json = {'run_id': run_id}
+        json = {"run_id": run_id}
         response = self._do_api_call(GET_RUN_ENDPOINT, json)
-        state = response['state']
+        state = response["state"]
         return RunState(**state)
 
     async def a_get_run_state(self, run_id: int) -> RunState:
         """
         Async version of `get_run_state()`.
+
         :param run_id: id of the run
         :return: state of the run
         """
-        json = {'run_id': run_id}
+        json = {"run_id": run_id}
         response = await self._a_do_api_call(GET_RUN_ENDPOINT, json)
-        state = response['state']
+        state = response["state"]
         return RunState(**state)
 
+    def get_run(self, run_id: int) -> dict[str, Any]:
+        """
+        Retrieve run information.
+
+        :param run_id: id of the run
+        :return: state of the run
+        """
+        json = {"run_id": run_id}
+        response = self._do_api_call(GET_RUN_ENDPOINT, json)
+        return response
+
+    async def a_get_run(self, run_id: int) -> dict[str, Any]:
+        """
+        Async version of `get_run`.
+
+        :param run_id: id of the run
+        :return: state of the run
+        """
+        json = {"run_id": run_id}
+        response = await self._a_do_api_call(GET_RUN_ENDPOINT, json)
+        return response
+
     def get_run_state_str(self, run_id: int) -> str:
         """
         Return the string representation of RunState.
@@ -273,7 +311,7 @@ class DatabricksHook(BaseDatabricksHook):
 
     def get_run_state_lifecycle(self, run_id: int) -> str:
         """
-        Returns the lifecycle state of the run
+        Returns the lifecycle state of the run.
 
         :param run_id: id of the run
         :return: string with lifecycle state
@@ -282,7 +320,7 @@ class DatabricksHook(BaseDatabricksHook):
 
     def get_run_state_result(self, run_id: int) -> str:
         """
-        Returns the resulting state of the run
+        Returns the resulting state of the run.
 
         :param run_id: id of the run
         :return: string with resulting state
@@ -291,7 +329,7 @@ class DatabricksHook(BaseDatabricksHook):
 
     def get_run_state_message(self, run_id: int) -> str:
         """
-        Returns the state message for the run
+        Returns the state message for the run.
 
         :param run_id: id of the run
         :return: string with state message
@@ -305,7 +343,7 @@ class DatabricksHook(BaseDatabricksHook):
         :param run_id: id of the run
         :return: output of the run
         """
-        json = {'run_id': run_id}
+        json = {"run_id": run_id}
         run_output = self._do_api_call(OUTPUT_RUNS_JOB_ENDPOINT, json)
         return run_output
 
@@ -315,9 +353,35 @@ class DatabricksHook(BaseDatabricksHook):
 
         :param run_id: id of the run
         """
-        json = {'run_id': run_id}
+        json = {"run_id": run_id}
         self._do_api_call(CANCEL_RUN_ENDPOINT, json)
 
+    def cancel_all_runs(self, job_id: int) -> None:
+        """
+        Cancels all active runs of a job. The runs are canceled asynchronously.
+
+        :param job_id: The canonical identifier of the job to cancel all runs of
+        """
+        json = {"job_id": job_id}
+        self._do_api_call(CANCEL_ALL_RUNS_ENDPOINT, json)
+
+    def delete_run(self, run_id: int) -> None:
+        """
+        Deletes a non-active run.
+
+        :param run_id: id of the run
+        """
+        json = {"run_id": run_id}
+        self._do_api_call(DELETE_RUN_ENDPOINT, json)
+
+    def repair_run(self, json: dict) -> None:
+        """
+        Re-run one or more tasks.
+
+        :param json: repair a job run.
+        """
+        self._do_api_call(REPAIR_RUN_ENDPOINT, json)
+
     def restart_cluster(self, json: dict) -> None:
         """
         Restarts the cluster.
@@ -362,49 +426,63 @@ class DatabricksHook(BaseDatabricksHook):
         """
         self._do_api_call(UNINSTALL_LIBS_ENDPOINT, json)
 
-    def update_repo(self, repo_id: str, json: Dict[str, Any]) -> dict:
+    def update_repo(self, repo_id: str, json: dict[str, Any]) -> dict:
         """
-        Updates given Databricks Repos
+        Updates given Databricks Repos.
 
         :param repo_id: ID of Databricks Repos
         :param json: payload
         :return: metadata from update
         """
-        repos_endpoint = ('PATCH', f'api/2.0/repos/{repo_id}')
+        repos_endpoint = ("PATCH", f"api/2.0/repos/{repo_id}")
         return self._do_api_call(repos_endpoint, json)
 
     def delete_repo(self, repo_id: str):
         """
-        Deletes given Databricks Repos
+        Deletes given Databricks Repos.
 
         :param repo_id: ID of Databricks Repos
         :return:
         """
-        repos_endpoint = ('DELETE', f'api/2.0/repos/{repo_id}')
+        repos_endpoint = ("DELETE", f"api/2.0/repos/{repo_id}")
         self._do_api_call(repos_endpoint)
 
-    def create_repo(self, json: Dict[str, Any]) -> dict:
+    def create_repo(self, json: dict[str, Any]) -> dict:
         """
-        Creates a Databricks Repos
+        Creates a Databricks Repos.
 
         :param json: payload
         :return:
         """
-        repos_endpoint = ('POST', 'api/2.0/repos')
+        repos_endpoint = ("POST", "api/2.0/repos")
         return self._do_api_call(repos_endpoint, json)
 
-    def get_repo_by_path(self, path: str) -> Optional[str]:
+    def get_repo_by_path(self, path: str) -> str | None:
         """
-        Obtains Repos ID by path
+        Obtains Repos ID by path.
+
         :param path: path to a repository
         :return: Repos ID if it exists, None if doesn't.
         """
         try:
-            result = self._do_api_call(WORKSPACE_GET_STATUS_ENDPOINT, {'path': path}, wrap_http_errors=False)
-            if result.get('object_type', '') == 'REPO':
-                return str(result['object_id'])
+            result = self._do_api_call(WORKSPACE_GET_STATUS_ENDPOINT, {"path": path}, wrap_http_errors=False)
+            if result.get("object_type", "") == "REPO":
+                return str(result["object_id"])
         except requests_exceptions.HTTPError as e:
             if e.response.status_code != 404:
                 raise e
 
         return None
+
+    def test_connection(self) -> tuple[bool, str]:
+        """Test the Databricks connectivity from UI."""
+        hook = DatabricksHook(databricks_conn_id=self.databricks_conn_id)
+        try:
+            hook._do_api_call(endpoint_info=SPARK_VERSIONS_ENDPOINT).get("versions")
+            status = True
+            message = "Connection successfully tested"
+        except Exception as e:
+            status = False
+            message = str(e)
+
+        return status, message
