@@ -16,6 +16,8 @@
 # specific language governing permissions and limitations
 # under the License.
 """This module contains a Google Dataflow Hook."""
+from __future__ import annotations
+
 import functools
 import json
 import re
@@ -25,13 +27,18 @@ import time
 import uuid
 import warnings
 from copy import deepcopy
-from typing import Any, Callable, Dict, Generator, List, Optional, Sequence, Set, TypeVar, Union, cast
+from typing import Any, Callable, Generator, Sequence, TypeVar, cast
 
+from google.cloud.dataflow_v1beta3 import GetJobRequest, Job, JobState, JobsV1Beta3AsyncClient, JobView
 from googleapiclient.discovery import build
 
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowException, AirflowProviderDeprecationWarning
 from airflow.providers.apache.beam.hooks.beam import BeamHook, BeamRunnerType, beam_options_to_args
-from airflow.providers.google.common.hooks.base_google import GoogleBaseHook
+from airflow.providers.google.common.hooks.base_google import (
+    PROVIDE_PROJECT_ID,
+    GoogleBaseAsyncHook,
+    GoogleBaseHook,
+)
 from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.timeout import timeout
 
@@ -48,31 +55,27 @@ T = TypeVar("T", bound=Callable)
 
 
 def process_line_and_extract_dataflow_job_id_callback(
-    on_new_job_id_callback: Optional[Callable[[str], None]]
+    on_new_job_id_callback: Callable[[str], None] | None
 ) -> Callable[[str], None]:
-    """
-    Returns callback which triggers function passed as `on_new_job_id_callback` when Dataflow job_id is found.
-    To be used for `process_line_callback` in
-    :py:class:`~airflow.providers.apache.beam.hooks.beam.BeamCommandRunner`
+    """Build callback that triggers the specified function.
+
+    The returned callback is intended to be used as ``process_line_callback`` in
+    :py:class:`~airflow.providers.apache.beam.hooks.beam.BeamCommandRunner`.
 
     :param on_new_job_id_callback: Callback called when the job ID is known
     """
 
-    def _process_line_and_extract_job_id(
-        line: str,
-        # on_new_job_id_callback: Optional[Callable[[str], None]]
-    ) -> None:
+    def _process_line_and_extract_job_id(line: str) -> None:
         # Job id info: https://goo.gl/SE29y9.
+        if on_new_job_id_callback is None:
+            return
         matched_job = JOB_ID_PATTERN.search(line)
-        if matched_job:
-            job_id = matched_job.group("job_id_java") or matched_job.group("job_id_python")
-            if on_new_job_id_callback:
-                on_new_job_id_callback(job_id)
+        if matched_job is None:
+            return
+        job_id = matched_job.group("job_id_java") or matched_job.group("job_id_python")
+        on_new_job_id_callback(job_id)
 
-    def wrap(line: str):
-        return _process_line_and_extract_job_id(line)
-
-    return wrap
+    return _process_line_and_extract_job_id
 
 
 def _fallback_variable_parameter(parameter_name: str, variable_key_name: str) -> Callable[[T], T]:
@@ -85,7 +88,7 @@ def _fallback_variable_parameter(parameter_name: str, variable_key_name: str) ->
         """
 
         @functools.wraps(func)
-        def inner_wrapper(self: "DataflowHook", *args, **kwargs):
+        def inner_wrapper(self: DataflowHook, *args, **kwargs):
             if args:
                 raise AirflowException(
                     "You must use keyword arguments in this methods rather than positional"
@@ -120,6 +123,7 @@ _fallback_to_project_id_from_variables = _fallback_variable_parameter("project_i
 class DataflowJobStatus:
     """
     Helper class with Dataflow job statuses.
+
     Reference: https://cloud.google.com/dataflow/docs/reference/rest/v1b3/projects.jobs#Job.JobState
     """
 
@@ -189,13 +193,13 @@ class _DataflowJobsController(LoggingMixin):
         project_number: str,
         location: str,
         poll_sleep: int = 10,
-        name: Optional[str] = None,
-        job_id: Optional[str] = None,
+        name: str | None = None,
+        job_id: str | None = None,
         num_retries: int = 0,
         multiple_jobs: bool = False,
         drain_pipeline: bool = False,
-        cancel_timeout: Optional[int] = 5 * 60,
-        wait_until_finished: Optional[bool] = None,
+        cancel_timeout: int | None = 5 * 60,
+        wait_until_finished: bool | None = None,
     ) -> None:
 
         super().__init__()
@@ -208,16 +212,15 @@ class _DataflowJobsController(LoggingMixin):
         self._num_retries = num_retries
         self._poll_sleep = poll_sleep
         self._cancel_timeout = cancel_timeout
-        self._jobs: Optional[List[dict]] = None
+        self._jobs: list[dict] | None = None
         self.drain_pipeline = drain_pipeline
         self._wait_until_finished = wait_until_finished
 
     def is_job_running(self) -> bool:
         """
-        Helper method to check if jos is still running in dataflow
+        Helper method to check if jos is still running in dataflow.
 
         :return: True if job is running.
-        :rtype: bool
         """
         self._refresh_jobs()
         if not self._jobs:
@@ -228,15 +231,16 @@ class _DataflowJobsController(LoggingMixin):
                 return True
         return False
 
-    def _get_current_jobs(self) -> List[dict]:
+    def _get_current_jobs(self) -> list[dict]:
         """
-        Helper method to get list of jobs that start with job name or id
+        Helper method to get list of jobs that start with job name or id.
 
         :return: list of jobs including id's
-        :rtype: list
         """
         if not self._multiple_jobs and self._job_id:
             return [self.fetch_job_by_id(self._job_id)]
+        elif self._jobs:
+            return [self.fetch_job_by_id(job["id"]) for job in self._jobs]
         elif self._job_name:
             jobs = self._fetch_jobs_by_prefix_name(self._job_name.lower())
             if len(jobs) == 1:
@@ -251,7 +255,6 @@ class _DataflowJobsController(LoggingMixin):
 
         :param job_id: Job ID to get.
         :return: the Job
-        :rtype: dict
         """
         return (
             self._dataflow.projects()
@@ -272,7 +275,6 @@ class _DataflowJobsController(LoggingMixin):
         :param job_id: Job ID to get.
         :return: the JobMetrics. See:
             https://cloud.google.com/dataflow/docs/reference/rest/v1b3/JobMetrics
-        :rtype: dict
         """
         result = (
             self._dataflow.projects()
@@ -292,7 +294,6 @@ class _DataflowJobsController(LoggingMixin):
         :param job_id: Job ID to get.
         :return: yields the ListJobMessagesResponse. See:
             https://cloud.google.com/dataflow/docs/reference/rest/v1b3/ListJobMessagesResponse
-        :rtype: Generator[dict, None, None]
         """
         request = (
             self._dataflow.projects()
@@ -314,42 +315,40 @@ class _DataflowJobsController(LoggingMixin):
                 .list_next(previous_request=request, previous_response=response)
             )
 
-    def fetch_job_messages_by_id(self, job_id: str) -> List[dict]:
+    def fetch_job_messages_by_id(self, job_id: str) -> list[dict]:
         """
         Helper method to fetch the job messages with the specified Job ID.
 
         :param job_id: Job ID to get.
         :return: the list of JobMessages. See:
             https://cloud.google.com/dataflow/docs/reference/rest/v1b3/ListJobMessagesResponse#JobMessage
-        :rtype: List[dict]
         """
-        messages: List[dict] = []
+        messages: list[dict] = []
         for response in self._fetch_list_job_messages_responses(job_id=job_id):
             messages.extend(response.get("jobMessages", []))
         return messages
 
-    def fetch_job_autoscaling_events_by_id(self, job_id: str) -> List[dict]:
+    def fetch_job_autoscaling_events_by_id(self, job_id: str) -> list[dict]:
         """
         Helper method to fetch the job autoscaling events with the specified Job ID.
 
         :param job_id: Job ID to get.
         :return: the list of AutoscalingEvents. See:
             https://cloud.google.com/dataflow/docs/reference/rest/v1b3/ListJobMessagesResponse#autoscalingevent
-        :rtype: List[dict]
         """
-        autoscaling_events: List[dict] = []
+        autoscaling_events: list[dict] = []
         for response in self._fetch_list_job_messages_responses(job_id=job_id):
             autoscaling_events.extend(response.get("autoscalingEvents", []))
         return autoscaling_events
 
-    def _fetch_all_jobs(self) -> List[dict]:
+    def _fetch_all_jobs(self) -> list[dict]:
         request = (
             self._dataflow.projects()
             .locations()
             .jobs()
             .list(projectId=self._project_number, location=self._job_location)
         )
-        all_jobs: List[dict] = []
+        all_jobs: list[dict] = []
         while request is not None:
             response = request.execute(num_retries=self._num_retries)
             jobs = response.get("jobs")
@@ -365,17 +364,16 @@ class _DataflowJobsController(LoggingMixin):
             )
         return all_jobs
 
-    def _fetch_jobs_by_prefix_name(self, prefix_name: str) -> List[dict]:
+    def _fetch_jobs_by_prefix_name(self, prefix_name: str) -> list[dict]:
         jobs = self._fetch_all_jobs()
         jobs = [job for job in jobs if job["name"].startswith(prefix_name)]
         return jobs
 
     def _refresh_jobs(self) -> None:
         """
-        Helper method to get all jobs by name
+        Helper method to get all jobs by name.
 
         :return: jobs
-        :rtype: list
         """
         self._jobs = self._get_current_jobs()
 
@@ -391,31 +389,29 @@ class _DataflowJobsController(LoggingMixin):
 
     def _check_dataflow_job_state(self, job) -> bool:
         """
-        Helper method to check the state of one job in dataflow for this task
-        if job failed raise exception
+        Helper method to check the state of one job in dataflow for this task if job failed raise exception.
 
         :return: True if job is done.
-        :rtype: bool
         :raise: Exception
         """
         if self._wait_until_finished is None:
-            wait_for_running = job.get('type') == DataflowJobType.JOB_TYPE_STREAMING
+            wait_for_running = job.get("type") == DataflowJobType.JOB_TYPE_STREAMING
         else:
             wait_for_running = not self._wait_until_finished
 
-        if job['currentState'] == DataflowJobStatus.JOB_STATE_DONE:
+        if job["currentState"] == DataflowJobStatus.JOB_STATE_DONE:
             return True
-        elif job['currentState'] == DataflowJobStatus.JOB_STATE_FAILED:
+        elif job["currentState"] == DataflowJobStatus.JOB_STATE_FAILED:
             raise Exception(f"Google Cloud Dataflow job {job['name']} has failed.")
-        elif job['currentState'] == DataflowJobStatus.JOB_STATE_CANCELLED:
+        elif job["currentState"] == DataflowJobStatus.JOB_STATE_CANCELLED:
             raise Exception(f"Google Cloud Dataflow job {job['name']} was cancelled.")
-        elif job['currentState'] == DataflowJobStatus.JOB_STATE_DRAINED:
+        elif job["currentState"] == DataflowJobStatus.JOB_STATE_DRAINED:
             raise Exception(f"Google Cloud Dataflow job {job['name']} was drained.")
-        elif job['currentState'] == DataflowJobStatus.JOB_STATE_UPDATED:
+        elif job["currentState"] == DataflowJobStatus.JOB_STATE_UPDATED:
             raise Exception(f"Google Cloud Dataflow job {job['name']} was updated.")
-        elif job['currentState'] == DataflowJobStatus.JOB_STATE_RUNNING and wait_for_running:
+        elif job["currentState"] == DataflowJobStatus.JOB_STATE_RUNNING and wait_for_running:
             return True
-        elif job['currentState'] in DataflowJobStatus.AWAITING_STATES:
+        elif job["currentState"] in DataflowJobStatus.AWAITING_STATES:
             return self._wait_until_finished is False
         self.log.debug("Current job: %s", str(job))
         raise Exception(f"Google Cloud Dataflow job {job['name']} was unknown state: {job['currentState']}")
@@ -429,13 +425,12 @@ class _DataflowJobsController(LoggingMixin):
             time.sleep(self._poll_sleep)
             self._refresh_jobs()
 
-    def get_jobs(self, refresh: bool = False) -> List[dict]:
+    def get_jobs(self, refresh: bool = False) -> list[dict]:
         """
         Returns Dataflow jobs.
 
         :param refresh: Forces the latest data to be fetched.
         :return: list of jobs
-        :rtype: list
         """
         if not self._jobs or refresh:
             self._refresh_jobs()
@@ -444,20 +439,20 @@ class _DataflowJobsController(LoggingMixin):
 
         return self._jobs
 
-    def _wait_for_states(self, expected_states: Set[str]):
+    def _wait_for_states(self, expected_states: set[str]):
         """Waiting for the jobs to reach a certain state."""
         if not self._jobs:
             raise ValueError("The _jobs should be set")
         while True:
             self._refresh_jobs()
-            job_states = {job['currentState'] for job in self._jobs}
+            job_states = {job["currentState"] for job in self._jobs}
             if not job_states.difference(expected_states):
                 return
-            unexpected_failed_end_states = expected_states - DataflowJobStatus.FAILED_END_STATES
+            unexpected_failed_end_states = DataflowJobStatus.FAILED_END_STATES - expected_states
             if unexpected_failed_end_states.intersection(job_states):
-                unexpected_failed_jobs = {
-                    job for job in self._jobs if job['currentState'] in unexpected_failed_end_states
-                }
+                unexpected_failed_jobs = [
+                    job for job in self._jobs if job["currentState"] in unexpected_failed_end_states
+                ]
                 raise AirflowException(
                     "Jobs failed: "
                     + ", ".join(
@@ -468,19 +463,20 @@ class _DataflowJobsController(LoggingMixin):
             time.sleep(self._poll_sleep)
 
     def cancel(self) -> None:
-        """Cancels or drains current job"""
-        jobs = self.get_jobs()
-        job_ids = [job["id"] for job in jobs if job["currentState"] not in DataflowJobStatus.TERMINAL_STATES]
+        """Cancels or drains current job."""
+        self._jobs = [
+            job for job in self.get_jobs() if job["currentState"] not in DataflowJobStatus.TERMINAL_STATES
+        ]
+        job_ids = [job["id"] for job in self._jobs]
         if job_ids:
-            batch = self._dataflow.new_batch_http_request()
             self.log.info("Canceling jobs: %s", ", ".join(job_ids))
-            for job in jobs:
+            for job in self._jobs:
                 requested_state = (
                     DataflowJobStatus.JOB_STATE_DRAINED
                     if self.drain_pipeline and job["type"] == DataflowJobType.JOB_TYPE_STREAMING
                     else DataflowJobStatus.JOB_STATE_CANCELLED
                 )
-                batch.add(
+                request = (
                     self._dataflow.projects()
                     .locations()
                     .jobs()
@@ -491,14 +487,16 @@ class _DataflowJobsController(LoggingMixin):
                         body={"requestedState": requested_state},
                     )
                 )
-            batch.execute()
+                request.execute(num_retries=self._num_retries)
             if self._cancel_timeout and isinstance(self._cancel_timeout, int):
                 timeout_error_message = (
                     f"Canceling jobs failed due to timeout ({self._cancel_timeout}s): {', '.join(job_ids)}"
                 )
                 tm = timeout(seconds=self._cancel_timeout, error_message=timeout_error_message)
                 with tm:
-                    self._wait_for_states({DataflowJobStatus.JOB_STATE_CANCELLED})
+                    self._wait_for_states(
+                        {DataflowJobStatus.JOB_STATE_CANCELLED, DataflowJobStatus.JOB_STATE_DRAINED}
+                    )
         else:
             self.log.info("No jobs to cancel")
 
@@ -514,22 +512,26 @@ class DataflowHook(GoogleBaseHook):
     def __init__(
         self,
         gcp_conn_id: str = "google_cloud_default",
-        delegate_to: Optional[str] = None,
         poll_sleep: int = 10,
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         drain_pipeline: bool = False,
-        cancel_timeout: Optional[int] = 5 * 60,
-        wait_until_finished: Optional[bool] = None,
+        cancel_timeout: int | None = 5 * 60,
+        wait_until_finished: bool | None = None,
+        **kwargs,
     ) -> None:
+        if kwargs.get("delegate_to") is not None:
+            raise RuntimeError(
+                "The `delegate_to` parameter has been deprecated before and finally removed in this version"
+                " of Google Provider. You MUST convert it to `impersonate_chain`"
+            )
         self.poll_sleep = poll_sleep
         self.drain_pipeline = drain_pipeline
         self.cancel_timeout = cancel_timeout
         self.wait_until_finished = wait_until_finished
-        self.job_id: Optional[str] = None
+        self.job_id: str | None = None
         self.beam_hook = BeamHook(BeamRunnerType.DataflowRunner)
         super().__init__(
             gcp_conn_id=gcp_conn_id,
-            delegate_to=delegate_to,
             impersonation_chain=impersonation_chain,
         )
 
@@ -547,10 +549,10 @@ class DataflowHook(GoogleBaseHook):
         variables: dict,
         jar: str,
         project_id: str,
-        job_class: Optional[str] = None,
+        job_class: str | None = None,
         append_job_name: bool = True,
         multiple_jobs: bool = False,
-        on_new_job_id_callback: Optional[Callable[[str], None]] = None,
+        on_new_job_id_callback: Callable[[str], None] | None = None,
         location: str = DEFAULT_DATAFLOW_LOCATION,
     ) -> None:
         """
@@ -573,7 +575,7 @@ class DataflowHook(GoogleBaseHook):
             to start pipeline and `providers.google.cloud.hooks.dataflow.DataflowHook.wait_for_done`
             to wait for the required pipeline state.
             """,
-            DeprecationWarning,
+            AirflowProviderDeprecationWarning,
             stacklevel=3,
         )
 
@@ -610,10 +612,10 @@ class DataflowHook(GoogleBaseHook):
         dataflow_template: str,
         project_id: str,
         append_job_name: bool = True,
-        on_new_job_id_callback: Optional[Callable[[str], None]] = None,
-        on_new_job_callback: Optional[Callable[[dict], None]] = None,
+        on_new_job_id_callback: Callable[[str], None] | None = None,
+        on_new_job_callback: Callable[[dict], None] | None = None,
         location: str = DEFAULT_DATAFLOW_LOCATION,
-        environment: Optional[dict] = None,
+        environment: dict | None = None,
     ) -> dict:
         """
         Starts Dataflow template job.
@@ -644,36 +646,10 @@ class DataflowHook(GoogleBaseHook):
         """
         name = self.build_dataflow_job_name(job_name, append_job_name)
 
-        environment = environment or {}
-        # available keys for runtime environment are listed here:
-        # https://cloud.google.com/dataflow/docs/reference/rest/v1b3/RuntimeEnvironment
-        environment_keys = [
-            "numWorkers",
-            "maxWorkers",
-            "zone",
-            "serviceAccountEmail",
-            "tempLocation",
-            "bypassTempDirValidation",
-            "machineType",
-            "additionalExperiments",
-            "network",
-            "subnetwork",
-            "additionalUserLabels",
-            "kmsKeyName",
-            "ipConfiguration",
-            "workerRegion",
-            "workerZone",
-        ]
-
-        for key in variables:
-            if key in environment_keys:
-                if key in environment:
-                    self.log.warning(
-                        "'%s' parameter in 'variables' will override of "
-                        "the same one passed in 'environment'!",
-                        key,
-                    )
-                environment.update({key: variables[key]})
+        environment = self._update_environment(
+            variables=variables,
+            environment=environment,
+        )
 
         service = self.get_conn()
 
@@ -699,7 +675,7 @@ class DataflowHook(GoogleBaseHook):
         if on_new_job_id_callback:
             warnings.warn(
                 "on_new_job_id_callback is Deprecated. Please start using on_new_job_callback",
-                DeprecationWarning,
+                AirflowProviderDeprecationWarning,
                 stacklevel=3,
             )
             on_new_job_id_callback(job.get("id"))
@@ -722,17 +698,51 @@ class DataflowHook(GoogleBaseHook):
         jobs_controller.wait_for_done()
         return response["job"]
 
+    def _update_environment(self, variables: dict, environment: dict | None = None) -> dict:
+        environment = environment or {}
+        # available keys for runtime environment are listed here:
+        # https://cloud.google.com/dataflow/docs/reference/rest/v1b3/RuntimeEnvironment
+        environment_keys = {
+            "numWorkers",
+            "maxWorkers",
+            "zone",
+            "serviceAccountEmail",
+            "tempLocation",
+            "bypassTempDirValidation",
+            "machineType",
+            "additionalExperiments",
+            "network",
+            "subnetwork",
+            "additionalUserLabels",
+            "kmsKeyName",
+            "ipConfiguration",
+            "workerRegion",
+            "workerZone",
+        }
+
+        def _check_one(key, val):
+            if key in environment:
+                self.log.warning(
+                    "%r parameter in 'variables' will override the same one passed in 'environment'!",
+                    key,
+                )
+            return key, val
+
+        environment.update(_check_one(key, val) for key, val in variables.items() if key in environment_keys)
+
+        return environment
+
     @GoogleBaseHook.fallback_to_default_project_id
     def start_flex_template(
         self,
         body: dict,
         location: str,
         project_id: str,
-        on_new_job_id_callback: Optional[Callable[[str], None]] = None,
-        on_new_job_callback: Optional[Callable[[dict], None]] = None,
-    ):
+        on_new_job_id_callback: Callable[[str], None] | None = None,
+        on_new_job_callback: Callable[[dict], None] | None = None,
+    ) -> dict:
         """
-        Starts flex templates with the Dataflow  pipeline.
+        Starts flex templates with the Dataflow pipeline.
 
         :param body: The request body. See:
             https://cloud.google.com/dataflow/docs/reference/rest/v1b3/projects.locations.flexTemplates/launch#request-body
@@ -756,7 +766,7 @@ class DataflowHook(GoogleBaseHook):
         if on_new_job_id_callback:
             warnings.warn(
                 "on_new_job_id_callback is Deprecated. Please start using on_new_job_callback",
-                DeprecationWarning,
+                AirflowProviderDeprecationWarning,
                 stacklevel=3,
             )
             on_new_job_id_callback(job.get("id"))
@@ -786,13 +796,13 @@ class DataflowHook(GoogleBaseHook):
         job_name: str,
         variables: dict,
         dataflow: str,
-        py_options: List[str],
+        py_options: list[str],
         project_id: str,
         py_interpreter: str = "python3",
-        py_requirements: Optional[List[str]] = None,
+        py_requirements: list[str] | None = None,
         py_system_site_packages: bool = False,
         append_job_name: bool = True,
-        on_new_job_id_callback: Optional[Callable[[str], None]] = None,
+        on_new_job_id_callback: Callable[[str], None] | None = None,
         location: str = DEFAULT_DATAFLOW_LOCATION,
     ):
         """
@@ -830,7 +840,7 @@ class DataflowHook(GoogleBaseHook):
             to start pipeline and `providers.google.cloud.hooks.dataflow.DataflowHook.wait_for_done`
             to wait for the required pipeline state.
             """,
-            DeprecationWarning,
+            AirflowProviderDeprecationWarning,
             stacklevel=3,
         )
 
@@ -881,23 +891,22 @@ class DataflowHook(GoogleBaseHook):
         name: str,
         project_id: str,
         location: str = DEFAULT_DATAFLOW_LOCATION,
-        variables: Optional[dict] = None,
+        variables: dict | None = None,
     ) -> bool:
         """
-        Helper method to check if jos is still running in dataflow
+        Helper method to check if jos is still running in dataflow.
 
         :param name: The name of the job.
         :param project_id: Optional, the Google Cloud project ID in which to start a job.
             If set to None or missing, the default project_id from the Google Cloud connection is used.
         :param location: Job location.
         :return: True if job is running.
-        :rtype: bool
         """
         if variables:
             warnings.warn(
                 "The variables parameter has been deprecated. You should pass location using "
                 "the location parameter.",
-                DeprecationWarning,
+                AirflowProviderDeprecationWarning,
                 stacklevel=4,
             )
         jobs_controller = _DataflowJobsController(
@@ -916,8 +925,8 @@ class DataflowHook(GoogleBaseHook):
     def cancel_job(
         self,
         project_id: str,
-        job_name: Optional[str] = None,
-        job_id: Optional[str] = None,
+        job_name: str | None = None,
+        job_id: str | None = None,
         location: str = DEFAULT_DATAFLOW_LOCATION,
     ) -> None:
         """
@@ -949,11 +958,11 @@ class DataflowHook(GoogleBaseHook):
         self,
         job_name: str,
         query: str,
-        options: Dict[str, Any],
+        options: dict[str, Any],
         project_id: str,
         location: str = DEFAULT_DATAFLOW_LOCATION,
-        on_new_job_id_callback: Optional[Callable[[str], None]] = None,
-        on_new_job_callback: Optional[Callable[[dict], None]] = None,
+        on_new_job_id_callback: Callable[[str], None] | None = None,
+        on_new_job_callback: Callable[[dict], None] | None = None,
     ):
         """
         Starts Dataflow SQL query.
@@ -1026,7 +1035,7 @@ class DataflowHook(GoogleBaseHook):
         if on_new_job_id_callback:
             warnings.warn(
                 "on_new_job_id_callback is Deprecated. Please start using on_new_job_callback",
-                DeprecationWarning,
+                AirflowProviderDeprecationWarning,
                 stacklevel=3,
             )
             on_new_job_id_callback(cast(str, job.get("id")))
@@ -1041,7 +1050,7 @@ class DataflowHook(GoogleBaseHook):
     def get_job(
         self,
         job_id: str,
-        project_id: str,
+        project_id: str = PROVIDE_PROJECT_ID,
         location: str = DEFAULT_DATAFLOW_LOCATION,
     ) -> dict:
         """
@@ -1053,7 +1062,6 @@ class DataflowHook(GoogleBaseHook):
         :param location: The location of the Dataflow job (for example europe-west1). See:
             https://cloud.google.com/dataflow/docs/concepts/regional-endpoints
         :return: the Job
-        :rtype: dict
         """
         jobs_controller = _DataflowJobsController(
             dataflow=self.get_conn(),
@@ -1079,7 +1087,6 @@ class DataflowHook(GoogleBaseHook):
             https://cloud.google.com/dataflow/docs/concepts/regional-endpoints
         :return: the JobMetrics. See:
             https://cloud.google.com/dataflow/docs/reference/rest/v1b3/JobMetrics
-        :rtype: dict
         """
         jobs_controller = _DataflowJobsController(
             dataflow=self.get_conn(),
@@ -1094,7 +1101,7 @@ class DataflowHook(GoogleBaseHook):
         job_id: str,
         project_id: str,
         location: str = DEFAULT_DATAFLOW_LOCATION,
-    ) -> List[dict]:
+    ) -> list[dict]:
         """
         Gets the job messages with the specified Job ID.
 
@@ -1104,7 +1111,6 @@ class DataflowHook(GoogleBaseHook):
         :param location: Job location.
         :return: the list of JobMessages. See:
             https://cloud.google.com/dataflow/docs/reference/rest/v1b3/ListJobMessagesResponse#JobMessage
-        :rtype: List[dict]
         """
         jobs_controller = _DataflowJobsController(
             dataflow=self.get_conn(),
@@ -1119,7 +1125,7 @@ class DataflowHook(GoogleBaseHook):
         job_id: str,
         project_id: str,
         location: str = DEFAULT_DATAFLOW_LOCATION,
-    ) -> List[dict]:
+    ) -> list[dict]:
         """
         Gets the job autoscaling events with the specified Job ID.
 
@@ -1129,7 +1135,6 @@ class DataflowHook(GoogleBaseHook):
         :param location: Job location.
         :return: the list of AutoscalingEvents. See:
             https://cloud.google.com/dataflow/docs/reference/rest/v1b3/ListJobMessagesResponse#autoscalingevent
-        :rtype: List[dict]
         """
         jobs_controller = _DataflowJobsController(
             dataflow=self.get_conn(),
@@ -1144,7 +1149,7 @@ class DataflowHook(GoogleBaseHook):
         job_name: str,
         location: str,
         project_id: str,
-        job_id: Optional[str] = None,
+        job_id: str | None = None,
         multiple_jobs: bool = False,
     ) -> None:
         """
@@ -1173,3 +1178,96 @@ class DataflowHook(GoogleBaseHook):
             wait_until_finished=self.wait_until_finished,
         )
         job_controller.wait_for_done()
+
+
+class AsyncDataflowHook(GoogleBaseAsyncHook):
+    """Async hook class for dataflow service."""
+
+    sync_hook_class = DataflowHook
+
+    def __init__(self, **kwargs):
+        if kwargs.get("delegate_to") is not None:
+            raise RuntimeError(
+                "The `delegate_to` parameter has been deprecated before and finally removed in this version"
+                " of Google Provider. You MUST convert it to `impersonate_chain`"
+            )
+        super().__init__(**kwargs)
+
+    async def initialize_client(self, client_class):
+        """
+        Initialize object of the given class.
+
+        Method is used to initialize asynchronous client. Because of the big amount of the classes which are
+        used for Dataflow service it was decided to initialize them the same way with credentials which are
+        received from the method of the GoogleBaseHook class.
+        :param client_class: Class of the Google cloud SDK
+        """
+        credentials = (await self.get_sync_hook()).get_credentials()
+        return client_class(
+            credentials=credentials,
+        )
+
+    async def get_project_id(self) -> str:
+        project_id = (await self.get_sync_hook()).project_id
+        return project_id
+
+    async def get_job(
+        self,
+        job_id: str,
+        project_id: str = PROVIDE_PROJECT_ID,
+        job_view: int = JobView.JOB_VIEW_SUMMARY,
+        location: str = DEFAULT_DATAFLOW_LOCATION,
+    ) -> Job:
+        """
+        Gets the job with the specified Job ID.
+
+        :param job_id: Job ID to get.
+        :param project_id: the Google Cloud project ID in which to start a job.
+            If set to None or missing, the default project_id from the Google Cloud connection is used.
+        :param job_view: Optional. JobView object which determines representation of the returned data
+        :param location: Optional. The location of the Dataflow job (for example europe-west1). See:
+            https://cloud.google.com/dataflow/docs/concepts/regional-endpoints
+        """
+        project_id = project_id or (await self.get_project_id())
+        client = await self.initialize_client(JobsV1Beta3AsyncClient)
+
+        request = GetJobRequest(
+            dict(
+                project_id=project_id,
+                job_id=job_id,
+                view=job_view,
+                location=location,
+            )
+        )
+
+        job = await client.get_job(
+            request=request,
+        )
+
+        return job
+
+    async def get_job_status(
+        self,
+        job_id: str,
+        project_id: str = PROVIDE_PROJECT_ID,
+        job_view: int = JobView.JOB_VIEW_SUMMARY,
+        location: str = DEFAULT_DATAFLOW_LOCATION,
+    ) -> JobState:
+        """
+        Gets the job status with the specified Job ID.
+
+        :param job_id: Job ID to get.
+        :param project_id: the Google Cloud project ID in which to start a job.
+            If set to None or missing, the default project_id from the Google Cloud connection is used.
+        :param job_view: Optional. JobView object which determines representation of the returned data
+        :param location: Optional. The location of the Dataflow job (for example europe-west1). See:
+            https://cloud.google.com/dataflow/docs/concepts/regional-endpoints
+        """
+        job = await self.get_job(
+            project_id=project_id,
+            job_id=job_id,
+            job_view=job_view,
+            location=location,
+        )
+        state = job.current_state
+        return state
