@@ -15,10 +15,10 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-#
 """This module contains a Google Cloud Dataproc Metastore hook."""
+from __future__ import annotations
 
-from typing import Any, Dict, Optional, Sequence, Tuple, Union
+from typing import Any, Sequence
 
 from google.api_core.client_options import ClientOptions
 from google.api_core.gapic_v1.method import DEFAULT, _MethodDefault
@@ -37,15 +37,32 @@ from airflow.providers.google.common.hooks.base_google import GoogleBaseHook
 class DataprocMetastoreHook(GoogleBaseHook):
     """Hook for Google Cloud Dataproc Metastore APIs."""
 
+    def __init__(self, **kwargs):
+        if kwargs.get("delegate_to") is not None:
+            raise RuntimeError(
+                "The `delegate_to` parameter has been deprecated before and finally removed in this version"
+                " of Google Provider. You MUST convert it to `impersonate_chain`"
+            )
+        super().__init__(**kwargs)
+
     def get_dataproc_metastore_client(self) -> DataprocMetastoreClient:
         """Returns DataprocMetastoreClient."""
-        client_options = ClientOptions(api_endpoint='metastore.googleapis.com:443')
+        client_options = ClientOptions(api_endpoint="metastore.googleapis.com:443")
 
         return DataprocMetastoreClient(
             credentials=self.get_credentials(), client_info=CLIENT_INFO, client_options=client_options
         )
 
-    def wait_for_operation(self, timeout: Optional[float], operation: Operation):
+    def get_dataproc_metastore_client_v1beta(self):
+        """Returns DataprocMetastoreClient (from v1 beta)."""
+        from google.cloud.metastore_v1beta import DataprocMetastoreClient
+
+        client_options = ClientOptions(api_endpoint="metastore.googleapis.com:443")
+        return DataprocMetastoreClient(
+            credentials=self.get_credentials(), client_info=CLIENT_INFO, client_options=client_options
+        )
+
+    def wait_for_operation(self, timeout: float | None, operation: Operation):
         """Waits for long-lasting operation to complete."""
         try:
             return operation.result(timeout=timeout)
@@ -59,12 +76,12 @@ class DataprocMetastoreHook(GoogleBaseHook):
         project_id: str,
         region: str,
         service_id: str,
-        backup: Union[Dict[Any, Any], Backup],
+        backup: dict[Any, Any] | Backup,
         backup_id: str,
-        request_id: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        request_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ):
         """
         Creates a new backup in a given project and location.
@@ -94,15 +111,15 @@ class DataprocMetastoreHook(GoogleBaseHook):
         :param timeout: The timeout for this request.
         :param metadata: Strings which should be sent along with the request as metadata.
         """
-        parent = f'projects/{project_id}/locations/{region}/services/{service_id}'
+        parent = f"projects/{project_id}/locations/{region}/services/{service_id}"
 
         client = self.get_dataproc_metastore_client()
         result = client.create_backup(
             request={
-                'parent': parent,
-                'backup': backup,
-                'backup_id': backup_id,
-                'request_id': request_id,
+                "parent": parent,
+                "backup": backup,
+                "backup_id": backup_id,
+                "request_id": request_id,
             },
             retry=retry,
             timeout=timeout,
@@ -116,12 +133,12 @@ class DataprocMetastoreHook(GoogleBaseHook):
         project_id: str,
         region: str,
         service_id: str,
-        metadata_import: MetadataImport,
+        metadata_import: dict | MetadataImport,
         metadata_import_id: str,
-        request_id: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        request_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ):
         """
         Creates a new MetadataImport in a given project and location.
@@ -152,15 +169,15 @@ class DataprocMetastoreHook(GoogleBaseHook):
         :param timeout: The timeout for this request.
         :param metadata: Strings which should be sent along with the request as metadata.
         """
-        parent = f'projects/{project_id}/locations/{region}/services/{service_id}'
+        parent = f"projects/{project_id}/locations/{region}/services/{service_id}"
 
         client = self.get_dataproc_metastore_client()
         result = client.create_metadata_import(
             request={
-                'parent': parent,
-                'metadata_import': metadata_import,
-                'metadata_import_id': metadata_import_id,
-                'request_id': request_id,
+                "parent": parent,
+                "metadata_import": metadata_import,
+                "metadata_import_id": metadata_import_id,
+                "request_id": request_id,
             },
             retry=retry,
             timeout=timeout,
@@ -173,12 +190,12 @@ class DataprocMetastoreHook(GoogleBaseHook):
         self,
         region: str,
         project_id: str,
-        service: Union[Dict, Service],
+        service: dict | Service,
         service_id: str,
-        request_id: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        request_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ):
         """
         Creates a metastore service in a project and location.
@@ -202,15 +219,15 @@ class DataprocMetastoreHook(GoogleBaseHook):
         :param timeout: The timeout for this request.
         :param metadata: Strings which should be sent along with the request as metadata.
         """
-        parent = f'projects/{project_id}/locations/{region}'
+        parent = f"projects/{project_id}/locations/{region}"
 
         client = self.get_dataproc_metastore_client()
         result = client.create_service(
             request={
-                'parent': parent,
-                'service_id': service_id,
-                'service': service if service else {},
-                'request_id': request_id,
+                "parent": parent,
+                "service_id": service_id,
+                "service": service if service else {},
+                "request_id": request_id,
             },
             retry=retry,
             timeout=timeout,
@@ -225,10 +242,10 @@ class DataprocMetastoreHook(GoogleBaseHook):
         region: str,
         service_id: str,
         backup_id: str,
-        request_id: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        request_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ):
         """
         Deletes a single backup.
@@ -253,13 +270,13 @@ class DataprocMetastoreHook(GoogleBaseHook):
         :param timeout: The timeout for this request.
         :param metadata: Strings which should be sent along with the request as metadata.
         """
-        name = f'projects/{project_id}/locations/{region}/services/{service_id}/backups/{backup_id}'
+        name = f"projects/{project_id}/locations/{region}/services/{service_id}/backups/{backup_id}"
 
         client = self.get_dataproc_metastore_client()
         result = client.delete_backup(
             request={
-                'name': name,
-                'request_id': request_id,
+                "name": name,
+                "request_id": request_id,
             },
             retry=retry,
             timeout=timeout,
@@ -273,10 +290,10 @@ class DataprocMetastoreHook(GoogleBaseHook):
         project_id: str,
         region: str,
         service_id: str,
-        request_id: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        request_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ):
         """
         Deletes a single service.
@@ -295,13 +312,13 @@ class DataprocMetastoreHook(GoogleBaseHook):
         :param timeout: The timeout for this request.
         :param metadata: Strings which should be sent along with the request as metadata.
         """
-        name = f'projects/{project_id}/locations/{region}/services/{service_id}'
+        name = f"projects/{project_id}/locations/{region}/services/{service_id}"
 
         client = self.get_dataproc_metastore_client()
         result = client.delete_service(
             request={
-                'name': name,
-                'request_id': request_id,
+                "name": name,
+                "request_id": request_id,
             },
             retry=retry,
             timeout=timeout,
@@ -316,11 +333,11 @@ class DataprocMetastoreHook(GoogleBaseHook):
         project_id: str,
         region: str,
         service_id: str,
-        request_id: Optional[str] = None,
-        database_dump_type: Optional[DatabaseDumpSpec] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        request_id: str | None = None,
+        database_dump_type: DatabaseDumpSpec | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ):
         """
         Exports metadata from a service.
@@ -345,15 +362,15 @@ class DataprocMetastoreHook(GoogleBaseHook):
         :param timeout: The timeout for this request.
         :param metadata: Strings which should be sent along with the request as metadata.
         """
-        service = f'projects/{project_id}/locations/{region}/services/{service_id}'
+        service = f"projects/{project_id}/locations/{region}/services/{service_id}"
 
         client = self.get_dataproc_metastore_client()
         result = client.export_metadata(
             request={
-                'destination_gcs_folder': destination_gcs_folder,
-                'service': service,
-                'request_id': request_id,
-                'database_dump_type': database_dump_type,
+                "destination_gcs_folder": destination_gcs_folder,
+                "service": service,
+                "request_id": request_id,
+                "database_dump_type": database_dump_type,
             },
             retry=retry,
             timeout=timeout,
@@ -367,9 +384,9 @@ class DataprocMetastoreHook(GoogleBaseHook):
         project_id: str,
         region: str,
         service_id: str,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ):
         """
         Gets the details of a single service.
@@ -387,12 +404,12 @@ class DataprocMetastoreHook(GoogleBaseHook):
         :param timeout: The timeout for this request.
         :param metadata: Strings which should be sent along with the request as metadata.
         """
-        name = f'projects/{project_id}/locations/{region}/services/{service_id}'
+        name = f"projects/{project_id}/locations/{region}/services/{service_id}"
 
         client = self.get_dataproc_metastore_client()
         result = client.get_service(
             request={
-                'name': name,
+                "name": name,
             },
             retry=retry,
             timeout=timeout,
@@ -407,9 +424,9 @@ class DataprocMetastoreHook(GoogleBaseHook):
         region: str,
         service_id: str,
         backup_id: str,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> Backup:
         """
         Get backup from a service.
@@ -428,11 +445,11 @@ class DataprocMetastoreHook(GoogleBaseHook):
         :param timeout: The timeout for this request.
         :param metadata: Strings which should be sent along with the request as metadata.
         """
-        backup = f'projects/{project_id}/locations/{region}/services/{service_id}/backups/{backup_id}'
+        backup = f"projects/{project_id}/locations/{region}/services/{service_id}/backups/{backup_id}"
         client = self.get_dataproc_metastore_client()
         result = client.get_backup(
             request={
-                'name': backup,
+                "name": backup,
             },
             retry=retry,
             timeout=timeout,
@@ -446,13 +463,13 @@ class DataprocMetastoreHook(GoogleBaseHook):
         project_id: str,
         region: str,
         service_id: str,
-        page_size: Optional[int] = None,
-        page_token: Optional[str] = None,
-        filter: Optional[str] = None,
-        order_by: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        page_size: int | None = None,
+        page_token: str | None = None,
+        filter: str | None = None,
+        order_by: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ):
         """
         Lists backups in a service.
@@ -489,16 +506,16 @@ class DataprocMetastoreHook(GoogleBaseHook):
         :param timeout: The timeout for this request.
         :param metadata: Strings which should be sent along with the request as metadata.
         """
-        parent = f'projects/{project_id}/locations/{region}/services/{service_id}/backups'
+        parent = f"projects/{project_id}/locations/{region}/services/{service_id}/backups"
 
         client = self.get_dataproc_metastore_client()
         result = client.list_backups(
             request={
-                'parent': parent,
-                'page_size': page_size,
-                'page_token': page_token,
-                'filter': filter,
-                'order_by': order_by,
+                "parent": parent,
+                "page_size": page_size,
+                "page_token": page_token,
+                "filter": filter,
+                "order_by": order_by,
             },
             retry=retry,
             timeout=timeout,
@@ -516,11 +533,11 @@ class DataprocMetastoreHook(GoogleBaseHook):
         backup_region: str,
         backup_service_id: str,
         backup_id: str,
-        restore_type: Optional[Restore] = None,
-        request_id: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        restore_type: Restore | None = None,
+        request_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ):
         """
         Restores a service from a backup.
@@ -550,19 +567,19 @@ class DataprocMetastoreHook(GoogleBaseHook):
         :param timeout: The timeout for this request.
         :param metadata: Strings which should be sent along with the request as metadata.
         """
-        service = f'projects/{project_id}/locations/{region}/services/{service_id}'
+        service = f"projects/{project_id}/locations/{region}/services/{service_id}"
         backup = (
-            f'projects/{backup_project_id}/locations/{backup_region}/services/'
-            f'{backup_service_id}/backups/{backup_id}'
+            f"projects/{backup_project_id}/locations/{backup_region}/services/"
+            f"{backup_service_id}/backups/{backup_id}"
         )
 
         client = self.get_dataproc_metastore_client()
         result = client.restore_service(
             request={
-                'service': service,
-                'backup': backup,
-                'restore_type': restore_type,
-                'request_id': request_id,
+                "service": service,
+                "backup": backup,
+                "restore_type": restore_type,
+                "request_id": request_id,
             },
             retry=retry,
             timeout=timeout,
@@ -576,12 +593,12 @@ class DataprocMetastoreHook(GoogleBaseHook):
         project_id: str,
         region: str,
         service_id: str,
-        service: Union[Dict, Service],
+        service: dict | Service,
         update_mask: FieldMask,
-        request_id: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        request_id: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ):
         """
         Updates the parameters of a single service.
@@ -615,18 +632,64 @@ class DataprocMetastoreHook(GoogleBaseHook):
         """
         client = self.get_dataproc_metastore_client()
 
-        service_name = f'projects/{project_id}/locations/{region}/services/{service_id}'
+        service_name = f"projects/{project_id}/locations/{region}/services/{service_id}"
 
         service["name"] = service_name
 
         result = client.update_service(
             request={
-                'service': service,
-                'update_mask': update_mask,
-                'request_id': request_id,
+                "service": service,
+                "update_mask": update_mask,
+                "request_id": request_id,
             },
             retry=retry,
             timeout=timeout,
             metadata=metadata,
         )
         return result
+
+    @GoogleBaseHook.fallback_to_default_project_id
+    def list_hive_partitions(
+        self,
+        project_id: str,
+        service_id: str,
+        region: str,
+        table: str,
+        partition_names: list[str] | None = None,
+    ) -> Operation:
+        """
+        Lists Hive partitions.
+
+        :param project_id: Optional. The ID of a dbt Cloud project.
+        :param service_id: Required. Dataproc Metastore service id.
+        :param region: Required. The ID of the Google Cloud region that the service belongs to.
+        :param table: Required. Name of the partitioned table
+        :param partition_names: Optional. List of table partitions to wait for.
+            A name of a partition should look like "ds=1", or "a=1/b=2" in case of multiple partitions.
+            Note that you cannot use logical or comparison operators as in HivePartitionSensor.
+            If not specified then the sensor will wait for at least one partition regardless its name.
+        """
+        # Remove duplicates from the `partition_names` and preserve elements order
+        # because dictionaries are ordered since Python 3.7+
+        _partitions = list(dict.fromkeys(partition_names)) if partition_names else []
+
+        query = f"""
+                SELECT *
+                FROM PARTITIONS
+                INNER JOIN TBLS
+                ON PARTITIONS.TBL_ID = TBLS.TBL_ID
+                WHERE
+                    TBLS.TBL_NAME = '{table}'"""
+        if _partitions:
+            query += f"""
+                    AND PARTITIONS.PART_NAME IN ({', '.join(f"'{p}'" for p in _partitions)})"""
+        query += ";"
+
+        client = self.get_dataproc_metastore_client_v1beta()
+        result = client.query_metadata(
+            request={
+                "service": f"projects/{project_id}/locations/{region}/services/{service_id}",
+                "query": query,
+            }
+        )
+        return result
