@@ -34,7 +34,7 @@ if TYPE_CHECKING:
 from google.api_core.exceptions import Conflict
 from google.cloud.exceptions import GoogleCloudError
 
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowException, AirflowProviderDeprecationWarning
 from airflow.providers.google.cloud.hooks.gcs import GCSHook
 from airflow.providers.google.cloud.operators.cloud_base import GoogleCloudBaseOperator
 from airflow.providers.google.common.links.storage import FileDetailsLink, StorageLink
@@ -43,8 +43,10 @@ from airflow.utils import timezone
 
 class GCSCreateBucketOperator(GoogleCloudBaseOperator):
     """
-    Creates a new bucket. Google Cloud Storage uses a flat namespace,
-    so you can't create a bucket with a name that is already in use.
+    Creates a new bucket.
+
+    Google Cloud Storage uses a flat namespace, so you
+    can't create a bucket with a name that is already in use.
 
         .. seealso::
             For more information, see Bucket Naming Guidelines:
@@ -74,9 +76,6 @@ class GCSCreateBucketOperator(GoogleCloudBaseOperator):
     :param project_id: The ID of the Google Cloud Project. (templated)
     :param labels: User-provided labels, in key/value pairs.
     :param gcp_conn_id: (Optional) The connection ID used to connect to Google Cloud.
-    :param delegate_to: The account to impersonate using domain-wide delegation of authority,
-        if any. For this to work, the service account making the request must have
-        domain-wide delegation enabled.
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -122,7 +121,6 @@ class GCSCreateBucketOperator(GoogleCloudBaseOperator):
         project_id: str | None = None,
         labels: dict | None = None,
         gcp_conn_id: str = "google_cloud_default",
-        delegate_to: str | None = None,
         impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
@@ -134,17 +132,11 @@ class GCSCreateBucketOperator(GoogleCloudBaseOperator):
         self.project_id = project_id
         self.labels = labels
         self.gcp_conn_id = gcp_conn_id
-        if delegate_to:
-            warnings.warn(
-                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
-            )
-        self.delegate_to = delegate_to
         self.impersonation_chain = impersonation_chain
 
     def execute(self, context: Context) -> None:
         hook = GCSHook(
             gcp_conn_id=self.gcp_conn_id,
-            delegate_to=self.delegate_to,
             impersonation_chain=self.impersonation_chain,
         )
         StorageLink.persist(
@@ -168,21 +160,18 @@ class GCSCreateBucketOperator(GoogleCloudBaseOperator):
 
 class GCSListObjectsOperator(GoogleCloudBaseOperator):
     """
-    List all objects from the bucket with the given string prefix and delimiter in name.
+    List all objects from the bucket filtered by given string prefix and delimiter in name or match_glob.
 
     This operator returns a python list with the name of objects which can be used by
     XCom in the downstream task.
 
     :param bucket: The Google Cloud Storage bucket to find the objects. (templated)
-    :param prefix: Prefix string which filters objects whose name begin with
-           this prefix. (templated)
-    :param delimiter: The delimiter by which you want to filter the objects. (templated)
-        For e.g to lists the CSV files from in a directory in GCS you would use
+    :param prefix: String or list of strings, which filter objects whose name begins with
+           it/them. (templated)
+    :param delimiter: (Deprecated) The delimiter by which you want to filter the objects. (templated)
+        For example, to list the CSV files from in a directory in GCS you would use
         delimiter='.csv'.
     :param gcp_conn_id: (Optional) The connection ID used to connect to Google Cloud.
-    :param delegate_to: The account to impersonate using domain-wide delegation of authority,
-        if any. For this to work, the service account making the request must have
-        domain-wide delegation enabled.
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -191,6 +180,8 @@ class GCSListObjectsOperator(GoogleCloudBaseOperator):
         If set as a sequence, the identities from the list must grant
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
+    :param match_glob: (Optional) filters objects based on the glob pattern given by the string
+        (e.g, ``'**/*/.json'``)
 
     **Example**:
         The following Operator would list all the Avro files from ``sales/sales-2017``
@@ -200,7 +191,7 @@ class GCSListObjectsOperator(GoogleCloudBaseOperator):
                 task_id='GCS_Files',
                 bucket='data',
                 prefix='sales/sales-2017/',
-                delimiter='.avro',
+                match_glob='**/*/.avro',
                 gcp_conn_id=google_cloud_conn_id
             )
     """
@@ -220,39 +211,47 @@ class GCSListObjectsOperator(GoogleCloudBaseOperator):
         self,
         *,
         bucket: str,
-        prefix: str | None = None,
+        prefix: str | list[str] | None = None,
         delimiter: str | None = None,
         gcp_conn_id: str = "google_cloud_default",
-        delegate_to: str | None = None,
         impersonation_chain: str | Sequence[str] | None = None,
+        match_glob: str | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
         self.bucket = bucket
         self.prefix = prefix
+        if delimiter:
+            warnings.warn(
+                "Usage of 'delimiter' is deprecated, please use 'match_glob' instead",
+                AirflowProviderDeprecationWarning,
+                stacklevel=2,
+            )
         self.delimiter = delimiter
         self.gcp_conn_id = gcp_conn_id
-        if delegate_to:
-            warnings.warn(
-                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
-            )
-        self.delegate_to = delegate_to
         self.impersonation_chain = impersonation_chain
+        self.match_glob = match_glob
 
     def execute(self, context: Context) -> list:
-
         hook = GCSHook(
             gcp_conn_id=self.gcp_conn_id,
-            delegate_to=self.delegate_to,
             impersonation_chain=self.impersonation_chain,
         )
 
-        self.log.info(
-            "Getting list of the files. Bucket: %s; Delimiter: %s; Prefix: %s",
-            self.bucket,
-            self.delimiter,
-            self.prefix,
-        )
+        if self.match_glob:
+            self.log.info(
+                "Getting list of the files. Bucket: %s; MatchGlob: %s; Prefix(es): %s",
+                self.bucket,
+                self.match_glob,
+                self.prefix,
+            )
+        else:
+            self.log.info(
+                "Getting list of the files. Bucket: %s; Delimiter: %s; Prefix(es): %s",
+                self.bucket,
+                self.delimiter,
+                self.prefix,
+            )
 
         StorageLink.persist(
             context=context,
@@ -260,25 +259,21 @@ class GCSListObjectsOperator(GoogleCloudBaseOperator):
             uri=self.bucket,
             project_id=hook.project_id,
         )
-
-        return hook.list(bucket_name=self.bucket, prefix=self.prefix, delimiter=self.delimiter)
+        return hook.list(
+            bucket_name=self.bucket, prefix=self.prefix, delimiter=self.delimiter, match_glob=self.match_glob
+        )
 
 
 class GCSDeleteObjectsOperator(GoogleCloudBaseOperator):
     """
-    Deletes objects from a Google Cloud Storage bucket, either
-    from an explicit list of object names or all objects
-    matching a prefix.
+    Deletes objects from a list or all objects matching a prefix from a Google Cloud Storage bucket.
 
     :param bucket_name: The GCS bucket to delete from
     :param objects: List of objects to delete. These should be the names
         of objects in the bucket, not including gs://bucket/
-    :param prefix: Prefix of objects to delete. All objects matching this
-        prefix in the bucket will be deleted.
+    :param prefix: String or list of strings, which filter objects whose name begin with
+           it/them. (templated)
     :param gcp_conn_id: (Optional) The connection ID used to connect to Google Cloud.
-    :param delegate_to: The account to impersonate using domain-wide delegation of authority,
-        if any. For this to work, the service account making the request must have
-        domain-wide delegation enabled.
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -303,20 +298,13 @@ class GCSDeleteObjectsOperator(GoogleCloudBaseOperator):
         objects: list[str] | None = None,
         prefix: str | None = None,
         gcp_conn_id: str = "google_cloud_default",
-        delegate_to: str | None = None,
         impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
-
         self.bucket_name = bucket_name
         self.objects = objects
         self.prefix = prefix
         self.gcp_conn_id = gcp_conn_id
-        if delegate_to:
-            warnings.warn(
-                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
-            )
-        self.delegate_to = delegate_to
         self.impersonation_chain = impersonation_chain
 
         if objects is None and prefix is None:
@@ -330,15 +318,13 @@ class GCSDeleteObjectsOperator(GoogleCloudBaseOperator):
     def execute(self, context: Context) -> None:
         hook = GCSHook(
             gcp_conn_id=self.gcp_conn_id,
-            delegate_to=self.delegate_to,
             impersonation_chain=self.impersonation_chain,
         )
 
-        if self.objects:
+        if self.objects is not None:
             objects = self.objects
         else:
             objects = hook.list(bucket_name=self.bucket_name, prefix=self.prefix)
-
         self.log.info("Deleting %s objects from %s", len(objects), self.bucket_name)
         for object_name in objects:
             hook.delete(bucket_name=self.bucket_name, object_name=object_name)
@@ -507,11 +493,11 @@ class GCSObjectCreateAclEntryOperator(GoogleCloudBaseOperator):
 
 class GCSFileTransformOperator(GoogleCloudBaseOperator):
     """
-    Copies data from a source GCS location to a temporary location on the
-    local filesystem. Runs a transformation on this file as specified by
-    the transformation script and uploads the output to a destination bucket.
-    If the output bucket is not specified the original file will be
-    overwritten.
+    Copies data from a source GCS location to a temporary location on the local filesystem.
+
+    Runs a transformation on this file as specified by the transformation script
+    and uploads the output to a destination bucket. If the output bucket is not
+    specified the original file will be overwritten.
 
     The locations of the source and the destination files in the local
     filesystem is provided as an first and second arguments to the
@@ -613,6 +599,8 @@ class GCSFileTransformOperator(GoogleCloudBaseOperator):
 
 class GCSTimeSpanFileTransformOperator(GoogleCloudBaseOperator):
     """
+    Copy objects that were modified during a time span, run a transform, and upload results to a bucket.
+
     Determines a list of objects that were added or modified at a GCS source
     location during a specific time-span, copies them to a temporary location
     on the local file system, runs a transform on this file as specified by
@@ -886,12 +874,15 @@ class GCSDeleteBucketOperator(GoogleCloudBaseOperator):
         If set as a sequence, the identities from the list must grant
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
+    :param user_project: (Optional) The identifier of the project to bill for this request.
+        Required for Requester Pays buckets.
     """
 
     template_fields: Sequence[str] = (
         "bucket_name",
         "gcp_conn_id",
         "impersonation_chain",
+        "user_project",
     )
 
     def __init__(
@@ -901,6 +892,7 @@ class GCSDeleteBucketOperator(GoogleCloudBaseOperator):
         force: bool = True,
         gcp_conn_id: str = "google_cloud_default",
         impersonation_chain: str | Sequence[str] | None = None,
+        user_project: str | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -909,10 +901,11 @@ class GCSDeleteBucketOperator(GoogleCloudBaseOperator):
         self.force: bool = force
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
+        self.user_project = user_project
 
     def execute(self, context: Context) -> None:
         hook = GCSHook(gcp_conn_id=self.gcp_conn_id, impersonation_chain=self.impersonation_chain)
-        hook.delete_bucket(bucket_name=self.bucket_name, force=self.force)
+        hook.delete_bucket(bucket_name=self.bucket_name, force=self.force, user_project=self.user_project)
 
 
 class GCSSynchronizeBucketsOperator(GoogleCloudBaseOperator):
@@ -944,9 +937,6 @@ class GCSSynchronizeBucketsOperator(GoogleCloudBaseOperator):
             This option can delete data quickly if you specify the wrong source/destination combination.
 
     :param gcp_conn_id: (Optional) The connection ID used to connect to Google Cloud.
-    :param delegate_to: The account to impersonate using domain-wide delegation of authority,
-        if any. For this to work, the service account making the request must have
-        domain-wide delegation enabled.
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -966,7 +956,6 @@ class GCSSynchronizeBucketsOperator(GoogleCloudBaseOperator):
         "delete_extra_files",
         "allow_overwrite",
         "gcp_conn_id",
-        "delegate_to",
         "impersonation_chain",
     )
     operator_extra_links = (StorageLink(),)
@@ -982,7 +971,6 @@ class GCSSynchronizeBucketsOperator(GoogleCloudBaseOperator):
         delete_extra_files: bool = False,
         allow_overwrite: bool = False,
         gcp_conn_id: str = "google_cloud_default",
-        delegate_to: str | None = None,
         impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
@@ -995,17 +983,11 @@ class GCSSynchronizeBucketsOperator(GoogleCloudBaseOperator):
         self.delete_extra_files = delete_extra_files
         self.allow_overwrite = allow_overwrite
         self.gcp_conn_id = gcp_conn_id
-        if delegate_to:
-            warnings.warn(
-                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
-            )
-        self.delegate_to = delegate_to
         self.impersonation_chain = impersonation_chain
 
     def execute(self, context: Context) -> None:
         hook = GCSHook(
             gcp_conn_id=self.gcp_conn_id,
-            delegate_to=self.delegate_to,
             impersonation_chain=self.impersonation_chain,
         )
         StorageLink.persist(
