@@ -14,16 +14,18 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Hook for HashiCorp Vault"""
+"""Hook for HashiCorp Vault."""
 from __future__ import annotations
 
 import json
 import warnings
+from typing import Any
 
 import hvac
 from hvac.exceptions import VaultError
 from requests import Response
 
+from airflow.exceptions import AirflowProviderDeprecationWarning
 from airflow.hooks.base import BaseHook
 from airflow.providers.hashicorp._internal_client.vault_client import (
     DEFAULT_KUBERNETES_JWT_PATH,
@@ -47,7 +49,7 @@ class VaultHook(BaseHook):
     The mount point should be placed as a path in the URL - similarly to Vault's URL schema:
     This indicates the "path" the secret engine is mounted on. Default id not specified is "secret".
     Note that this ``mount_point`` is not used for authentication if authentication is done via a
-    different engines. Each engine uses it's own engine-specific authentication mount_point.
+    different engines. Each engine uses its own engine-specific authentication mount_point.
 
     The extras in the connection are named the same as the parameters ('kv_engine_version', 'auth_type', ...).
 
@@ -147,7 +149,7 @@ class VaultHook(BaseHook):
                 warnings.warn(
                     """The usage of role_id for AppRole authentication has been deprecated.
                     Please use connection login.""",
-                    DeprecationWarning,
+                    AirflowProviderDeprecationWarning,
                     stacklevel=2,
                 )
             elif self.connection.extra_dejson.get("role_id"):
@@ -155,7 +157,7 @@ class VaultHook(BaseHook):
                 warnings.warn(
                     """The usage of role_id in connection extra for AppRole authentication has been
                     deprecated. Please use connection login.""",
-                    DeprecationWarning,
+                    AirflowProviderDeprecationWarning,
                     stacklevel=2,
                 )
             elif self.connection.login:
@@ -186,20 +188,22 @@ class VaultHook(BaseHook):
             else (None, None)
         )
 
-        key_id = self.connection.extra_dejson.get("key_id")
-        if not key_id:
-            key_id = self.connection.login
-
-        if self.connection.conn_type == "vault":
-            conn_protocol = "http"
-        elif self.connection.conn_type == "vaults":
-            conn_protocol = "https"
-        elif self.connection.conn_type == "http":
-            conn_protocol = "http"
-        elif self.connection.conn_type == "https":
-            conn_protocol = "https"
+        if self.connection.extra_dejson.get("use_tls") is not None:
+            if bool(self.connection.extra_dejson.get("use_tls")):
+                conn_protocol = "https"
+            else:
+                conn_protocol = "http"
         else:
-            raise VaultError("The url schema must be one of ['http', 'https', 'vault', 'vaults' ]")
+            if self.connection.conn_type == "vault":
+                conn_protocol = "http"
+            elif self.connection.conn_type == "vaults":
+                conn_protocol = "https"
+            elif self.connection.conn_type == "http":
+                conn_protocol = "http"
+            elif self.connection.conn_type == "https":
+                conn_protocol = "https"
+            else:
+                raise VaultError("The url schema must be one of ['http', 'https', 'vault', 'vaults' ]")
 
         url = f"{conn_protocol}://{self.connection.host}"
         if self.connection.port:
@@ -209,30 +213,28 @@ class VaultHook(BaseHook):
         mount_point = self.connection.schema if self.connection.schema else "secret"
 
         client_kwargs.update(
-            **dict(
-                url=url,
-                auth_type=auth_type,
-                auth_mount_point=auth_mount_point,
-                mount_point=mount_point,
-                kv_engine_version=kv_engine_version,
-                token=self.connection.password,
-                token_path=token_path,
-                username=self.connection.login,
-                password=self.connection.password,
-                key_id=self.connection.login,
-                secret_id=self.connection.password,
-                role_id=role_id,
-                kubernetes_role=kubernetes_role,
-                kubernetes_jwt_path=kubernetes_jwt_path,
-                gcp_key_path=gcp_key_path,
-                gcp_keyfile_dict=gcp_keyfile_dict,
-                gcp_scopes=gcp_scopes,
-                azure_tenant_id=azure_tenant_id,
-                azure_resource=azure_resource,
-                radius_host=radius_host,
-                radius_secret=self.connection.password,
-                radius_port=radius_port,
-            )
+            url=url,
+            auth_type=auth_type,
+            auth_mount_point=auth_mount_point,
+            mount_point=mount_point,
+            kv_engine_version=kv_engine_version,
+            token=self.connection.password,
+            token_path=token_path,
+            username=self.connection.login,
+            password=self.connection.password,
+            key_id=self.connection.login,
+            secret_id=self.connection.password,
+            role_id=role_id,
+            kubernetes_role=kubernetes_role,
+            kubernetes_jwt_path=kubernetes_jwt_path,
+            gcp_key_path=gcp_key_path,
+            gcp_keyfile_dict=gcp_keyfile_dict,
+            gcp_scopes=gcp_scopes,
+            azure_tenant_id=azure_tenant_id,
+            azure_resource=azure_resource,
+            radius_host=radius_host,
+            radius_secret=self.connection.password,
+            radius_port=radius_port,
         )
 
         self.vault_client = _VaultClient(**client_kwargs)
@@ -361,3 +363,56 @@ class VaultHook(BaseHook):
         return self.vault_client.create_or_update_secret(
             secret_path=secret_path, secret=secret, method=method, cas=cas
         )
+
+    @classmethod
+    def get_connection_form_widgets(cls) -> dict[str, Any]:
+        """Returns connection widgets to add to connection form."""
+        from flask_appbuilder.fieldwidgets import BS3TextFieldWidget
+        from flask_babel import lazy_gettext
+        from wtforms import BooleanField, IntegerField, StringField
+        from wtforms.validators import NumberRange, Optional, any_of
+
+        return {
+            "auth_type": StringField(lazy_gettext("Auth type"), widget=BS3TextFieldWidget()),
+            "auth_mount_point": StringField(lazy_gettext("Auth mount point"), widget=BS3TextFieldWidget()),
+            "kv_engine_version": IntegerField(
+                lazy_gettext("KV engine version"),
+                validators=[any_of([1, 2])],
+                widget=BS3TextFieldWidget(),
+                description="Must be 1 or 2.",
+                default=DEFAULT_KV_ENGINE_VERSION,
+            ),
+            "role_id": StringField(lazy_gettext("Role ID (deprecated)"), widget=BS3TextFieldWidget()),
+            "kubernetes_role": StringField(lazy_gettext("Kubernetes role"), widget=BS3TextFieldWidget()),
+            "kubernetes_jwt_path": StringField(
+                lazy_gettext("Kubernetes jwt path"), widget=BS3TextFieldWidget()
+            ),
+            "token_path": StringField(lazy_gettext("Token path"), widget=BS3TextFieldWidget()),
+            "gcp_key_path": StringField(lazy_gettext("GCP key path"), widget=BS3TextFieldWidget()),
+            "gcp_scopes": StringField(lazy_gettext("GCP scopes"), widget=BS3TextFieldWidget()),
+            "azure_tenant_id": StringField(lazy_gettext("Azure tenant ID"), widget=BS3TextFieldWidget()),
+            "azure_resource": StringField(lazy_gettext("Azure resource"), widget=BS3TextFieldWidget()),
+            "radius_host": StringField(lazy_gettext("Radius host"), widget=BS3TextFieldWidget()),
+            "radius_port": IntegerField(
+                lazy_gettext("Radius port"),
+                widget=BS3TextFieldWidget(),
+                validators=[Optional(), NumberRange(min=0)],
+            ),
+            "use_tls": BooleanField(lazy_gettext("Use TLS"), default=True),
+        }
+
+    @staticmethod
+    def get_ui_field_behaviour() -> dict[str, Any]:
+        """Returns custom field behaviour."""
+        return {
+            "hidden_fields": ["extra"],
+            "relabeling": {},
+        }
+
+    def test_connection(self) -> tuple[bool, str]:
+        """Test Vault connectivity from UI."""
+        try:
+            self.get_conn()
+            return True, "Connection successfully tested"
+        except Exception as e:
+            return False, str(e)
