@@ -56,8 +56,11 @@ class PsrpOperator(BaseOperator):
     :param cmdlet:
         cmdlet to execute on remote host (templated). Also used as the default
         value for `task_id`.
+    :param arguments:
+        When using the `cmdlet` or `powershell` option, use `arguments` to
+        provide arguments (templated).
     :param parameters:
-        When using the `cmdlet` or `powershell` arguments, use this parameter to
+        When using the `cmdlet` or `powershell` option, use `parameters` to
         provide parameters (templated). Note that a parameter with a value of `None`
         becomes an *argument* (i.e., switch).
     :param logging_level:
@@ -79,6 +82,7 @@ class PsrpOperator(BaseOperator):
     template_fields: Sequence[str] = (
         "cmdlet",
         "command",
+        "arguments",
         "parameters",
         "powershell",
     )
@@ -92,6 +96,7 @@ class PsrpOperator(BaseOperator):
         command: str | None = None,
         powershell: str | None = None,
         cmdlet: str | None = None,
+        arguments: list[str] | None = None,
         parameters: dict[str, str] | None = None,
         logging_level: int = DEBUG,
         runspace_options: dict[str, Any] | None = None,
@@ -102,6 +107,8 @@ class PsrpOperator(BaseOperator):
         args = {command, powershell, cmdlet}
         if not exactly_one(*args):
             raise ValueError("Must provide exactly one of 'command', 'powershell', or 'cmdlet'")
+        if arguments and not cmdlet:
+            raise ValueError("Arguments only allowed with 'cmdlet'")
         if parameters and not cmdlet:
             raise ValueError("Parameters only allowed with 'cmdlet'")
         if cmdlet:
@@ -111,6 +118,7 @@ class PsrpOperator(BaseOperator):
         self.command = command
         self.powershell = powershell
         self.cmdlet = cmdlet
+        self.arguments = arguments
         self.parameters = parameters
         self.logging_level = logging_level
         self.runspace_options = runspace_options
@@ -134,6 +142,8 @@ class PsrpOperator(BaseOperator):
                     ps.add_cmdlet(self.cmdlet)
                 else:
                     ps.add_script(self.powershell)
+                for argument in self.arguments or ():
+                    ps.add_argument(argument)
                 if self.parameters:
                     ps.add_parameters(self.parameters)
                 if self.do_xcom_push:
