@@ -22,25 +22,27 @@ RESOURCE_ADMIN_MENU = "Admin"
 RESOURCE_AIRFLOW = "Airflow"
 RESOURCE_AUDIT_LOG = "Audit Logs"
 RESOURCE_BROWSE_MENU = "Browse"
-RESOURCE_DAG = "DAGs"
-RESOURCE_DAG_PREFIX = "DAG:"
-RESOURCE_LOGIN = "Logins"
-RESOURCE_DOCS_MENU = "Docs"
-RESOURCE_DOCS = "Documentation"
 RESOURCE_CONFIG = "Configurations"
 RESOURCE_CONNECTION = "Connections"
-RESOURCE_DAG_DEPENDENCIES = "DAG Dependencies"
+RESOURCE_DAG = "DAGs"
 RESOURCE_DAG_CODE = "DAG Code"
+RESOURCE_DAG_DEPENDENCIES = "DAG Dependencies"
+RESOURCE_DAG_PREFIX = "DAG:"
 RESOURCE_DAG_RUN = "DAG Runs"
-RESOURCE_IMPORT_ERROR = "ImportError"
 RESOURCE_DAG_WARNING = "DAG Warnings"
+RESOURCE_CLUSTER_ACTIVITY = "Cluster Activity"
+RESOURCE_DATASET = "Datasets"
+RESOURCE_DOCS = "Documentation"
+RESOURCE_DOCS_MENU = "Docs"
+RESOURCE_IMPORT_ERROR = "ImportError"
 RESOURCE_JOB = "Jobs"
+RESOURCE_LOGIN = "Logins"
 RESOURCE_MY_PASSWORD = "My Password"
 RESOURCE_MY_PROFILE = "My Profile"
 RESOURCE_PASSWORD = "Passwords"
 RESOURCE_PERMISSION = "Permission Views"  # Refers to a Perm <-> View mapping, not an MVC View.
-RESOURCE_POOL = "Pools"
 RESOURCE_PLUGIN = "Plugins"
+RESOURCE_POOL = "Pools"
 RESOURCE_PROVIDER = "Providers"
 RESOURCE_RESOURCE = "View Menus"
 RESOURCE_ROLE = "Roles"
@@ -54,8 +56,6 @@ RESOURCE_USER_STATS_CHART = "User Stats Chart"
 RESOURCE_VARIABLE = "Variables"
 RESOURCE_WEBSITE = "Website"
 RESOURCE_XCOM = "XComs"
-RESOURCE_DATASET = "Datasets"
-
 
 # Action Constants
 ACTION_CAN_CREATE = "can_create"
@@ -70,7 +70,7 @@ DAG_ACTIONS = {ACTION_CAN_READ, ACTION_CAN_EDIT, ACTION_CAN_DELETE}
 
 
 def resource_name_for_dag(root_dag_id: str) -> str:
-    """Returns the resource name for a DAG id.
+    """Return the resource name for a DAG id.
 
     Note that since a sub-DAG should follow the permission of its
     parent DAG, you should pass ``DagModel.root_dag_id`` to this function,
