@@ -22,6 +22,7 @@ from typing import TYPE_CHECKING
 import attr
 from sqlalchemy.orm.session import Session
 
+from airflow.exceptions import TaskNotFound
 from airflow.utils.state import State
 
 if TYPE_CHECKING:
@@ -84,14 +85,20 @@ class DepContext:
 
     def ensure_finished_tis(self, dag_run: DagRun, session: Session) -> list[TaskInstance]:
         """
-        This method makes sure finished_tis is populated if it's currently None.
-        This is for the strange feature of running tasks without dag_run.
+        Ensure finished_tis is populated if it's currently None, which allows running tasks without dag_run.
 
-        :param dag_run: The DagRun for which to find finished tasks
-        :return: A list of all the finished tasks of this DAG and execution_date
+         :param dag_run: The DagRun for which to find finished tasks
+         :return: A list of all the finished tasks of this DAG and execution_date
         """
         if self.finished_tis is None:
             finished_tis = dag_run.get_task_instances(state=State.finished, session=session)
+            for ti in finished_tis:
+                if not hasattr(ti, "task") and dag_run.dag:
+                    try:
+                        ti.task = dag_run.dag.get_task(ti.task_id)
+                    except TaskNotFound:
+                        pass
+
             self.finished_tis = finished_tis
         else:
             finished_tis = self.finished_tis
