@@ -18,7 +18,6 @@
 """This module defines dep for pool slots availability."""
 from __future__ import annotations
 
-from airflow.ti_deps.dependencies_states import EXECUTION_STATES
 from airflow.ti_deps.deps.base_ti_dep import BaseTIDep
 from airflow.utils.session import provide_session
 
@@ -32,7 +31,7 @@ class PoolSlotsAvailableDep(BaseTIDep):
     @provide_session
     def _get_dep_statuses(self, ti, session, dep_context=None):
         """
-        Determines if the pool task instance is in has available slots.
+        Determine if the pool task instance is in has available slots.
 
         :param ti: the task instance to get the dependency status for
         :param session: database session
@@ -49,12 +48,12 @@ class PoolSlotsAvailableDep(BaseTIDep):
                 reason=f"Tasks using non-existent pool '{pool_name}' will not be scheduled"
             )
             return
-        else:
-            # Controlled by UNIQUE key in slot_pool table,
-            # only one result can be returned.
-            open_slots = pools[0].open_slots(session=session)
+        # Controlled by UNIQUE key in slot_pool table,
+        # only one result can be returned.
+        open_slots = pools[0].open_slots(session=session)
 
-        if ti.state in EXECUTION_STATES:
+        occupied_states = pools[0].get_occupied_states()
+        if ti.state in occupied_states:
             open_slots += ti.pool_slots
 
         if open_slots <= (ti.pool_slots - 1):
