@@ -101,6 +101,7 @@ def dag_edges(dag: DAG):
 
     # Collect all the edges between individual tasks
     edges = set()
+    setup_teardown_edges = set()
 
     tasks_to_trace: list[Operator] = dag.roots
     while tasks_to_trace:
@@ -108,6 +109,8 @@ def dag_edges(dag: DAG):
         for task in tasks_to_trace:
             for child in task.downstream_list:
                 edge = (task.task_id, child.task_id)
+                if task.is_setup and child.is_teardown:
+                    setup_teardown_edges.add(edge)
                 if edge in edges:
                     continue
                 edges.add(edge)
@@ -120,6 +123,8 @@ def dag_edges(dag: DAG):
     for source_id, target_id in sorted(edges.union(edges_to_add) - edges_to_skip):
         record = {"source_id": source_id, "target_id": target_id}
         label = dag.get_edge_info(source_id, target_id).get("label")
+        if (source_id, target_id) in setup_teardown_edges:
+            record["is_setup_teardown"] = True
         if label:
             record["label"] = label
         result.append(record)
