@@ -15,7 +15,9 @@
 # specific language governing permissions and limitations
 # under the License.
 """
-This module took inspiration from the community maintenance dag
+This module took inspiration from the community maintenance dag.
+
+See:
 (https://github.com/teamclairvoyant/airflow-maintenance-dags/blob/4e5c7682a808082561d60cbc9cafaa477b0d8c65/db-cleanup/airflow-db-cleanup.py).
 """
 from __future__ import annotations
@@ -28,7 +30,7 @@ from dataclasses import dataclass
 from typing import Any
 
 from pendulum import DateTime
-from sqlalchemy import and_, column, false, func, inspect, table, text
+from sqlalchemy import and_, column, false, func, inspect, select, table, text
 from sqlalchemy.exc import OperationalError, ProgrammingError
 from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.orm import Query, Session, aliased
@@ -36,6 +38,7 @@ from sqlalchemy.sql.expression import ClauseElement, Executable, tuple_
 
 from airflow import AirflowException
 from airflow.cli.simple_table import AirflowConsole
+from airflow.configuration import conf
 from airflow.models import Base
 from airflow.utils import timezone
 from airflow.utils.db import reflect_tables
@@ -50,7 +53,7 @@ ARCHIVE_TABLE_PREFIX = "_airflow_deleted__"
 @dataclass
 class _TableConfig:
     """
-    Config class for performing cleanup on a table
+    Config class for performing cleanup on a table.
 
     :param table_name: the table
     :param extra_columns: any columns besides recency_column_name that we'll need in queries
@@ -113,6 +116,9 @@ config_list: list[_TableConfig] = [
     _TableConfig(table_name="celery_tasksetmeta", recency_column_name="date_done"),
 ]
 
+if conf.get("webserver", "session_backend") == "database":
+    config_list.append(_TableConfig(table_name="session", recency_column_name="expiry"))
+
 config_dict: dict[str, _TableConfig] = {x.orm_model.name: x for x in sorted(config_list)}
 
 
@@ -141,13 +147,14 @@ def _dump_table_to_file(*, target_table, file_path, export_format, session):
 
 
 def _do_delete(*, query, orm_model, skip_archive, session):
-    import re
     from datetime import datetime
 
+    import re2
+
     print("Performing Delete...")
     # using bulk delete
     # create a new table and copy the rows there
-    timestamp_str = re.sub(r"[^\d]", "", datetime.utcnow().isoformat())[:14]
+    timestamp_str = re2.sub(r"[^\d]", "", datetime.utcnow().isoformat())[:14]
     target_table_name = f"{ARCHIVE_TABLE_PREFIX}{orm_model.name}__{timestamp_str}"
     print(f"Moving data to table {target_table_name}")
     bind = session.get_bind()
@@ -155,7 +162,7 @@ def _do_delete(*, query, orm_model, skip_archive, session):
     if dialect_name == "mysql":
         # MySQL with replication needs this split into two queries, so just do it for all MySQL
         # ERROR 1786 (HY000): Statement violates GTID consistency: CREATE TABLE ... SELECT.
-        session.execute(f"CREATE TABLE {target_table_name} LIKE {orm_model.name}")
+        session.execute(text(f"CREATE TABLE {target_table_name} LIKE {orm_model.name}"))
         metadata = reflect_tables([target_table_name], session)
         target_table = metadata.tables[target_table_name]
         insert_stm = target_table.insert().from_select(target_table.c, query)
@@ -176,7 +183,7 @@ def _do_delete(*, query, orm_model, skip_archive, session):
         pk_cols = source_table.primary_key.columns
         delete = source_table.delete().where(
             tuple_(*pk_cols).in_(
-                session.query(*[target_table.c[x.name] for x in source_table.primary_key.columns]).subquery()
+                select(*[target_table.c[x.name] for x in source_table.primary_key.columns]).subquery()
             )
         )
     else:
@@ -187,13 +194,14 @@ def _do_delete(*, query, orm_model, skip_archive, session):
     session.execute(delete)
     session.commit()
     if skip_archive:
+        metadata.bind = session.get_bind()
         target_table.drop()
     session.commit()
     print("Finished Performing Delete")
 
 
 def _subquery_keep_last(*, recency_column, keep_last_filters, group_by_columns, max_date_colname, session):
-    subquery = session.query(*group_by_columns, func.max(recency_column).label(max_date_colname))
+    subquery = select(*group_by_columns, func.max(recency_column).label(max_date_colname))
 
     if keep_last_filters is not None:
         for entry in keep_last_filters:
@@ -340,6 +348,7 @@ def _print_config(*, configs: dict[str, _TableConfig]):
 def _suppress_with_logging(table, session):
     """
     Suppresses errors but logs them.
+
     Also stores the exception instance so it can be referred to after exiting context.
     """
     try:
