@@ -18,21 +18,21 @@ from __future__ import annotations
 
 import logging
 import time
+from functools import cached_property
 from typing import Iterator
 
 from sqlalchemy.orm.session import Session
 
-from airflow.compat.functools import cached_property
 from airflow.configuration import conf
 from airflow.models.taskinstance import TaskInstance
 from airflow.utils.helpers import render_log_filename
 from airflow.utils.log.logging_mixin import ExternalLoggingMixin
 from airflow.utils.session import NEW_SESSION, provide_session
-from airflow.utils.state import State
+from airflow.utils.state import TaskInstanceState
 
 
 class TaskLogReader:
-    """Task log reader"""
+    """Task log reader."""
 
     STREAM_LOOP_SLEEP_SECONDS = 0.5
     """Time to sleep between loops while waiting for more logs"""
@@ -41,7 +41,7 @@ class TaskLogReader:
         self, ti: TaskInstance, try_number: int | None, metadata
     ) -> tuple[list[tuple[tuple[str, str]]], dict[str, str]]:
         """
-        Reads chunks of Task Instance logs.
+        Read chunks of Task Instance logs.
 
         :param ti: The taskInstance
         :param try_number: If provided, logs for the given try will be returned.
@@ -65,7 +65,7 @@ class TaskLogReader:
 
     def read_log_stream(self, ti: TaskInstance, try_number: int | None, metadata: dict) -> Iterator[str]:
         """
-        Used to continuously read log to the end
+        Continuously read log to the end.
 
         :param ti: The Task Instance
         :param try_number: the task try number
@@ -86,7 +86,8 @@ class TaskLogReader:
                 for host, log in logs[0]:
                     yield "\n".join([host or "", log]) + "\n"
                 if "end_of_log" not in metadata or (
-                    not metadata["end_of_log"] and ti.state not in [State.RUNNING, State.DEFERRED]
+                    not metadata["end_of_log"]
+                    and ti.state not in (TaskInstanceState.RUNNING, TaskInstanceState.DEFERRED)
                 ):
                     if not logs[0]:
                         # we did not receive any logs in this loop
@@ -133,7 +134,7 @@ class TaskLogReader:
         session: Session = NEW_SESSION,
     ) -> str:
         """
-        Renders the log attachment filename
+        Render the log attachment filename.
 
         :param ti: The task instance
         :param try_number: The task try number
