@@ -25,6 +25,8 @@ from airflow.configuration import conf
 from airflow.utils.context import Context
 
 if typing.TYPE_CHECKING:
+    import multiprocessing.context
+
     from airflow.models.operator import Operator
 
 
@@ -33,8 +35,9 @@ class MultiprocessingStartMethodMixin:
 
     def _get_multiprocessing_start_method(self) -> str:
         """
-        Determine method of creating new processes by checking if the
-        mp_start_method is set in configs, else, it uses the OS default.
+        Determine method of creating new processes.
+
+        Checks if the mp_start_method is set in configs, else, it uses the OS default.
         """
         if conf.has_option("core", "mp_start_method"):
             return conf.get_mandatory_value("core", "mp_start_method")
@@ -44,6 +47,10 @@ class MultiprocessingStartMethodMixin:
             raise ValueError("Failed to determine start method")
         return method
 
+    def _get_multiprocessing_context(self) -> multiprocessing.context.DefaultContext:
+        mp_start_method = self._get_multiprocessing_start_method()
+        return multiprocessing.get_context(mp_start_method)  # type: ignore
+
 
 class ResolveMixin:
     """A runtime-resolved value."""
