@@ -19,19 +19,20 @@ from __future__ import annotations
 
 import functools
 import gzip
+import itertools
 import json
 import logging
 from io import BytesIO as IO
-from itertools import chain
 from typing import Callable, TypeVar, cast
 
 import pendulum
-from flask import after_this_request, g, request
+from flask import after_this_request, request
 from pendulum.parsing.exceptions import ParserError
 
 from airflow.models import Log
 from airflow.utils.log import secrets_masker
 from airflow.utils.session import create_session
+from airflow.www.extensions.init_auth_manager import get_auth_manager
 
 T = TypeVar("T", bound=Callable)
 
@@ -40,9 +41,10 @@ logger = logging.getLogger(__name__)
 
 def _mask_variable_fields(extra_fields):
     """
+    Mask the 'val_content' field if 'key_content' is in the mask list.
+
     The variable requests values and args comes in this form:
     [('key', 'key_content'),('val', 'val_content'), ('description', 'description_content')]
-    So we need to mask the 'val_content' field if 'key_content' is in the mask list.
     """
     result = []
     keyname = None
@@ -60,7 +62,7 @@ def _mask_variable_fields(extra_fields):
 
 
 def _mask_connection_fields(extra_fields):
-    """Mask connection fields"""
+    """Mask connection fields."""
     result = []
     for k, v in extra_fields:
         if k == "extra":
@@ -76,7 +78,7 @@ def _mask_connection_fields(extra_fields):
 
 
 def action_logging(func: Callable | None = None, event: str | None = None) -> Callable[[T], T]:
-    """Decorator to log user actions"""
+    """Decorator to log user actions."""
 
     def log_action(f: T) -> T:
         @functools.wraps(f)
@@ -84,15 +86,15 @@ def action_logging(func: Callable | None = None, event: str | None = None) -> Ca
             __tracebackhide__ = True  # Hide from pytest traceback.
 
             with create_session() as session:
-                if g.user.is_anonymous:
+                if not get_auth_manager().is_logged_in():
                     user = "anonymous"
                 else:
-                    user = f"{g.user.username} ({g.user.get_full_name()})"
+                    user = get_auth_manager().get_user_name()
 
                 fields_skip_logging = {"csrf_token", "_csrf_token"}
                 extra_fields = [
                     (k, secrets_masker.redact(v, k))
-                    for k, v in chain(request.values.items(multi=True), request.view_args.items())
+                    for k, v in itertools.chain(request.values.items(multi=True), request.view_args.items())
                     if k not in fields_skip_logging
                 ]
                 if event and event.startswith("variable."):
@@ -100,7 +102,7 @@ def action_logging(func: Callable | None = None, event: str | None = None) -> Ca
                 if event and event.startswith("connection."):
                     extra_fields = _mask_connection_fields(extra_fields)
 
-                params = {k: v for k, v in chain(request.values.items(), request.view_args.items())}
+                params = {k: v for k, v in itertools.chain(request.values.items(), request.view_args.items())}
 
                 log = Log(
                     event=event or f.__name__,
@@ -132,7 +134,7 @@ def action_logging(func: Callable | None = None, event: str | None = None) -> Ca
 
 
 def gzipped(f: T) -> T:
-    """Decorator to make a view compressed"""
+    """Decorator to make a view compressed."""
 
     @functools.wraps(f)
     def view_func(*args, **kwargs):
