@@ -37,8 +37,9 @@ from pendulum.datetime import DateTime
 from pygments import highlight, lexers
 from pygments.formatters import HtmlFormatter
 from pygments.lexer import Lexer
-from sqlalchemy import func, types
+from sqlalchemy import delete, func, select, types
 from sqlalchemy.ext.associationproxy import AssociationProxy
+from sqlalchemy.sql import Select
 
 from airflow.exceptions import RemovedInAirflow3Warning
 from airflow.models import errors
@@ -49,17 +50,19 @@ from airflow.utils import timezone
 from airflow.utils.code_utils import get_python_source
 from airflow.utils.helpers import alchemy_to_dict
 from airflow.utils.json import WebEncoder
+from airflow.utils.sqlalchemy import tuple_in_condition
 from airflow.utils.state import State, TaskInstanceState
 from airflow.www.forms import DateTimeWithTimezoneField
 from airflow.www.widgets import AirflowDateTimePickerWidget
 
 if TYPE_CHECKING:
-    from sqlalchemy.orm.query import Query
     from sqlalchemy.orm.session import Session
     from sqlalchemy.sql.operators import ColumnOperators
 
     from airflow.www.fab_security.sqla.manager import SecurityManager
 
+TI = TaskInstance
+
 
 def datetime_to_string(value: DateTime | None) -> str | None:
     if value is None:
@@ -68,27 +71,31 @@ def datetime_to_string(value: DateTime | None) -> str | None:
 
 
 def get_mapped_instances(task_instance, session):
-    return (
-        session.query(TaskInstance)
-        .filter(
+    return session.scalars(
+        select(TaskInstance)
+        .where(
             TaskInstance.dag_id == task_instance.dag_id,
             TaskInstance.run_id == task_instance.run_id,
             TaskInstance.task_id == task_instance.task_id,
         )
         .order_by(TaskInstance.map_index)
-        .all()
-    )
+    ).all()
 
 
 def get_instance_with_map(task_instance, session):
     if task_instance.map_index == -1:
-        return alchemy_to_dict(task_instance)
+        data = alchemy_to_dict(task_instance)
+        # Fetch execution_date explicitly since it's not a column and a proxy
+        data["execution_date"] = task_instance.execution_date
+        return data
     mapped_instances = get_mapped_instances(task_instance, session)
     return get_mapped_summary(task_instance, mapped_instances)
 
 
 def get_try_count(try_number: int, state: State):
-    return try_number + 1 if state in [State.DEFERRED, State.UP_FOR_RESCHEDULE] else try_number
+    if state in (TaskInstanceState.DEFERRED, TaskInstanceState.UP_FOR_RESCHEDULE):
+        return try_number + 1
+    return try_number
 
 
 priority: list[None | TaskInstanceState] = [
@@ -118,6 +125,10 @@ def get_mapped_summary(parent_instance, task_instances):
             group_state = state
             break
 
+    group_queued_dttm = datetime_to_string(
+        min((ti.queued_dttm for ti in task_instances if ti.queued_dttm), default=None)
+    )
+
     group_start_date = datetime_to_string(
         min((ti.start_date for ti in task_instances if ti.start_date), default=None)
     )
@@ -129,10 +140,12 @@ def get_mapped_summary(parent_instance, task_instances):
         "task_id": parent_instance.task_id,
         "run_id": parent_instance.run_id,
         "state": group_state,
+        "queued_dttm": group_queued_dttm,
         "start_date": group_start_date,
         "end_date": group_end_date,
         "mapped_states": mapped_states,
         "try_number": get_try_count(parent_instance._try_number, parent_instance.state),
+        "execution_date": parent_instance.execution_date,
     }
 
 
@@ -179,14 +192,16 @@ def encode_dag_run(
 
 def check_import_errors(fileloc, session):
     # Check dag import errors
-    import_errors = session.query(errors.ImportError).filter(errors.ImportError.filename == fileloc).all()
+    import_errors = session.scalars(
+        select(errors.ImportError).where(errors.ImportError.filename == fileloc)
+    ).all()
     if import_errors:
         for import_error in import_errors:
-            flash("Broken DAG: [{ie.filename}] {ie.stacktrace}".format(ie=import_error), "dag_import_error")
+            flash(f"Broken DAG: [{import_error.filename}] {import_error.stacktrace}", "dag_import_error")
 
 
 def check_dag_warnings(dag_id, session):
-    dag_warnings = session.query(DagWarning).filter(DagWarning.dag_id == dag_id).all()
+    dag_warnings = session.scalars(select(DagWarning).where(DagWarning.dag_id == dag_id)).all()
     if dag_warnings:
         for dag_warning in dag_warnings:
             flash(dag_warning.message, "warning")
@@ -221,7 +236,7 @@ def should_hide_value_for_key(key_name):
 
 
 def get_params(**kwargs):
-    """Return URL-encoded params"""
+    """Return URL-encoded params."""
     return urlencode({d: v for d, v in kwargs.items() if v is not None}, True)
 
 
@@ -236,11 +251,12 @@ def generate_pages(
     sorting_direction=None,
 ):
     """
-    Generates the HTML for a paging component using a similar logic to the paging
-    auto-generated by Flask managed views. The paging component defines a number of
-    pages visible in the pager (window) and once the user goes to a page beyond the
-    largest visible, it would scroll to the right the page numbers and keeps the
-    current one in the middle of the pager component. When in the last pages,
+    Generates the HTML for a paging component.
+
+    Uses a similar logic to the paging auto-generated by Flask managed views. The paging
+    component defines a number of pages visible in the pager (window) and once the user
+    goes to a page beyond the largest visible, it would scroll to the right the page numbers
+    and keeps the current one in the middle of the pager component. When in the last pages,
     the pages won't scroll and just keep moving until the last page. Pager also contains
     <first, previous, ..., next, last> pages.
     This component takes into account custom parameters such as search, status, and tags
@@ -388,12 +404,12 @@ def generate_pages(
 
 
 def epoch(dttm):
-    """Returns an epoch-type date (tuple with no timezone)"""
+    """Returns an epoch-type date (tuple with no timezone)."""
     return (int(time.mktime(dttm.timetuple())) * 1000,)
 
 
 def make_cache_key(*args, **kwargs):
-    """Used by cache to get a unique key per URL"""
+    """Used by cache to get a unique key per URL."""
     path = request.path
     args = str(hash(frozenset(request.args.items())))
     return (path + args).encode("ascii", "ignore")
@@ -428,7 +444,7 @@ def task_instance_link(attr):
 
 
 def state_token(state):
-    """Returns a formatted string with HTML for a given State"""
+    """Returns a formatted string with HTML for a given State."""
     color = State.color(state)
     fg_color = State.color_fg(state)
     return Markup(
@@ -440,13 +456,13 @@ def state_token(state):
 
 
 def state_f(attr):
-    """Gets 'state' & returns a formatted string with HTML for a given State"""
+    """Gets 'state' & returns a formatted string with HTML for a given State."""
     state = attr.get("state")
     return state_token(state)
 
 
 def nobr_f(attr_name):
-    """Returns a formatted string with HTML with a Non-breaking Text element"""
+    """Returns a formatted string with HTML with a Non-breaking Text element."""
 
     def nobr(attr):
         f = attr.get(attr_name)
@@ -456,7 +472,7 @@ def nobr_f(attr_name):
 
 
 def datetime_f(attr_name):
-    """Returns a formatted string with HTML for given DataTime"""
+    """Returns a formatted string with HTML for given DataTime."""
 
     def dt(attr):
         f = attr.get(attr_name)
@@ -466,7 +482,7 @@ def datetime_f(attr_name):
 
 
 def datetime_html(dttm: DateTime | None) -> str:
-    """Return an HTML formatted string with time element to support timezone changes in UI"""
+    """Return an HTML formatted string with time element to support timezone changes in UI."""
     as_iso = dttm.isoformat() if dttm else ""
     if not as_iso:
         return Markup("")
@@ -478,7 +494,7 @@ def datetime_html(dttm: DateTime | None) -> str:
 
 
 def json_f(attr_name):
-    """Returns a formatted string with HTML for given JSON serializable"""
+    """Returns a formatted string with HTML for given JSON serializable."""
 
     def json_(attr):
         f = attr.get(attr_name)
@@ -519,18 +535,21 @@ def _get_run_ordering_expr(name: str) -> ColumnOperators:
     return expr.desc()
 
 
-def sorted_dag_runs(query: Query, *, ordering: Sequence[str], limit: int) -> Sequence[DagRun]:
+def sorted_dag_runs(
+    query: Select, *, ordering: Sequence[str], limit: int, session: Session
+) -> Sequence[DagRun]:
     """Produce DAG runs sorted by specified columns.
 
-    :param query: An ORM query object against *DagRun*.
+    :param query: An ORM select object against *DagRun*.
     :param ordering: Column names to sort the runs. should generally come from a
         timetable's ``run_ordering``.
     :param limit: Number of runs to limit to.
+    :param session: SQLAlchemy ORM session object
     :return: A list of DagRun objects ordered by the specified columns. The list
         contains only the *last* objects, but in *ascending* order.
     """
     ordering_exprs = (_get_run_ordering_expr(name) for name in ordering)
-    runs = query.order_by(*ordering_exprs, DagRun.id.desc()).limit(limit).all()
+    runs = session.scalars(query.order_by(*ordering_exprs, DagRun.id.desc()).limit(limit)).all()
     runs.reverse()
     return runs
 
@@ -544,7 +563,7 @@ def format_map_index(attr: dict) -> str:
 
 
 def pygment_html_render(s, lexer=lexers.TextLexer):
-    """Highlight text using a given Lexer"""
+    """Highlight text using a given Lexer."""
     return highlight(s, lexer(), HtmlFormatter(linenos=True))
 
 
@@ -580,7 +599,7 @@ def render(obj: Any, lexer: Lexer, handler: Callable[[Any], str] | None = None):
 
 
 def json_render(obj, lexer):
-    """Render a given Python object with json lexer"""
+    """Render a given Python object with json lexer."""
     out = ""
     if isinstance(obj, str):
         out = Markup(pygment_html_render(obj, lexer))
@@ -600,7 +619,7 @@ def wrapped_markdown(s, css_class="rich_doc"):
 
 
 def get_attr_renderer():
-    """Return Dictionary containing different Pygments Lexers for Rendering & Highlighting"""
+    """Return Dictionary containing different Pygments Lexers for Rendering & Highlighting."""
     return {
         "bash": lambda x: render(x, lexers.BashLexer),
         "bash_command": lambda x: render(x, lexers.BashLexer),
@@ -628,11 +647,12 @@ def get_attr_renderer():
 
 def get_chart_height(dag):
     """
-    We use the number of tasks in the DAG as a heuristic to
-    approximate the size of generated chart (otherwise the charts are tiny and unreadable
-    when DAGs have a large number of tasks). Ideally nvd3 should allow for dynamic-height
-    charts, that is charts that take up space based on the size of the components within.
-    TODO(aoen): See [AIRFLOW-1263]
+    Use the number of tasks in the DAG to approximate the size of generated chart.
+
+    Without this the charts are tiny and unreadable when DAGs have a large number of tasks).
+    Ideally nvd3 should allow for dynamic-height charts, that is charts that take up space
+    based on the size of the components within.
+    TODO(aoen): See [AIRFLOW-1263].
     """
     return 600 + len(dag.tasks) * 10
 
@@ -760,7 +780,7 @@ class AirflowFilterConverter(fab_sqlafilters.SQLAFilterConverter):
     def __init__(self, datamodel):
         super().__init__(datamodel)
 
-        for (method, filters) in self.conversion_table:
+        for method, filters in self.conversion_table:
             if FilterIsNull not in filters:
                 filters.append(FilterIsNull)
             if FilterIsNotNull not in filters:
@@ -769,10 +789,9 @@ class AirflowFilterConverter(fab_sqlafilters.SQLAFilterConverter):
 
 class CustomSQLAInterface(SQLAInterface):
     """
-    FAB does not know how to handle columns with leading underscores because
-    they are not supported by WTForm. This hack will remove the leading
-    '_' from the key to lookup the column names.
+    FAB does not know how to handle columns with leading underscores because they are not supported by WTForm.
 
+    This hack will remove the leading '_' from the key to lookup the column names.
     """
 
     def __init__(self, obj, session: Session | None = None):
@@ -791,8 +810,8 @@ class CustomSQLAInterface(SQLAInterface):
                 continue
             proxy_instance = getattr(self.obj, obj_attr)
             if hasattr(proxy_instance.remote_attr.prop, "columns"):
-                self.list_columns[desc.value_attr] = proxy_instance.remote_attr.prop.columns[0]
-                self.list_properties[desc.value_attr] = proxy_instance.remote_attr.prop
+                self.list_columns[obj_attr] = proxy_instance.remote_attr.prop.columns[0]
+                self.list_properties[obj_attr] = proxy_instance.remote_attr.prop
 
     def is_utcdatetime(self, col_name):
         """Check if the datetime is a UTC one."""
@@ -808,7 +827,7 @@ class CustomSQLAInterface(SQLAInterface):
         return False
 
     def is_extendedjson(self, col_name):
-        """Checks if it is a special extended JSON type"""
+        """Checks if it is a special extended JSON type."""
         from airflow.utils.sqlalchemy import ExtendedJSON
 
         if col_name in self.list_columns:
@@ -838,13 +857,18 @@ class DagRunCustomSQLAInterface(CustomSQLAInterface):
     """
 
     def delete(self, item: Model, raise_exception: bool = False) -> bool:
-        self.session.query(TaskInstance).where(TaskInstance.run_id == item.run_id).delete()
+        self.session.execute(delete(TI).where(TI.dag_id == item.dag_id, TI.run_id == item.run_id))
         return super().delete(item, raise_exception=raise_exception)
 
     def delete_all(self, items: list[Model]) -> bool:
-        self.session.query(TaskInstance).where(
-            TaskInstance.run_id.in_(item.run_id for item in items)
-        ).delete()
+        self.session.execute(
+            delete(TI).where(
+                tuple_in_condition(
+                    (TI.dag_id, TI.run_id),
+                    ((x.dag_id, x.run_id) for x in items),
+                )
+            )
+        )
         return super().delete_all(items)
 
 
@@ -858,7 +882,7 @@ FieldConverter.conversion_table = (
 
 class UIAlert:
     """
-    Helper for alerts messages shown on the UI
+    Helper for alerts messages shown on the UI.
 
     :param message: The message to display, either a string or Markup
     :param category: The category of the message, one of "info", "warning", "error", or any custom category.
@@ -920,6 +944,6 @@ class UIAlert:
                 # Unable to obtain user role - default to not showing
                 return False
 
-            if not user_roles.intersection(set(self.roles)):
+            if user_roles.isdisjoint(self.roles):
                 return False
         return True
